# Virtuous for Drupal 11

A comprehensive integration module between Drupal 11 and the Virtuous CRM fundraising platform.

## Overview

The Virtuous module provides seamless integration between Drupal 11 and the Virtuous CRM fundraising platform. It allows for automatic creation and updating of Drupal user accounts when contacts are created, updated, or deleted in Virtuous, with optional two-way synchronization.

## Requirements

- Drupal 11
- PHP 8.3
- Required modules:
  - drupal:user
  - drupal:field
  - drupal:taxonomy
  - drupal:telephone
  - drupal:datetime
  - drupal:views
  - key:key
  - field_group:field_group

## Installation

1. As you would any contributed module via Composer.
2. Install the module using Drush: `drush en virtuous`
4. **Important**: After installing, run database updates to install the Virtuous Contacts view:
   - Using Drush: `drush updb`
   - Or through the UI: Navigate to `/update.php` and follow the prompts

## Configuration

### API Configuration

1. Navigate to `/admin/config/virtuous`
2. Select API Mode (Test/Live)
3. Configure the Virtuous API Key (create a key through the Key module first)
4. Test the API connection

### Synchronization Settings

- Configure automatic sync frequency (Never, Daily, Weekly, Monthly)
- View last synchronization statistics
- Run manual sync operations

### Webhook Settings

- Enable webhooks for real-time updates
- Configure webhook security token
- Use provided webhook URLs in Virtuous:
  - Contact Created
  - Contact Updated

### Two-way Synchronization

- Optionally enable two-way synchronization to send Drupal user updates back to Virtuous

## Features

### Contact Synchronization

- Import contacts from Virtuous to Drupal user accounts
- Map Virtuous contact fields to Drupal user fields
- Process primary and secondary individual data
- Handle contact addresses

### Webhook Integration

- Real-time updates when contacts are modified in Virtuous
- Secure webhook endpoints with token validation
- Automated processing of contact created/updated events

### Two-way Sync

- Push changes from Drupal back to Virtuous
- Maintain data consistency between systems

### Administration

- Dedicated admin UI for configuration
- Synchronization status reporting
- Manual sync options for individual contacts

## Usage

### Manual Synchronization

1. Navigate to `/admin/config/virtuous/sync-now`
2. Options for syncing:
   - Enter a specific Virtuous Contact ID to sync an individual contact
   - Click "Sync All Contacts" to synchronize all contacts from Virtuous

### Individual Contact Synchronization

There are two ways to sync an individual contact:

1. From the sync page (`/admin/config/virtuous/sync-now`):
   - Enter the Virtuous Contact ID in the "Sync Individual Contact" field
   - Click "Sync This Contact Only"

2. From a user edit page:
   - For users linked to a Virtuous contact, a sync button appears at the top of the form
   - Click "Sync from Virtuous" to update that specific user from Virtuous

### User Integration

- Synchronized contacts appear as regular Drupal user accounts
- Contact details are displayed on user pages
- Changes to user profiles can be sent back to Virtuous (if two-way sync is enabled)

### Virtuous Contacts View

The module provides a "Virtuous Contacts" view that displays all synchronized contacts. To access it:

1. Navigate to `/admin/virtuous-contacts`
2. This view displays contact information including names, types, and addresses
3. You can filter, sort, and perform actions on contacts from this interface

## Data Handling Details

### Webhook Data Processing

The module handles webhook payloads from Virtuous with the following considerations:

- **Partial Data**: Webhook payloads may contain a subset of fields compared to full API responses. Some fields like `deceasedDate` might be missing from webhooks but present in API GET responses.
- **Processing Strategy**: The module processes whatever fields are available in the webhook payload, leaving missing fields as NULL.
- **Data Structure Differences**: The module handles both single address objects (from API GET) and address arrays (from webhooks).

### Date Format Handling

- The module creates a custom date format called "Virtuous Deceased Date" during installation to properly display dates in the format "04/22/2025".
- This ensures correct formatting for date fields in both form display and view display.

### Field Mapping

- Contact fields are mapped to taxonomy terms where appropriate (contact types, gender, marital status)
- Address fields handle both single address objects and address arrays
- Primary and secondary individuals have separate field sets
- Contact methods (email, phone) are processed with priority given to primary methods

### Data Synchronization Flow

1. When a webhook is received (contact created/updated):
   - The module extracts the Contact ID
   - It processes available data from the payload
   - Fields not included in the webhook payload remain NULL
   
2. During full or manual synchronization:
   - The module fetches complete contact data via the API
   - All fields, including those missing from webhooks, are populated

## Permissions

The module defines the following permissions:

- `administer virtuous configuration`: Allows users to configure the Virtuous CRM integration settings
- `access virtuous contacts`: Allows users to view the list of contacts synchronized from Virtuous
- `synchronize virtuous contacts`: Allows users to run manual synchronization of contacts from Virtuous CRM

## Troubleshooting

- Check the logs at `/admin/reports/dblog` for Virtuous-related errors
- Ensure your API key has proper permissions in Virtuous
- Verify webhook URLs are properly configured in Virtuous
- Test the API connection using the provided test button
- If deceased date fields aren't populating, they may be missing from webhook payloads; consider using the API GET request for complete data
- If the Virtuous Contacts view is missing, run database updates (`drush updb` or visit `/update.php`)

## Development

### Custom Hooks

The module provides the following hooks:

- `hook_virtuous_contact_synced`: Invoked after a contact has been synchronized

### Testing

The module includes comprehensive tests:
- Functional tests for configuration forms
- Kernel tests for API services
- Unit tests for data processing functions

## License

This project is licensed under GPL v2 or later.

## Credits

Developed for Drupal 11.

---

# Virtuous Personalization Submodule

## Overview

Virtuous Personalization is a Drupal 11 submodule for the Virtuous CRM module that enables dynamic content personalization based on the Virtuous Responsive Listener. This module allows your website to display personalized content to visitors who have been identified by Virtuous, such as greeting them by name or showing custom content based on their donor status.

## Features

- **Dynamic Content Personalization**: Display visitor names and custom greetings for identified Virtuous contacts
- **Conditional Content Display**: Show or hide content based on visitor identification status
- **Secure Configuration**: Uses Drupal's Key module to securely store the Virtuous Organization ID
- **Visitor Mapping**: Creates persistent mappings between Virtuous visitor IDs and Drupal user accounts
- **Easy Integration**: Automatically injects the required Virtuous Responsive Listener JavaScript
- **Performance Optimized**: Implements caching for visitor identification to reduce database queries
- **Privacy Focused**: Includes comprehensive privacy settings for GDPR and CCPA compliance
- **Detailed Statistics**: Provides administrators with detailed analytics on personalization effectiveness

## Requirements

- Drupal 11
- Virtuous CRM module (parent module)
- Key module (for secure credential storage)
- A valid Virtuous account with Responsive Listener access

## Installation

Install the module via Drush:

```bash
drush en virtuous_personalization
```

Or through the Drupal admin interface at `/admin/modules`

## Configuration

### Basic Setup

1. Navigate to `/admin/config/virtuous/personalization`
2. Check the "Enable personalization" checkbox
3. Create a Key in the Key module to store your Virtuous Organization ID:
   - Go to `/admin/config/system/keys/add`
   - Create a new key with the Key type set to "Authentication"
   - Store your Virtuous Organization ID (UUID format)
4. Return to the personalization settings and select your newly created key
5. Save the configuration

### Advanced Configuration

#### Performance Settings

- **Cache Lifetime**: Configure how long visitor identification data is cached (default: 1 hour)
- **Cleanup Batch Size**: Set the number of records to process in each batch during cleanup operations

#### Privacy Settings

- **Data Retention Period**: Configure how long to keep personalization statistics and mapping data
- **IP Recording**: Choose whether to record anonymized IP addresses with statistics events
- **Consent Requirements**: Enable if you need explicit visitor consent before activating personalization
- **Consent Cookie Name**: Specify which cookie indicates visitor consent

### Where to Find Your Organization ID

You can obtain your Organization ID by logging into your Virtuous account and visiting:
https://app.virtuoussoftware.com/Marketing/Home/ResponsiveListener

The Organization ID is a UUID in the format: `6936b0fa-2a91-47b3-8c5b-19e2c7ab02c5`

## Usage

### Adding Personalized Content to Your Site

Add HTML with special CSS classes to your templates or content:

```html
<!-- Content only shown to anonymous/unidentified visitors -->
<div class="virtuous-anonymous-only">
  <p>Join us today</p>
</div>

<!-- Content only shown to identified Virtuous contacts -->
<div class="virtuous-identified-only" style="display: none;">
  <p>Welcome back, <span class="virtuous-personalized-name">friend</span>!</p>
</div>

<!-- Automatic greeting (will display "Welcome back, [Name]!") -->
<div class="virtuous-personalized-greeting"></div>
```

### Available CSS Classes

- `virtuous-anonymous-only`: Content that only displays for unidentified visitors
- `virtuous-identified-only`: Content that only displays for identified Virtuous contacts
- `virtuous-personalized-name`: Will be replaced with the contact's name
- `virtuous-personalized-greeting`: Will display "Welcome back, [Name]!"

### Personalization Examples

#### Example 1: Personalized Donation Appeal

```html
<div class="virtuous-identified-only" style="display: none;">
  <h2><span class="virtuous-personalized-name">Friend</span>, your support makes a difference!</h2>
  <p>Thank you for your continued partnership with our mission.</p>
  <a href="/donate" class="button">Make another gift today</a>
</div>

<div class="virtuous-anonymous-only">
  <h2>Join our community of supporters!</h2>
  <p>Your gift today will help us reach our goals.</p>
  <a href="/donate" class="button">Make your first gift</a>
</div>
```

#### Example 2: Personalized Page Header

```html
<header class="page-header">
  <div class="virtuous-personalized-greeting"></div>
  <h1>Our Mission</h1>
</header>
```

#### Example 3: Personalized Content Block

```html
<div class="content-block">
  <p>Thank you for your interest in our work, <span class="virtuous-personalized-name">friend</span>!</p>
  
  <div class="virtuous-identified-only" style="display: none;">
    <p>As one of our valued supporters, you have access to the following resources:</p>
    <ul>
      <li><a href="/resources/supporters">Supporter Resources</a></li>
      <li><a href="/events/special">Special Event Invitations</a></li>
    </ul>
  </div>
</div>
```

## How It Works

1. The module injects the Virtuous Responsive Listener script on your site
2. When a visitor arrives, the script attempts to identify them
3. If identified, our JavaScript queries the Drupal backend to find a matching user
4. If a match is found, personalized content is displayed
5. The visitor-to-user mapping is stored for future visits

### Identification Methods

The module attempts to identify visitors in the following order:
1. Looking up the visitor ID in the mapping table
2. For logged-in users, associating their account with the visitor ID
3. Matching by primary_id or secondary_id fields
4. Matching by contact_id directly

### Performance Considerations

- Visitor identification results are cached to reduce database queries
- Cache lifetime is configurable in the module settings
- Data cleanup runs automatically via cron to prevent database bloat

## API

### JavaScript Events

The module triggers several JavaScript events that you can listen for in your custom code:

```javascript
// Listen for successful personalization
document.addEventListener('virtuousPersonalizationComplete', function(e) {
  console.log('Personalization complete: ', e.detail);
  // e.detail contains: success, visitorId, isIdentified, contactName
});

// Listen for DOM updates from personalization
document.addEventListener('virtuousPersonalizationDOMUpdated', function(e) {
  console.log('DOM updated with personalization: ', e.detail);
  // e.detail contains: isIdentified, name
});

// Listen for errors
document.addEventListener('virtuousPersonalizationError', function(e) {
  console.log('Personalization error: ', e.detail);
  // e.detail contains: code, message, critical, details
});
```

### PHP Hooks

You can implement these hooks in your custom module:

```php
/**
 * Alter visitor data before it's returned to JavaScript.
 *
 * @param array $data
 *   The visitor data array.
 * @param \Drupal\user\UserInterface $user
 *   The user entity.
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 */
function hook_virtuous_personalization_visitor_data_alter(array &$data, $user, $visitor_id) {
  // Add custom data to be used in JavaScript
  $data['myCustomField'] = $user->get('field_my_custom')->value;
}

/**
 * React to visitor identification events.
 *
 * @param array $data
 *   The visitor data array.
 * @param \Drupal\user\UserInterface $user
 *   The user entity.
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 */
function hook_virtuous_personalization_visitor_identified($data, $user, $visitor_id) {
  // Perform custom actions when a visitor is identified
  \Drupal::logger('my_module')->notice('Virtuous visitor identified: @id', [
    '@id' => $visitor_id,
  ]);
}
```

## Maintenance

### Statistics & Monitoring

The module provides a statistics page at `/admin/config/virtuous/personalization/statistics` with:

- Identification success rates
- Detailed event logs
- Filtering and sorting options

### Data Cleanup

Old data is automatically cleaned up based on your data retention settings:

1. Data cleanup runs during cron
2. Manual cleanup is available via the "Clean up old data" button on the statistics page
3. Batch processing ensures large data sets don't cause timeouts

## Troubleshooting

### Personalized Content Not Displaying

- Verify the module is enabled and properly configured
- Check that your Organization ID is correct
- Ensure the visitor has been identified by Virtuous (may require previous interaction)
- Check browser console for JavaScript errors
- Verify your visitor mapping approach (settings for primary/secondary individuals)
- Inspect your HTML to ensure the correct CSS classes are being used

### JavaScript Console Errors

- "Virtuous tracker not found": The Responsive Listener script failed to load or initialize
- "No visitor ID found": The visitor couldn't be identified via cookies or JavaScript
- "Cookie rejected": Browser privacy settings may be blocking necessary cookies
- "API failure": Check that your Drupal routes are accessible

### Privacy and Consent Issues

- If using the consent requirement feature, ensure your consent management solution sets the specified cookie
- Test with browser developer tools to verify cookie behavior
- Check server logs for any API errors related to personalization
