# Virtuous Personalization Developer Guide

This document provides detailed technical information for developers working with the Virtuous Personalization module.

## Architecture Overview

The Virtuous Personalization module follows a service-oriented architecture with clear separation of concerns:

```
┌─────────────────────┐     ┌───────────────────┐     ┌─────────────────┐
│                     │     │                   │     │                 │
│  JavaScript Layer   │◄────┤ Controller Layer  │◄────┤  Service Layer  │
│  (Browser Client)   │     │ (API Endpoints)   │     │  (Business      │
│                     │     │                   │     │   Logic)        │
└─────────────────────┘     └───────────────────┘     └─────────────────┘
         ▲                           ▲                        ▲
         │                           │                        │
         │                           │                        │
         │                           │                        │
         │                   ┌───────────────────┐            │
         └───────────────────┤                   │            │
                             │  Entity Layer     │◄───────────┘
                             │  (Drupal Users)   │
                             │                   │
                             └───────────────────┘
```

### Key Components

1. **Services**:
   - `VisitorManager`: Handles mapping between Virtuous visitor IDs and Drupal users
   - `Statistics`: Records and retrieves personalization statistics
   - `VirtuousPersonalizationService`: Manages data cleanup and maintenance

2. **Controllers**:
   - `PersonalizationController`: API endpoints for visitor identification and mapping
   - `StatisticsController`: Handles displaying statistics and logs
   - `MaintenanceController`: Provides data cleanup functionality

3. **JavaScript**:
   - `virtuous-personalization.js`: Frontend logic for visitor identification and DOM updates
   - `virtuous-clipboard.js`: Utility for copying HTML templates

4. **Forms**:
   - `PersonalizationSettingsForm`: Configuration form for personalization settings
   - `StatisticsFilterForm`: Filtering form for statistics data

## Database Schema

The module creates two database tables:

### virtuous_visitor_map

Maps Virtuous visitor IDs to Drupal users.

| Column       | Type            | Description                                         |
|--------------|-----------------|-----------------------------------------------------|
| visitor_id   | varchar(64)     | Primary key - The Virtuous visitor ID               |
| uid          | int (unsigned)  | The Drupal user ID                                  |
| individual_id| varchar(32)     | Which individual (primary/secondary) this refers to |
| created      | int             | Unix timestamp of creation                          |

### virtuous_personalization_stats

Stores statistics about personalization events.

| Column       | Type            | Description                                         |
|--------------|-----------------|-----------------------------------------------------|
| id           | serial          | Primary key - Unique statistics ID                  |
| event_type   | varchar(64)     | Type of event (identification, mapping, etc.)       |
| status       | varchar(32)     | Status of the event (success, failure, etc.)        |
| visitor_id   | varchar(64)     | The Virtuous visitor ID                             |
| uid          | int (unsigned)  | The Drupal user ID                                  |
| individual_id| varchar(32)     | Which individual this record refers to              |
| message      | varchar(255)    | Additional information about the event              |
| timestamp    | int             | Unix timestamp when the event occurred              |
| ip           | varchar(40)     | Anonymized IP address (last octet replaced with 0)  |

## API Reference

### PHP Services

#### VisitorManager

```php
/**
 * Gets a user by Virtuous visitor ID.
 *
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 *
 * @return array|null
 *   An array containing the user entity and individual type, or NULL if not found.
 */
public function getUserByVisitorId($visitor_id);

/**
 * Creates or updates a visitor mapping.
 *
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 * @param int $uid
 *   The Drupal user ID.
 * @param string $individual_id
 *   Which individual this visitor ID represents ('primary' or 'secondary').
 *
 * @return bool
 *   TRUE on success, FALSE on failure.
 */
public function saveVisitorMapping($visitor_id, $uid, $individual_id = NULL);
```

#### Statistics

```php
/**
 * Records a statistics event.
 *
 * @param string $event_type
 *   The type of event (identification, mapping, etc.).
 * @param string $status
 *   The status of the event (success, failure, etc.).
 * @param array $data
 *   Additional data to store with the event.
 */
public function recordEvent($event_type, $status, array $data = []);

/**
 * Gets identification statistics.
 *
 * @param string $period
 *   The time period to get statistics for (day, week, month, all).
 *
 * @return array
 *   An array of statistics.
 */
public function getIdentificationStats($period = 'all');

/**
 * Gets filtered event log entries.
 *
 * @param string $period
 *   The time period to get statistics for (day, week, month, all).
 * @param int $limit
 *   Maximum number of records to return.
 * @param array $filters
 *   Filters to apply to the query.
 * @param string $sort_field
 *   Field to sort by.
 * @param string $sort_order
 *   Sort order (ASC or DESC).
 *
 * @return array
 *   An array of log entries.
 */
public function getFilteredEventLog($period = 'all', $limit = 50, array $filters = [], $sort_field = 'id', $sort_order = 'DESC');

/**
 * Sanitizes personal data according to privacy settings.
 *
 * @param array $options
 *   Options for sanitization (uid or visitor_id).
 *
 * @return int
 *   The number of records affected.
 */
public function sanitizePersonalData(array $options);
```

#### VirtuousPersonalizationService

```php
/**
 * Cleans up old personalization data.
 *
 * @return array
 *   An array containing counts of deleted records.
 */
public function cleanupOldData();

/**
 * Counts old data that would be cleaned up.
 *
 * @param int $data_retention
 *   The data retention period in seconds.
 *
 * @return array
 *   Array with counts of old records in each table.
 */
public function countOldData($data_retention);

/**
 * Deletes old statistics records.
 *
 * @param int $limit
 *   Maximum number of records to delete in one batch.
 * @param int $data_retention
 *   The data retention period in seconds.
 *
 * @return int
 *   Number of records deleted.
 */
public function deleteOldStatistics($limit, $data_retention);

/**
 * Deletes old visitor mapping records.
 *
 * @param int $limit
 *   Maximum number of records to delete in one batch.
 * @param int $data_retention
 *   The data retention period in seconds.
 *
 * @return int
 *   Number of records deleted.
 */
public function deleteOldMappings($limit, $data_retention);
```

### JavaScript API

#### Drupal.virtuousPersonalization

```javascript
/**
 * Initializes the personalization process.
 */
Drupal.virtuousPersonalization.initPersonalization();

/**
 * Checks if consent is required for personalization.
 *
 * @return {boolean}
 *   TRUE if consent is required and not yet given.
 */
Drupal.virtuousPersonalization.requiresConsent();

/**
 * Shows an error message to the user.
 *
 * @param {string} errorCode
 *   The error code from errorCodes object.
 * @param {string} details
 *   Additional technical details about the error.
 */
Drupal.virtuousPersonalization.showError(errorCode, details);

/**
 * Format the visitor's name based on settings.
 *
 * @param {object} data
 *   Visitor data from the server.
 *
 * @return {string}
 *   Formatted name.
 */
Drupal.virtuousPersonalization.formatVisitorName(data);

/**
 * Update DOM elements with visitor data.
 *
 * @param {object} data
 *   Visitor data from the server.
 */
Drupal.virtuousPersonalization.updateDOM(data);

/**
 * Record visitor-to-user mapping.
 *
 * @param {string} visitorId
 *   The Virtuous visitor ID.
 */
Drupal.virtuousPersonalization.recordVisitorMapping(visitorId);

/**
 * Trigger a custom event.
 *
 * @param {string} eventName
 *   The name of the event to trigger.
 * @param {object} detail
 *   The data to pass with the event.
 */
Drupal.virtuousPersonalization.triggerEvent(eventName, detail);
```

## Module Hooks

### hook_virtuous_personalization_visitor_data_alter

```php
/**
 * Alter visitor data before it's returned to JavaScript.
 *
 * @param array $data
 *   The visitor data array.
 * @param \Drupal\user\UserInterface $user
 *   The user entity.
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 */
function hook_virtuous_personalization_visitor_data_alter(array &$data, $user, $visitor_id) {
  // Example: Add a custom field value to the visitor data
  if ($user->hasField('field_membership_level')) {
    $data['membershipLevel'] = $user->get('field_membership_level')->value;
  }
}
```

### hook_virtuous_personalization_visitor_identified

```php
/**
 * React to visitor identification events.
 *
 * @param array $data
 *   The visitor data array.
 * @param \Drupal\user\UserInterface $user
 *   The user entity.
 * @param string $visitor_id
 *   The Virtuous visitor ID.
 */
function hook_virtuous_personalization_visitor_identified($data, $user, $visitor_id) {
  // Example: Log to a custom activity log
  \Drupal::database()->insert('custom_activity_log')
    ->fields([
      'event' => 'visitor_identified',
      'uid' => $user->id(),
      'data' => json_encode([
        'visitor_id' => $visitor_id,
        'timestamp' => \Drupal::time()->getRequestTime(),
      ]),
    ])
    ->execute();
}
```

## Configuration Schema

The module defines the following configuration schema:

```yaml
virtuous_personalization.settings:
  type: config_object
  label: 'Virtuous Personalization settings'
  mapping:
    enabled:
      type: boolean
      label: 'Enable personalization'
    visitor_recording:
      type: boolean
      label: 'Record visitor mappings'
    organization_id_key:
      type: string
      label: 'Key ID for Virtuous Organization ID'
    name_display:
      type: string
      label: 'Name display format'
    greeting_text:
      type: string
      label: 'Greeting text template'
    data_retention:
      type: integer
      label: 'Data retention period in seconds'
    record_ip:
      type: boolean
      label: 'Record anonymized IP addresses'
    consent_required:
      type: boolean
      label: 'Require visitor consent'
    consent_cookie:
      type: string
      label: 'Name of the consent cookie'
    cache_lifetime:
      type: integer
      label: 'Cache lifetime in seconds'
    batch_size:
      type: integer
      label: 'Cleanup batch size'
```

## Class Diagram

Below is a simplified class diagram of the main module components:

```
+-------------------------+      +-------------------------+
| PersonalizationService  |----->| VisitorManager         |
+-------------------------+      +-------------------------+
| - cleanupOldData()      |      | - getUserByVisitorId() |
| - countOldData()        |      | - saveVisitorMapping() |
| - deleteOldStatistics() |      | - lookupUserById()     |
| - deleteOldMappings()   |      | - invalidateCache()    |
+-------------------------+      +-------------------------+
          |                                  |
          |                                  |
          v                                  v
+-------------------------+      +-------------------------+
| Statistics              |      | Cache                  |
+-------------------------+      +-------------------------+
| - recordEvent()         |      | - get()                |
| - getIdentificationStats|      | - set()                |
| - getFilteredEventLog() |      | - invalidate()         |
| - sanitizePersonalData()|      | - invalidateTags()     |
+-------------------------+      +-------------------------+
          |
          |
          v
+-------------------------+
| Database                |
+-------------------------+
| - virtuous_visitor_map  |
| - virtuous_stats        |
+-------------------------+
```

## Performance Considerations

### Caching Strategy

The module implements a multi-level caching strategy:

1. **Results Cache**: Visitor identification results are cached with configurable TTL
2. **Cache Tags**: Uses Drupal's cache tags system for proper invalidation
3. **Cache Contexts**: Response caching uses appropriate contexts to avoid leaking personal data

### Database Optimization

- Efficient indexes are created on both database tables
- Batch processing is used for data cleanup operations
- Statistics queries use efficient SQL and appropriate filtering

### JavaScript Performance

- Event delegation is used for DOM updates 
- Debouncing prevents excessive API calls
- Fails gracefully when errors occur

## Security Considerations

### Data Protection

1. **IP Anonymization**: IP addresses are anonymized by default (last octet replaced with 0)
2. **Data Retention**: Configurable retention periods for all personal data
3. **Sanitization API**: Method to sanitize personal data for GDPR compliance

### API Security

1. **CSRF Protection**: All POST endpoints use CSRF tokens
2. **Input Validation**: All input parameters are validated and sanitized
3. **Error Handling**: Errors are logged but not exposed to end users

## Testing

### Unit Tests

Unit tests are provided for core services:

- `tests/src/Unit/VisitorManagerTest.php`
- `tests/src/Unit/StatisticsTest.php`

### Functional Tests

Functional tests verify the module's integration with Drupal:

- `tests/src/Functional/PersonalizationTest.php`
- `tests/src/Functional/StatisticsPageTest.php`

### JavaScript Tests

JavaScript testing is done using Drupal's JavaScript testing framework:

- `tests/src/FunctionalJavascript/PersonalizationJsTest.php`

## Common Customizations

### Custom Visitor Identification Logic

To implement custom identification logic:

```php
/**
 * Implements hook_virtuous_personalization_visitor_data_alter().
 */
function mymodule_virtuous_personalization_visitor_data_alter(array &$data, $user, $visitor_id) {
  // Add additional identification logic here
  if ($user->hasField('field_custom_id')) {
    $custom_id = $user->get('field_custom_id')->value;
    // Add the custom ID to the returned data
    $data['customId'] = $custom_id;
  }
}
```

### Custom DOM Updates

To perform custom DOM updates when a visitor is identified:

```javascript
(function ($, Drupal) {
  'use strict';
  
  Drupal.behaviors.myCustomPersonalization = {
    attach: function (context, settings) {
      // Listen for personalization completion
      document.addEventListener('virtuousPersonalizationComplete', function(e) {
        if (e.detail.success && e.detail.isIdentified) {
          // Perform custom DOM updates
          $('.my-custom-element').text('Welcome, ' + e.detail.contactName);
          
          // Show/hide custom elements
          $('.show-for-identified').show();
          $('.hide-for-identified').hide();
        }
      });
    }
  };
})(jQuery, Drupal);
```

### Custom Statistics Reporting

To implement custom statistics reporting:

```php
/**
 * Implements hook_cron().
 */
function mymodule_cron() {
  // Get statistics service
  $statistics = \Drupal::service('virtuous_personalization.statistics');
  
  // Get last 30 days stats
  $monthly_stats = $statistics->getIdentificationStats('month');
  
  // Send email report if success rate is below threshold
  if ($monthly_stats['success_rate'] < 50) {
    $params = [
      'subject' => 'Low Personalization Success Rate Alert',
      'body' => 'The personalization success rate is currently ' . $monthly_stats['success_rate'] . '%.',
    ];
    \Drupal::service('plugin.manager.mail')->mail('mymodule', 'personalization_alert', 'admin@example.com', 'en', $params);
  }
}
```

## Resources

For more information on Virtuous integration, refer to:

- [Virtuous API Documentation](https://api.virtuoussoftware.com/Help)
- [Responsive Listener Guide](https://support.virtuoussoftware.com/hc/en-us/articles/360021620794-Responsive-Listener-Getting-Started)
- [Drupal Key Module](https://www.drupal.org/project/key)
- [Drupal Caching Best Practices](https://www.drupal.org/docs/drupal-apis/cache-api/cache-api)