/**
 * @file
 * JavaScript for Virtuous personalization.
 * 
 * This file handles the dynamic content personalization based on visitor 
 * identification from the Virtuous CRM system. It includes functionality for
 * detecting visitors, retrieving their information, and updating the page
 * content accordingly.
 */

(function ($, Drupal, drupalSettings, once) {
  'use strict';

  /**
   * Behavior for Virtuous personalization.
   * 
   * Initializes personalization on page load and handles the entire
   * identification and content update process.
   */
  Drupal.behaviors.virtuousPersonalization = {
    attach: function (context, settings) {
      // Only run once per page load
      once('virtuous-personalization', 'body', context).forEach(function () {
        // Wait for Virtuous tracker to load
        Drupal.virtuousPersonalization.waitForTracker(function () {
          // When tracker is available, check if the visitor is identified
          Drupal.virtuousPersonalization.checkVisitorIdentification();
        });
      });
    }
  };

  /**
   * Error messages for different failure scenarios.
   */
  const errorMessages = {
    trackerNotFound: 'The Virtuous visitor tracking system could not be loaded. Personalized content is not available.',
    noVisitorId: 'A visitor ID could not be determined. This may be due to cookie restrictions in your browser.',
    cookieRejected: 'The visitor tracking cookie was rejected by your browser. This may be due to privacy settings or cross-domain restrictions.',
    apiFailure: 'Could not retrieve personalization data from the server. Personalized content is unavailable at this time.',
    mappingFailure: 'Failed to store visitor connection information. Your personalization preferences may not persist between visits.'
  };

  /**
   * Virtuous personalization namespace.
   * Contains all the functionality for visitor identification and content personalization.
   */
  Drupal.virtuousPersonalization = {
    /**
     * Shows an error message to the user.
     * 
     * @param {string} errorType
     *   The type of error from the errorMessages object.
     * @param {string} details
     *   Additional technical details about the error (only shown in console).
     */
    showError: function(errorType, details) {
      // Log detailed error to console for administrators/developers
      console.error('Virtuous Personalization Error: ' + errorMessages[errorType], details);
      
      // Only show user-facing errors for critical failures that affect functionality
      if (['trackerNotFound', 'apiFailure'].includes(errorType)) {
        // Find or create an error container
        let $errorContainer = $('.virtuous-personalization-error');
        if (!$errorContainer.length) {
          // Create a visually subtle error container
          $errorContainer = $('<div class="virtuous-personalization-error" role="alert" aria-live="polite"></div>')
            .css({
              'background-color': '#f9f9f9',
              'border': '1px solid #ddd',
              'border-radius': '4px',
              'padding': '8px 12px',
              'margin': '10px 0',
              'color': '#666',
              'font-size': '0.9em'
            })
            .hide();
          
          // Insert it before the first personalization element if it exists
          // otherwise add it to the content area
          const $firstElement = $('.virtuous-personalized-greeting, .virtuous-personalized-name, .virtuous-identified-only, .virtuous-anonymous-only').first();
          if ($firstElement.length) {
            $firstElement.before($errorContainer);
          } else {
            $('main').prepend($errorContainer);
          }
        }
        
        // Show the error message
        $errorContainer.html('<p>' + errorMessages[errorType] + '</p>').fadeIn();
      }
    },

    /**
     * Wait for Virtuous tracker to be available with improved error handling.
     *
     * @param {function} callback
     *   Function to call when tracker is available.
     */
    waitForTracker: function (callback) {
      var attempts = 0;
      var maxAttempts = 10;
      var interval = 500;

      var checkTracker = function () {
        attempts++;
        
        // Check if Virtuous tracker is available
        if (typeof window.virtuousTracker !== 'undefined' || 
            typeof window.virtuousTrackerShim !== 'undefined') {
          callback();
          return;
        }
        
        // Give up after max attempts
        if (attempts >= maxAttempts) {
          Drupal.virtuousPersonalization.showError('trackerNotFound', 'No tracker found after ' + (maxAttempts * interval / 1000) + ' seconds.');
          return;
        }
        
        // Try again after delay
        setTimeout(checkTracker, interval);
      };
      
      // Start checking
      checkTracker();
    },
    
    /**
     * Check if the visitor is identified by Virtuous.
     * 
     * This function attempts to get the visitor ID using multiple methods,
     * then queries the Drupal backend for personalization data.
     */
    checkVisitorIdentification: function () {
      try {
        // Get visitor ID using different possible methods
        var visitorId = null;
        
        // Method 1: Try the getVisitorId function if it exists
        if (typeof window.virtuousTracker !== 'undefined' && 
            typeof window.virtuousTracker.getVisitorId === 'function') {
          visitorId = window.virtuousTracker.getVisitorId();
        } 
        // Method 2: Try to get from the shim if it exists
        else if (typeof window.virtuousTrackerShim !== 'undefined' && 
                 typeof window.virtuousTrackerShim.getVisitorId === 'function') {
          visitorId = window.virtuousTrackerShim.getVisitorId();
        }
        // Method 3: Try to get from secure cookie storage
        else {
          visitorId = Drupal.virtuousPersonalization.getSecureCookie('vcrmvid');
        }
        
        if (!visitorId) {
          // Check if there's a cookie rejection error
          if (document.cookie.indexOf('vcrmvid=') === -1 && 
              window.console && 
              window.console.error && 
              window.console.error.toString().indexOf('rejected for invalid domain') !== -1) {
            Drupal.virtuousPersonalization.showError('cookieRejected', 'Cookie domain validation failed');
          } else {
            Drupal.virtuousPersonalization.showError('noVisitorId', 'No visitor ID found in tracker or cookies');
          }
          return;
        }
        
        console.log('Using visitor ID: ' + visitorId);
        
        // Query Drupal backend to get visitor data
        $.ajax({
          url: '/virtuous/visitor/' + encodeURIComponent(visitorId),
          type: 'GET',
          dataType: 'json',
          success: function (data) {
            console.log('Visitor data received:', data);
            
            // Update DOM with visitor data
            Drupal.virtuousPersonalization.updateDOM(data);
            
            // Record the mapping if enabled
            if (drupalSettings.virtuousPersonalization && drupalSettings.virtuousPersonalization.visitorRecording) {
              Drupal.virtuousPersonalization.recordVisitorMapping(visitorId);
            }
          },
          error: function (xhr, status, error) {
            Drupal.virtuousPersonalization.showError('apiFailure', 'AJAX error: ' + error + ', Status: ' + xhr.status);
          }
        });
      } catch (e) {
        Drupal.virtuousPersonalization.showError('apiFailure', 'Exception: ' + e.message);
      }
    },
    
    /**
     * Get a cookie value by name with added security measures.
     *
     * @param {string} name
     *   The name of the cookie.
     *
     * @return {string|null}
     *   The cookie value, or null if not found.
     */
    getSecureCookie: function (name) {
      // Use a more secure approach to cookie handling
      try {
        const cookies = document.cookie.split(';');
        const cookieRegex = new RegExp(`^\\s*${name}=\\s*(.*?)\\s*$`);
        
        for (let i = 0; i < cookies.length; i++) {
          const match = cookies[i].match(cookieRegex);
          if (match) {
            // Decode the cookie value
            try {
              return decodeURIComponent(match[1]);
            } catch (e) {
              console.warn('Error decoding cookie value:', e);
              return match[1]; // Return raw value if decoding fails
            }
          }
        }
      } catch (e) {
        console.error('Error accessing cookies:', e);
      }
      return null;
    },
    
    /**
     * Format the visitor's name based on settings.
     *
     * @param {object} data
     *   Visitor data from the server.
     *
     * @return {string}
     *   Formatted name.
     */
    formatVisitorName: function (data) {
      // Default to the full name (which is what the server provides)
      var name = data.contactName || 'friend';
      
      // If name display setting exists and is set to 'first', extract the first name
      if (drupalSettings.virtuousPersonalization && 
          drupalSettings.virtuousPersonalization.nameDisplay === 'first') {
        // Extract first name (everything before the first space)
        var spaceIndex = name.indexOf(' ');
        if (spaceIndex > 0) {
          name = name.substring(0, spaceIndex);
        }
      }
      
      return name;
    },
    
    /**
     * Format the greeting text with the visitor's name.
     *
     * @param {string} name
     *   Visitor's name to include in greeting.
     *
     * @return {string}
     *   Formatted greeting.
     */
    formatGreeting: function (name) {
      var template = 'Welcome back, [NAME]!';
      
      // Use custom greeting template if provided
      if (drupalSettings.virtuousPersonalization && 
          drupalSettings.virtuousPersonalization.greetingText) {
        template = drupalSettings.virtuousPersonalization.greetingText;
      }
      
      // Replace [NAME] with the visitor's name
      return template.replace('[NAME]', name);
    },
    
    /**
     * Update DOM elements with visitor data.
     *
     * @param {object} data
     *   Visitor data from the server.
     */
    updateDOM: function (data) {
      if (!data.isIdentified) {
        // If not identified, hide elements that should only be shown to identified visitors
        $('.virtuous-identified-only').hide();
        $('.virtuous-anonymous-only').show();
        return;
      }
      
      // Get formatted name based on settings
      var formattedName = Drupal.virtuousPersonalization.formatVisitorName(data);
      
      // Visitor is identified
      // Update personalized name elements
      $('.virtuous-personalized-name').text(formattedName);
      
      // Update greeting elements
      var greeting = Drupal.virtuousPersonalization.formatGreeting(formattedName);
      $('.virtuous-personalized-greeting').text(greeting);
      
      // Show/hide elements based on identification status
      $('.virtuous-identified-only').show();
      $('.virtuous-anonymous-only').hide();
    },
    
    /**
     * Record visitor-to-user mapping.
     *
     * @param {string} visitorId
     *   The Virtuous visitor ID.
     */
    recordVisitorMapping: function (visitorId) {
      // Get the CSRF token for more secure AJAX requests
      const csrfToken = Drupal.virtuousPersonalization.getCsrfToken();
      
      $.ajax({
        url: '/virtuous/record-visitor/' + encodeURIComponent(visitorId),
        type: 'POST', // Changed from GET to POST for more security
        headers: {
          'X-CSRF-Token': csrfToken
        },
        dataType: 'json',
        success: function(response) {
          console.log('Visitor mapping recorded successfully');
        },
        error: function (xhr, status, error) {
          Drupal.virtuousPersonalization.showError('mappingFailure', 'Error recording mapping: ' + error);
        }
      });
    },
    
    /**
     * Get the CSRF token for protected AJAX requests.
     * 
     * @return {string}
     *   The CSRF token or an empty string if not available.
     */
    getCsrfToken: function() {
      // Try to get the token from drupalSettings
      if (drupalSettings.csrf_token) {
        return drupalSettings.csrf_token;
      }
      
      // Try to get from the meta tag
      const tokenMeta = document.querySelector('meta[name="csrf-token"]');
      if (tokenMeta) {
        return tokenMeta.getAttribute('content');
      }
      
      return '';
    }
  };

})(jQuery, Drupal, drupalSettings, once);
