<?php

namespace Drupal\virtuous_personalization\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\virtuous_personalization\Service\VirtuousPersonalizationService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Controller for maintenance tasks.
 */
class MaintenanceController extends ControllerBase {

  /**
   * The Virtuous personalization service.
   *
   * @var \Drupal\virtuous_personalization\Service\VirtuousPersonalizationService
   */
  protected $personalizationService;

  /**
   * Constructs a new MaintenanceController.
   *
   * @param \Drupal\virtuous_personalization\Service\VirtuousPersonalizationService $personalization_service
   *   The personalization service.
   */
  public function __construct(VirtuousPersonalizationService $personalization_service) {
    $this->personalizationService = $personalization_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('virtuous_personalization.personalization_service')
    );
  }

  /**
   * Cleans up old data using batch processing.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   A render array or a redirect response.
   */
  public function cleanupData() {
    // Get config settings for batch size
    $config = $this->config('virtuous_personalization.settings');
    $batch_size = $config->get('batch_size') ?: 100;
    $data_retention = $config->get('data_retention') ?: 7776000; // Default 90 days
    
    // Get estimate of records to process
    $stats_count = $this->personalizationService->countOldData($data_retention);
    
    // If there's nothing to process, show a message and return
    if ($stats_count['total'] == 0) {
      $this->messenger()->addStatus($this->t('No old data found to clean up.'));
      return new RedirectResponse(Url::fromRoute('virtuous_personalization.statistics')->toString());
    }
    
    // Build the batch
    $batch_builder = new BatchBuilder();
    $batch_builder
      ->setTitle($this->t('Cleaning up old personalization data'))
      ->setInitMessage($this->t('Initializing cleanup...'))
      ->setProgressMessage($this->t('Processed @current out of @total.'))
      ->setErrorMessage($this->t('An error occurred during cleanup.'))
      ->setFinishCallback(['\Drupal\virtuous_personalization\Controller\MaintenanceController', 'batchFinished']);
    
    // Calculate number of operations needed
    $stats_operations = ceil($stats_count['statistics'] / $batch_size);
    $mappings_operations = ceil($stats_count['mappings'] / $batch_size);
    
    // Add statistics cleanup operations
    for ($i = 0; $i < $stats_operations; $i++) {
      $batch_builder->addOperation(
        ['\Drupal\virtuous_personalization\Controller\MaintenanceController', 'batchCleanupStatistics'],
        [$batch_size, $data_retention]
      );
    }
    
    // Add mappings cleanup operations
    for ($i = 0; $i < $mappings_operations; $i++) {
      $batch_builder->addOperation(
        ['\Drupal\virtuous_personalization\Controller\MaintenanceController', 'batchCleanupMappings'],
        [$batch_size, $data_retention]
      );
    }
    
    // Set batch and redirect to the batch processing page
    batch_set($batch_builder->toArray());
    return batch_process(Url::fromRoute('virtuous_personalization.statistics'));
  }
  
  /**
   * Batch operation: Clean up old statistics records.
   *
   * @param int $batch_size
   *   Number of records to process in this batch operation.
   * @param int $data_retention
   *   The data retention period in seconds.
   * @param array $context
   *   The batch context array.
   */
  public static function batchCleanupStatistics($batch_size, $data_retention, &$context) {
    // Get the personalization service
    $personalization_service = \Drupal::service('virtuous_personalization.personalization_service');
    
    // Initialize batch results if not already set
    if (!isset($context['results']['statistics'])) {
      $context['results']['statistics'] = 0;
    }
    
    // Delete a batch of old statistics
    $deleted = $personalization_service->deleteOldStatistics($batch_size, $data_retention);
    
    // Update progress
    $context['results']['statistics'] += $deleted;
    $context['message'] = t('Deleted @count old statistics records', ['@count' => $deleted]);
    
    // Set finished status
    $context['finished'] = ($deleted < $batch_size) ? 1 : 0;
  }
  
  /**
   * Batch operation: Clean up old visitor mappings.
   *
   * @param int $batch_size
   *   Number of records to process in this batch operation.
   * @param int $data_retention
   *   The data retention period in seconds.
   * @param array $context
   *   The batch context array.
   */
  public static function batchCleanupMappings($batch_size, $data_retention, &$context) {
    // Get the personalization service
    $personalization_service = \Drupal::service('virtuous_personalization.personalization_service');
    
    // Initialize batch results if not already set
    if (!isset($context['results']['mappings'])) {
      $context['results']['mappings'] = 0;
    }
    
    // Delete a batch of old mappings
    $deleted = $personalization_service->deleteOldMappings($batch_size, $data_retention);
    
    // Update progress
    $context['results']['mappings'] += $deleted;
    $context['message'] = t('Deleted @count old visitor mappings', ['@count' => $deleted]);
    
    // Set finished status
    $context['finished'] = ($deleted < $batch_size) ? 1 : 0;
  }
  
  /**
   * Batch finished callback.
   *
   * @param bool $success
   *   Whether the batch completed successfully.
   * @param array $results
   *   The batch results.
   * @param array $operations
   *   The batch operations.
   */
  public static function batchFinished($success, $results, $operations) {
    $messenger = \Drupal::messenger();
    
    if ($success) {
      // Report success with statistics
      $stats_count = $results['statistics'] ?? 0;
      $mappings_count = $results['mappings'] ?? 0;
      $total_count = $stats_count + $mappings_count;
      
      $messenger->addStatus(t('Successfully cleaned up @total old records (@stats statistics and @mappings visitor mappings).', [
        '@total' => $total_count,
        '@stats' => $stats_count,
        '@mappings' => $mappings_count,
      ]));
      
      // Update the last cleanup time in state
      \Drupal::state()->set('virtuous_personalization.last_cleanup', \Drupal::time()->getRequestTime());
    }
    else {
      // Report error
      $messenger->addError(t('An error occurred while cleaning up old data.'));
    }
  }
}
  