<?php

namespace Drupal\virtuous_personalization\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\virtuous_personalization\Statistics;

/**
 * Controller for Virtuous personalization features.
 */
class PersonalizationController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * Constructs a new PersonalizationController.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\virtuous_personalization\Statistics $statistics
   *   The statistics service.
   */
  public function __construct(
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    AccountProxyInterface $current_user,
    Statistics $statistics
  ) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('virtuous_personalization');
    $this->currentUser = $current_user;
    $this->statistics = $statistics;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
      $container->get('current_user'),
      $container->get('virtuous_personalization.statistics')
    );
  }

  /**
   * Returns visitor data for personalization.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing personalization data.
   */
  public function getVisitorData($visitor_id) {
    $user = NULL;
    $individual_id = NULL;
    
    // Add debug log
    $this->logger->notice('Visitor data requested for ID: @id', [
      '@id' => $visitor_id,
    ]);
    
    try {
      // Record the start of an identification attempt
      $this->statistics->recordEvent('identification', 'attempt', [
        'visitor_id' => $visitor_id,
      ]);
      
      // 1. First check the mapping table
      $query = $this->database->select('virtuous_visitor_map', 'vvm')
        ->fields('vvm', ['uid', 'individual_id'])
        ->condition('vvm.visitor_id', $visitor_id)
        ->execute();
      
      $result = $query->fetchAssoc();
      
      if ($result && !empty($result['uid'])) {
        $user = $this->entityTypeManager->getStorage('user')->load($result['uid']);
        $individual_id = $result['individual_id'] ?? NULL;
        
        if ($user) {
          $this->logger->notice('User found via mapping table: @uid', [
            '@uid' => $user->id(),
          ]);
          
          $this->statistics->recordEvent('identification', 'success', [
            'visitor_id' => $visitor_id,
            'uid' => $user->id(),
            'individual_id' => $individual_id,
            'message' => 'Identified via mapping table',
          ]);
        }
      }
      
      // 2. If not found and user is logged in, check current user
      if (!$user && $this->currentUser->isAuthenticated()) {
        $current_user = $this->entityTypeManager->getStorage('user')->load($this->currentUser->id());
        // For logged-in users, we'll associate them with this visitor ID
        if ($current_user) {
          $user = $current_user;
          $individual_id = 'primary'; // Default to primary
          
          $this->logger->notice('User identified via current session: @uid', [
            '@uid' => $user->id(),
          ]);
          
          // Record this mapping for future use
          $this->recordVisitorMapping($visitor_id, $user->id(), $individual_id);
          
          $this->statistics->recordEvent('identification', 'success', [
            'visitor_id' => $visitor_id,
            'uid' => $user->id(),
            'individual_id' => $individual_id,
            'message' => 'Identified via current user session',
          ]);
        }
      }
      
      // 3. If still not found, try by contact_id or individual IDs
      if (!$user) {
        // Try to find by primary_id
        $primary_users = $this->entityTypeManager->getStorage('user')
          ->loadByProperties(['primary_id' => $visitor_id]);
        
        if ($primary_users) {
          $user = reset($primary_users);
          $individual_id = 'primary';
          
          $this->logger->notice('User identified via primary_id: @uid', [
            '@uid' => $user->id(),
          ]);
          
          // Record this mapping for future use
          $this->recordVisitorMapping($visitor_id, $user->id(), $individual_id);
          
          $this->statistics->recordEvent('identification', 'success', [
            'visitor_id' => $visitor_id,
            'uid' => $user->id(),
            'individual_id' => $individual_id,
            'message' => 'Identified via primary_id field',
          ]);
        }
        else {
          // Try to find by secondary_id
          $secondary_users = $this->entityTypeManager->getStorage('user')
            ->loadByProperties(['secondary_id' => $visitor_id]);
          
          if ($secondary_users) {
            $user = reset($secondary_users);
            $individual_id = 'secondary';
            
            $this->logger->notice('User identified via secondary_id: @uid', [
              '@uid' => $user->id(),
            ]);
            
            // Record this mapping for future use
            $this->recordVisitorMapping($visitor_id, $user->id(), $individual_id);
            
            $this->statistics->recordEvent('identification', 'success', [
              'visitor_id' => $visitor_id,
              'uid' => $user->id(),
              'individual_id' => $individual_id,
              'message' => 'Identified via secondary_id field',
            ]);
          }
          else {
            // Try by contact_id as a last resort
            $contact_users = $this->entityTypeManager->getStorage('user')
              ->loadByProperties(['contact_id' => $visitor_id]);
            
            $user = reset($contact_users);
            
            if ($user) {
              // Default to primary individual if we only have contact_id
              $individual_id = 'primary';
              
              $this->logger->notice('User identified via contact_id: @uid', [
                '@uid' => $user->id(),
              ]);
              
              // Record this mapping for future use
              $this->recordVisitorMapping($visitor_id, $user->id(), $individual_id);
              
              $this->statistics->recordEvent('identification', 'success', [
                'visitor_id' => $visitor_id,
                'uid' => $user->id(),
                'individual_id' => $individual_id,
                'message' => 'Identified via contact_id field',
              ]);
            }
          }
        }
      }
      
      // If we found a user, return their personalization data
      if ($user) {
        $data = [
          'isIdentified' => TRUE,
          'contactId' => $user->get('contact_id')->value ?? NULL,
        ];
        
        // Determine which individual to use for personalization
        if ($individual_id === 'primary' || !$individual_id) {
          // Use primary individual data
          $data['individualId'] = $user->get('primary_id')->value ?? NULL;
          
          // Determine name to use
          if (!$user->get('primary_first_name')->isEmpty() && !$user->get('primary_last_name')->isEmpty()) {
            $data['contactName'] = $user->get('primary_first_name')->value . ' ' . $user->get('primary_last_name')->value;
          }
          elseif (!$user->get('primary_first_name')->isEmpty()) {
            $data['contactName'] = $user->get('primary_first_name')->value;
          }
          else {
            $data['contactName'] = $user->getAccountName();
          }
        } 
        elseif ($individual_id === 'secondary') {
          // Use secondary individual data
          $data['individualId'] = $user->get('secondary_id')->value ?? NULL;
          
          // Determine name to use
          if (!$user->get('secondary_first_name')->isEmpty() && !$user->get('secondary_last_name')->isEmpty()) {
            $data['contactName'] = $user->get('secondary_first_name')->value . ' ' . $user->get('secondary_last_name')->value;
          }
          elseif (!$user->get('secondary_first_name')->isEmpty()) {
            $data['contactName'] = $user->get('secondary_first_name')->value;
          }
          else {
            $data['contactName'] = $user->getAccountName();
          }
        }
        
        $this->logger->notice('Returning personalization data for user @uid: @name', [
          '@uid' => $user->id(),
          '@name' => $data['contactName'] ?? 'unnamed',
        ]);
        
        return new JsonResponse($data);
      }
      else {
        // Record failed identification attempt
        $this->logger->notice('No user found for visitor ID: @id', [
          '@id' => $visitor_id,
        ]);
        
        $this->statistics->recordEvent('identification', 'failure', [
          'visitor_id' => $visitor_id,
          'message' => 'No matching user found',
        ]);
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Error retrieving visitor data: @error', [
        '@error' => $e->getMessage(),
      ]);
      
      // Record error in statistics
      $this->statistics->recordEvent('identification', 'error', [
        'visitor_id' => $visitor_id,
        'message' => 'Error: ' . $e->getMessage(),
      ]);
    }
    
    // No user found or error occurred
    return new JsonResponse(['isIdentified' => FALSE]);
  }

  /**
   * Records a mapping between Virtuous visitor ID and Drupal user ID.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   * @param int $uid
   *   The Drupal user ID.
   * @param string $individual_id
   *   Which individual this visitor ID represents ('primary' or 'secondary').
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response indicating success or failure.
   */
  public function recordVisitorMapping($visitor_id, $uid = NULL, $individual_id = NULL) {
    // Add debug log
    $this->logger->notice('Recording visitor mapping for ID: @id, UID: @uid', [
      '@id' => $visitor_id,
      '@uid' => $uid ?? 'from session',
    ]);
    
    if ($uid === NULL && $this->currentUser->isAuthenticated()) {
      $uid = $this->currentUser->id();
    }
    
    if (!$uid) {
      $this->logger->warning('Cannot record mapping: No user ID provided for visitor @id', [
        '@id' => $visitor_id,
      ]);
      
      $this->statistics->recordEvent('mapping', 'failure', [
        'visitor_id' => $visitor_id,
        'message' => 'No user ID provided',
      ]);
      
      return new JsonResponse(['success' => FALSE, 'message' => 'No user ID provided']);
    }
    
    try {
      // Check if mapping already exists
      $existing = $this->database->select('virtuous_visitor_map', 'vvm')
        ->fields('vvm', ['visitor_id'])
        ->condition('vvm.visitor_id', $visitor_id)
        ->execute()
        ->fetchField();
      
      if ($existing) {
        // Update existing mapping
        $fields = [
          'uid' => $uid,
          'created' => \Drupal::time()->getRequestTime(),
        ];
        
        // Add individual_id if provided
        if ($individual_id) {
          $fields['individual_id'] = $individual_id;
        }
        
        $this->database->update('virtuous_visitor_map')
          ->fields($fields)
          ->condition('visitor_id', $visitor_id)
          ->execute();
        
        $this->logger->notice('Updated existing visitor mapping: @id to @uid', [
          '@id' => $visitor_id, 
          '@uid' => $uid,
        ]);
        
        $this->statistics->recordEvent('mapping', 'update', [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'individual_id' => $individual_id,
        ]);
      }
      else {
        // Insert new mapping
        $fields = [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'created' => \Drupal::time()->getRequestTime(),
        ];
        
        // Add individual_id if provided
        if ($individual_id) {
          $fields['individual_id'] = $individual_id;
        }
        
        $this->database->insert('virtuous_visitor_map')
          ->fields($fields)
          ->execute();
        
        $this->logger->notice('Created new visitor mapping: @id to @uid', [
          '@id' => $visitor_id, 
          '@uid' => $uid,
        ]);
        
        $this->statistics->recordEvent('mapping', 'create', [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'individual_id' => $individual_id,
        ]);
      }
      
      $this->logger->notice('Recorded visitor mapping: visitor_id=@visitor_id, uid=@uid, individual=@individual', [
        '@visitor_id' => $visitor_id,
        '@uid' => $uid,
        '@individual' => $individual_id ?? 'unknown',
      ]);
      
      return new JsonResponse(['success' => TRUE]);
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to record visitor mapping: @error', [
        '@error' => $e->getMessage(),
      ]);
      
      $this->statistics->recordEvent('mapping', 'error', [
        'visitor_id' => $visitor_id,
        'uid' => $uid,
        'individual_id' => $individual_id,
        'message' => 'Error: ' . $e->getMessage(),
      ]);
      
      return new JsonResponse(['success' => FALSE, 'message' => 'Database error']);
    }
  }
}
