<?php

namespace Drupal\virtuous_personalization\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Url;
use Drupal\virtuous_personalization\Form\StatisticsFilterForm;
use Drupal\virtuous_personalization\Statistics;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for statistics pages.
 */
class StatisticsController extends ControllerBase {

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs a new StatisticsController.
   *
   * @param \Drupal\virtuous_personalization\Statistics $statistics
   *   The statistics service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(
    Statistics $statistics,
    DateFormatterInterface $date_formatter,
    FormBuilderInterface $form_builder
  ) {
    $this->statistics = $statistics;
    $this->dateFormatter = $date_formatter;
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('virtuous_personalization.statistics'),
      $container->get('date.formatter'),
      $container->get('form_builder')
    );
  }

  /**
   * Displays an overview of statistics.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array
   *   A render array for the overview page.
   */
  public function overview(Request $request) {
    // Get statistics for different time periods
    $daily_stats = $this->statistics->getIdentificationStats('day');
    $weekly_stats = $this->statistics->getIdentificationStats('week');
    $monthly_stats = $this->statistics->getIdentificationStats('month');
    $all_time_stats = $this->statistics->getIdentificationStats('all');
    
    // Build the filter form
    $filter_form = $this->formBuilder->getForm(StatisticsFilterForm::class);
    
    // Get filters from the request
    $filters = [];
    $valid_filters = [
      'event_type', 'status', 'individual_type', 'first_name', 
      'last_name', 'message',
    ];
    
    foreach ($valid_filters as $filter) {
      if ($request->query->has($filter) && !empty($request->query->get($filter))) {
        $filters[$filter] = $request->query->get($filter);
      }
    }
    
    // Get sort parameters
    $sort_field = $request->query->get('sort') ?: 'id';
    $sort_order = $request->query->get('order') ?: 'DESC';
    
    // Get query builder for event log with filters and sorting
    $query = $this->statistics->getFilteredEventQueryBuilder(
      'all', 
      $filters,
      $sort_field,
      $sort_order
    );
    
    // Add the pager
    $pager = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')
      ->limit(20); // Show 20 items per page
    
    $log_entries = $pager->execute()->fetchAll();
    
    $formatted_log = [];
    
    foreach ($log_entries as $entry) {
      // Get additional user data if a user ID exists
      $contact_id = NULL;
      $individual_id_value = NULL;
      $first_name = NULL;
      $last_name = NULL;
      
      if (!empty($entry->uid)) {
        $user = $this->entityTypeManager()->getStorage('user')->load($entry->uid);
        if ($user) {
          // Load fields based on individual type
          $individual_type = $entry->individual_id ?? 'primary';
          $field_prefix = ($individual_type === 'secondary') ? 'secondary_' : 'primary_';
          
          $contact_id = $user->get('contact_id')->value ?? NULL;
          $individual_id_value = $user->get($field_prefix . 'id')->value ?? NULL;
          $first_name = $user->get($field_prefix . 'first_name')->value ?? NULL;
          $last_name = $user->get($field_prefix . 'last_name')->value ?? NULL;
        }
      }
      
      // Generate sorting URLs
      $sort_links = [];
      $sortable_fields = ['id', 'uid', 'contact_id', 'individual_id', 'timestamp'];
      
      foreach ($sortable_fields as $field) {
        $current_order = ($sort_field == $field) ? $sort_order : 'ASC';
        $new_order = ($current_order == 'ASC') ? 'DESC' : 'ASC';
        
        $sort_link_url = Url::fromRoute('virtuous_personalization.statistics', [], [
          'query' => array_merge(
            $request->query->all(),
            ['sort' => $field, 'order' => $new_order]
          ),
        ]);
        
        $sort_links[$field] = [
          'url' => $sort_link_url,
          'current' => ($sort_field == $field),
          'order' => $current_order,
        ];
      }
      
      $formatted_log[] = [
        'id' => $entry->id,
        'event_type' => $entry->event_type,
        'status' => $entry->status,
        'visitor_id' => $entry->visitor_id,
        'uid' => $entry->uid,
        'contact_id' => $contact_id,
        'individual_type' => $entry->individual_id,
        'individual_id' => $individual_id_value,
        'first_name' => $first_name,
        'last_name' => $last_name,
        'message' => $entry->message,
        'timestamp' => $this->dateFormatter->format($entry->timestamp, 'short'),
        'ip' => $entry->ip,
      ];
    }
    
    // Build the output
    $build = [];
    $build['filter_form'] = $filter_form;
    $build['statistics'] = [
      '#theme' => 'virtuous_personalization_statistics',
      '#attached' => [
        'library' => ['virtuous_personalization/statistics'],
      ],
      '#stats' => [
        'daily' => $daily_stats,
        'weekly' => $weekly_stats,
        'monthly' => $monthly_stats,
        'all_time' => $all_time_stats,
      ],
      '#log_entries' => $formatted_log,
      '#sort_links' => $sort_links ?? [],
      '#filters' => $filters,
    ];
    
    // Add the pager as a separate render element
    $build['pager'] = [
      '#type' => 'pager',
    ];
    
    return $build;
  }
}
