<?php

namespace Drupal\virtuous_personalization\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\key\KeyRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Virtuous Personalization settings.
 */
class PersonalizationSettingsForm extends ConfigFormBase {

  /**
   * The key repository service.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * Constructs a PersonalizationSettingsForm object.
   *
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository.
   */
  public function __construct(KeyRepositoryInterface $key_repository) {
    $this->keyRepository = $key_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('key.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['virtuous_personalization.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'virtuous_personalization_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('virtuous_personalization.settings');

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable personalization'),
      '#description' => $this->t('When enabled, the site will display personalized content for visitors identified by the Virtuous Responsive Listener.'),
      '#default_value' => $config->get('enabled') ?: FALSE,
    ];
    
    $form['responsive_listener'] = [
      '#type' => 'details',
      '#title' => $this->t('Responsive Listener Configuration'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['responsive_listener']['info'] = [
      '#markup' => '<p>' . $this->t('Only the Organization ID is required to configure the Responsive Listener.') . '</p>' .
        '<p>' . $this->t('You can obtain your Organization ID by logging into your Virtuous account and visiting:') . ' <a href="https://app.virtuoussoftware.com/Marketing/Home/ResponsiveListener" target="_blank">https://app.virtuoussoftware.com/Marketing/Home/ResponsiveListener</a></p>',
    ];
    
    // Get all available API keys.
    $key_options = [];
    $keys = $this->keyRepository->getKeys();
    foreach ($keys as $key) {
      $key_options[$key->id()] = $key->label();
    }

    $form['responsive_listener']['organization_id_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Organization ID Key'),
      '#description' => $this->t('Select a key that contains your Virtuous Organization ID (a UUID like 6936b0fa-2a91-47b3-8c5b-19e2c7ab02c5). If you need to create a new key, <a href=":url">click here</a>.', [
        ':url' => Url::fromRoute('entity.key.add_form')->toString(),
      ]),
      '#options' => $key_options,
      '#default_value' => $config->get('organization_id_key'),
      '#required' => FALSE,
      '#empty_option' => $this->t('- Select a key -'),
    ];

    // Only show the script preview if a key is selected.
    $selected_key_id = $config->get('organization_id_key');
    $organization_id = '';
    if ($selected_key_id) {
      $key = $this->keyRepository->getKey($selected_key_id);
      if ($key) {
        $organization_id = $key->getKeyValue();
      }
    }

    $form['responsive_listener']['script_preview'] = [
      '#type' => 'markup',
      '#markup' => '<div id="script-preview" class="form-item">' .
        '<label>' . $this->t('Responsive Listener Script (Preview)') . '</label>' .
        '<div class="description">' . $this->t('This script will be automatically added to your site after saving this form.') . '</div>' .
        '<pre><code>' . 
        htmlspecialchars('<script>
  (function (config) { 
    var s = document.createElement(\'script\'); 
    s.src = \'https://cdn.virtuoussoftware.com/tracker/virtuous.tracker.shim.min.js\';
    s.type = \'text/javascript\'; 
    s.onload = function () { virtuousTrackerShim.init(config); };
    document.getElementsByTagName(\'script\')[0].parentNode.appendChild(s);
  }({
    organizationId: \'' . ($organization_id ?: 'your-organization-id') . '\'
  }));
</script>') . 
        '</code></pre>' .
        '</div>',
      '#states' => [
        'visible' => [
          ':input[name="organization_id_key"]' => ['!value' => ''],
        ],
      ],
    ];
    
    $form['visitor_recording'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Record visitor mappings'),
      '#description' => $this->t('When enabled, the system will create and maintain mappings between Virtuous visitor IDs and Drupal user accounts.'),
      '#default_value' => $config->get('visitor_recording') ?: TRUE,
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];
    
    // Name display format
    $form['name_display'] = [
      '#type' => 'radios',
      '#title' => $this->t('Name Display Format'),
      '#options' => [
        'first' => $this->t('First name only'),
        'full' => $this->t('First and last name'),
      ],
      '#default_value' => $config->get('name_display') ?: 'full',
      '#description' => $this->t('Select how visitor names should be displayed in personalized content.'),
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];
    
    // Add privacy settings section
    $form['privacy_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Privacy Settings'),
      '#open' => TRUE,
      '#description' => $this->t('Configure privacy-related settings for personalization data.'),
    ];
    
    $form['privacy_settings']['data_retention'] = [
      '#type' => 'select',
      '#title' => $this->t('Data Retention Period'),
      '#description' => $this->t('How long to keep personalization statistics data before automatic cleanup.'),
      '#options' => [
        '2592000' => $this->t('30 days'),
        '7776000' => $this->t('90 days'),
        '15552000' => $this->t('180 days'),
        '31536000' => $this->t('1 year'),
        '0' => $this->t('Keep indefinitely'),
      ],
      '#default_value' => $config->get('data_retention') ?: '7776000',
    ];
    
    $form['privacy_settings']['record_ip'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Record anonymized IP addresses'),
      '#description' => $this->t('When enabled, the system will record anonymized IP addresses (last octet removed) with statistics events. This can help with troubleshooting but may not be necessary for all sites.'),
      '#default_value' => $config->get('record_ip') ?? TRUE,
    ];
    
    $form['privacy_settings']['consent_required'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Require visitor consent'),
      '#description' => $this->t('When enabled, personalization will only be activated after the visitor has given explicit consent (e.g., via a cookie consent module).'),
      '#default_value' => $config->get('consent_required') ?: FALSE,
    ];
    
    $form['privacy_settings']['consent_cookie'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Consent Cookie Name'),
      '#description' => $this->t('The name of the cookie that indicates visitor consent has been given. Leave empty to use default Drupal behavior.'),
      '#default_value' => $config->get('consent_cookie') ?: 'drupal_visitor_consent',
      '#states' => [
        'visible' => [
          ':input[name="consent_required"]' => ['checked' => TRUE],
        ],
      ],
    ];
    
    // Add performance settings section
    $form['performance'] = [
      '#type' => 'details',
      '#title' => $this->t('Performance Settings'),
      '#open' => TRUE,
      '#description' => $this->t('Configure settings that affect module performance.'),
    ];
    
    $form['performance']['cache_lifetime'] = [
      '#type' => 'select',
      '#title' => $this->t('Cache Lifetime'),
      '#description' => $this->t('How long to cache visitor identification data.'),
      '#options' => [
        '300' => $this->t('5 minutes'),
        '900' => $this->t('15 minutes'),
        '1800' => $this->t('30 minutes'),
        '3600' => $this->t('1 hour'),
        '7200' => $this->t('2 hours'),
        '14400' => $this->t('4 hours'),
        '86400' => $this->t('1 day'),
      ],
      '#default_value' => $config->get('cache_lifetime') ?: '3600',
    ];
    
    $form['performance']['batch_size'] = [
      '#type' => 'number',
      '#title' => $this->t('Cleanup Batch Size'),
      '#description' => $this->t('Number of records to process in each batch during cleanup operations.'),
      '#min' => 10,
      '#max' => 1000,
      '#step' => 10,
      '#default_value' => $config->get('batch_size') ?: 100,
    ];
    
    // Personalization Options section
    $form['personalization_options'] = [
      '#type' => 'details',
      '#title' => $this->t('Personalization Options'),
      '#open' => TRUE,
    ];

    // Detailed feature descriptions
    $options_content = '<div class="personalization-option">';
    $options_content .= '<h3>' . $this->t('Display content only for anonymous visitors') . '</h3>';
    $options_content .= '<p>' . $this->t('Content within this div is displayed exclusively to visitors who haven\'t been identified by the Virtuous CRM system. When a visitor is identified, this content automatically disappears. Use this for call-to-action messages, signup forms, or introductory content intended for new visitors who don\'t have a relationship with your organization yet.') . '</p>';
    
    $options_content .= '<ul>';
    $options_content .= '<li><strong>' . $this->t('Scope and purpose:') . '</strong> ' . $this->t('Controls visibility of content for non-identified visitors only.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Implementation:') . '</strong> ' . $this->t('Uses &lt;div class="virtuous-anonymous-only"&gt; to show or hide content.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Functionality:') . '</strong> ' . $this->t('Content is visible by default and automatically hidden when a visitor is identified.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Use cases:') . '</strong> ' . $this->t('Sign-up forms, donation calls-to-action, introductory content, and first-time visitor messaging.') . '</li>';
    $options_content .= '</ul>';
    $options_content .= '</div>';
    
    $options_content .= '<div class="personalization-option">';
    $options_content .= '<h3>' . $this->t('Display content only for identified visitors') . '</h3>';
    $options_content .= '<p>' . $this->t('This container is initially hidden for all visitors (note the style="display: none;" attribute). When a visitor is successfully identified as a contact in your Virtuous CRM, this content automatically becomes visible. Use this for personalized messaging, donor-specific information, or content meant only for individuals with an existing relationship with your organization. The CSS class handles the visibility switching automatically based on visitor identification status.') . '</p>';
    
    $options_content .= '<ul>';
    $options_content .= '<li><strong>' . $this->t('Scope and purpose:') . '</strong> ' . $this->t('Controls visibility of content for identified visitors only.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Implementation:') . '</strong> ' . $this->t('Uses &lt;div class="virtuous-identified-only" style="display: none;"&gt; to show or hide content.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Functionality:') . '</strong> ' . $this->t('Content is hidden by default and automatically displayed when a visitor is identified.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Use cases:') . '</strong> ' . $this->t('Personalized welcome messages, donor-specific content, member resources, and returning visitor information.') . '</li>';
    $options_content .= '</ul>';
    $options_content .= '</div>';
    
    $options_content .= '<div class="personalization-option">';
    $options_content .= '<h3>' . $this->t('Display Visitor\'s Name') . '</h3>';
    $options_content .= '<p>' . $this->t('The word "friend" is a placeholder that will be dynamically replaced with the visitor\'s name when they are identified through Virtuous CRM. This personalization works anywhere on your site - inside paragraphs, headings, or other content areas. You can place this element inside either regular content (which everyone will see with "friend" if not identified) or inside identified-only content blocks. The name display format (first name or full name) is controlled by the "Name Display Format" setting.') . '</p>';
    $options_content .= '<p><strong>' . $this->t('Note:') . '</strong> ' . $this->t('Everyone will see "friend" if not identified.') . '</p>';
    
    $options_content .= '<ul>';
    $options_content .= '<li><strong>' . $this->t('Scope and purpose:') . '</strong> ' . $this->t('Personalizes content with the visitor\'s actual name.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Implementation:') . '</strong> ' . $this->t('Uses &lt;span class="virtuous-personalized-name"&gt;friend&lt;/span&gt; to replace placeholder text.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Functionality:') . '</strong> ' . $this->t('Replaces the word "friend" with the visitor\'s name when identified.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Use cases:') . '</strong> ' . $this->t('Personal greetings, customized headings, tailored message content, and individualized calls-to-action.') . '</li>';
    $options_content .= '</ul>';
    $options_content .= '</div>';
    
    $options_content .= '<div class="personalization-option">';
    $options_content .= '<h3>' . $this->t('Display Automatic Greeting') . '</h3>';
    $options_content .= '<p>' . $this->t('This empty div will be automatically populated with a complete greeting message (e.g., "Welcome back, John!") when a visitor is identified. Unlike the personalized name span, you don\'t need to provide any content inside this element - the greeting is generated automatically, using the visitor\'s name from Virtuous CRM. The name display format (first name or full name) is controlled by the "Name Display Format" setting. If the visitor is not identified, this element remains empty and takes up no space on the page.') . '</p>';
    $options_content .= '<p><strong>' . $this->t('Note:') . '</strong> ' . $this->t('You can customize the greeting text format in the "Automatic Greeting Template" setting.') . '</p>';
    
    $options_content .= '<ul>';
    $options_content .= '<li><strong>' . $this->t('Scope and purpose:') . '</strong> ' . $this->t('Provides a complete greeting message with minimal markup.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Implementation:') . '</strong> ' . $this->t('Uses &lt;div class="virtuous-personalized-greeting"&gt;&lt;/div&gt; empty element.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Functionality:') . '</strong> ' . $this->t('Automatically populates with a complete greeting including the visitor\'s name; remains empty if not identified.') . '</li>';
    $options_content .= '<li><strong>' . $this->t('Use cases:') . '</strong> ' . $this->t('Page headers, site banners, welcome sections, and areas where you want a greeting to appear only for identified visitors.') . '</li>';
    $options_content .= '</ul>';
    $options_content .= '</div>';

    $form['personalization_options']['markup'] = [
      '#markup' => $options_content,
    ];
    
    // HTML Templates section
    $form['html_templates'] = [
      '#type' => 'details',
      '#title' => $this->t('HTML Templates'),
      '#open' => TRUE,
    ];

    // Example 1: Anonymous-only content
    $form['html_templates']['anonymous_only_title'] = [
      '#markup' => '<h3>' . $this->t('Display content only for anonymous visitors') . '</h3>',
    ];
    
    $form['html_templates']['anonymous_only'] = [
      '#type' => 'textarea',
      '#value' => '<div class="virtuous-anonymous-only">
  <p>Join us today</p>
</div>',
      '#attributes' => [
        'readonly' => 'readonly',
        'onclick' => 'this.select();',
        'class' => ['virtuous-clipboard-example'],
      ],
      '#rows' => 3,
    ];

    // Example 2: Identified-only content
    $form['html_templates']['identified_only_title'] = [
      '#markup' => '<h3>' . $this->t('Display content only for identified visitors') . '</h3>',
    ];
    
    $form['html_templates']['identified_only'] = [
      '#type' => 'textarea',
      '#value' => '<div class="virtuous-identified-only" style="display: none;">
  <p>Welcome back, <span class="virtuous-personalized-name">friend</span>!</p>
</div>',
      '#attributes' => [
        'readonly' => 'readonly',
        'onclick' => 'this.select();',
        'class' => ['virtuous-clipboard-example'],
      ],
      '#rows' => 3,
    ];

    // Example 3: Personalized name
    $form['html_templates']['personalized_name_title'] = [
      '#markup' => '<h3>' . $this->t('Display Visitor\'s Name') . '</h3>',
    ];
    
    $form['html_templates']['personalized_name'] = [
      '#type' => 'textarea',
      '#value' => '<span class="virtuous-personalized-name">friend</span>',
      '#attributes' => [
        'readonly' => 'readonly',
        'onclick' => 'this.select();',
        'class' => ['virtuous-clipboard-example'],
      ],
      '#rows' => 1,
    ];

    // Example 4: Personalized greeting
    $form['html_templates']['personalized_greeting_title'] = [
      '#markup' => '<h3>' . $this->t('Display Automatic Greeting') . '</h3>',
    ];
    
    $form['html_templates']['personalized_greeting'] = [
      '#type' => 'textarea',
      '#value' => '<div class="virtuous-personalized-greeting"></div>',
      '#attributes' => [
        'readonly' => 'readonly',
        'onclick' => 'this.select();',
        'class' => ['virtuous-clipboard-example'],
      ],
      '#rows' => 1,
    ];
    
    // Add greeting text configuration (moved here from above)
    $form['greeting_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Automatic Greeting Template'),
      '#description' => $this->t('Customize the greeting text used for automatic greetings. Use [NAME] as a placeholder for the visitor\'s name.'),
      '#default_value' => $config->get('greeting_text') ?: 'Welcome back, [NAME]!',
      '#states' => [
        'visible' => [
          ':input[name="enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Add clipboard.js functionality
    $form['html_templates']['clipboard_js'] = [
      '#attached' => [
        'library' => ['virtuous_personalization/clipboard'],
        'drupalSettings' => [
          'virtuousPersonalization' => [
            'clipboardSelector' => '.virtuous-clipboard-example',
          ],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('virtuous_personalization.settings')
      ->set('enabled', $form_state->getValue('enabled'))
      ->set('visitor_recording', $form_state->getValue('visitor_recording'))
      ->set('name_display', $form_state->getValue('name_display'))
      ->set('organization_id_key', $form_state->getValue('organization_id_key'))
      ->set('greeting_text', $form_state->getValue('greeting_text'))
      ->set('data_retention', $form_state->getValue('data_retention'))
      ->set('record_ip', $form_state->getValue('record_ip'))
      ->set('consent_required', $form_state->getValue('consent_required'))
      ->set('consent_cookie', $form_state->getValue('consent_cookie'))
      ->set('cache_lifetime', $form_state->getValue('cache_lifetime'))
      ->set('batch_size', $form_state->getValue('batch_size'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
