<?php

namespace Drupal\virtuous_personalization\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for filtering statistics.
 */
class StatisticsFilterForm extends FormBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new StatisticsFilterForm object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'virtuous_personalization_statistics_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Get filter options from database for select lists.
    $event_types = $this->getDistinctValues('event_type');
    $statuses = $this->getDistinctValues('status');
    $individual_types = $this->getDistinctValues('individual_id');
    
    // Define message options
    $message_options = [
      '' => $this->t('- All -'),
      'Identified via mapping table' => $this->t('Identified via mapping table'),
      'Identified via current user session' => $this->t('Identified via current user session'),
      'Identified via primary_id field' => $this->t('Identified via primary_id field'),
      'Identified via secondary_id field' => $this->t('Identified via secondary_id field'),
      'Identified via contact_id field' => $this->t('Identified via contact_id field'),
      'No matching user found' => $this->t('No matching user found'),
      'No user ID provided' => $this->t('No user ID provided'),
    ];
    
    // Get all messages from the database to ensure we include all possibilities
    $db_messages = $this->database->select('virtuous_personalization_stats', 's')
      ->fields('s', ['message'])
      ->distinct()
      ->execute()
      ->fetchCol();
    
    foreach ($db_messages as $message) {
      if (!empty($message) && !isset($message_options[$message])) {
        $message_options[$message] = $message;
      }
    }
    
    // Get current filter values from the URL.
    $filters = \Drupal::request()->query->all();
    
    $form['filters'] = [
      '#type' => 'details',
      '#title' => $this->t('Filter statistics'),
      '#open' => !empty($filters),
    ];
    
    $form['filters']['event_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Event Type'),
      '#options' => ['' => $this->t('- All -')] + $event_types,
      '#default_value' => $filters['event_type'] ?? '',
    ];
    
    $form['filters']['status'] = [
      '#type' => 'select',
      '#title' => $this->t('Status'),
      '#options' => ['' => $this->t('- All -')] + $statuses,
      '#default_value' => $filters['status'] ?? '',
    ];
    
    $form['filters']['individual_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Individual'),
      '#options' => ['' => $this->t('- All -')] + $individual_types + ['secondary' => 'secondary'],
      '#default_value' => $filters['individual_type'] ?? '',
    ];
    
    $form['filters']['first_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('First Name'),
      '#size' => 20,
      '#default_value' => $filters['first_name'] ?? '',
      '#description' => $this->t('Filter by first name (contains)'),
    ];
    
    $form['filters']['last_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Last Name'),
      '#size' => 20,
      '#default_value' => $filters['last_name'] ?? '',
      '#description' => $this->t('Filter by last name (contains)'),
    ];
    
    $form['filters']['message'] = [
      '#type' => 'select',
      '#title' => $this->t('Message'),
      '#options' => $message_options,
      '#default_value' => $filters['message'] ?? '',
    ];
    
    $form['filters']['actions'] = [
      '#type' => 'actions',
      '#weight' => 100,
    ];
    
    $form['filters']['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];
    
    if (!empty($filters)) {
      $form['filters']['actions']['reset'] = [
        '#type' => 'link',
        '#title' => $this->t('Reset'),
        '#url' => Url::fromRoute('virtuous_personalization.statistics'),
        '#attributes' => ['class' => ['button']],
      ];
    }
    
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $filters = [];
    $filter_fields = [
      'event_type',
      'status',
      'individual_type',
      'first_name',
      'last_name',
      'message',
    ];
    
    foreach ($filter_fields as $field) {
      $value = $form_state->getValue($field);
      if (!empty($value)) {
        $filters[$field] = $value;
      }
    }
    
    // Maintain sort parameters if they exist.
    $current_request = \Drupal::request();
    if ($current_request->query->has('sort')) {
      $filters['sort'] = $current_request->query->get('sort');
    }
    if ($current_request->query->has('order')) {
      $filters['order'] = $current_request->query->get('order');
    }
    
    $form_state->setRedirect('virtuous_personalization.statistics', [], ['query' => $filters]);
  }

  /**
   * Gets distinct values for a field from the statistics table.
   *
   * @param string $field
   *   The field to get distinct values for.
   *
   * @return array
   *   An array of distinct values.
   */
  protected function getDistinctValues($field) {
    $query = $this->database->select('virtuous_personalization_stats', 's')
      ->fields('s', [$field])
      ->distinct()
      ->orderBy($field, 'ASC');
      
    $results = $query->execute()->fetchCol();
    
    $values = [];
    foreach ($results as $result) {
      if (!empty($result)) {
        $values[$result] = $result;
      }
    }
    
    return $values;
  }

}
