<?php

namespace Drupal\virtuous_personalization\Service;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\virtuous_personalization\Statistics;

/**
 * Provides personalization services for Virtuous integration.
 */
class VirtuousPersonalizationService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new VirtuousPersonalizationService.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\virtuous_personalization\Statistics $statistics
   *   The statistics service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    AccountProxyInterface $current_user,
    Statistics $statistics,
    ConfigFactoryInterface $config_factory,
    StateInterface $state,
    TimeInterface $time
  ) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('virtuous_personalization');
    $this->currentUser = $current_user;
    $this->statistics = $statistics;
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->time = $time;
  }

  /**
   * Cleans up old personalization data.
   *
   * @return array
   *   An array containing counts of deleted records.
   */
  public function cleanupOldData() {
    // Get data retention setting
    $config = $this->configFactory->get('virtuous_personalization.settings');
    $data_retention = $config->get('data_retention') ?: 7776000; // Default: 90 days
    
    // Skip if retention is set to 0 (keep indefinitely)
    if ($data_retention === 0) {
      $this->logger->notice('Data cleanup skipped: data retention is set to keep indefinitely.');
      return ['statistics' => 0, 'mappings' => 0, 'total' => 0];
    }
    
    try {
      // Delete old statistics
      $stats_deleted = $this->deleteOldStatistics(1000, $data_retention);
      
      // Delete old visitor mappings
      $mappings_deleted = $this->deleteOldMappings(1000, $data_retention);
      
      // Log the cleanup
      $this->logger->notice('Cleaned up old personalization data: @stats statistics and @maps mappings.', [
        '@stats' => $stats_deleted,
        '@maps' => $mappings_deleted,
      ]);
      
      // Update the last cleanup time
      $this->state->set('virtuous_personalization.last_cleanup', $this->time->getRequestTime());
      
      return [
        'statistics' => $stats_deleted,
        'mappings' => $mappings_deleted,
        'total' => $stats_deleted + $mappings_deleted,
      ];
    }
    catch (\Exception $e) {
      $this->logger->error('Error cleaning up old data: @error', [
        '@error' => $e->getMessage(),
      ]);
      return ['statistics' => 0, 'mappings' => 0, 'total' => 0];
    }
  }

  /**
   * Counts old data that would be cleaned up.
   *
   * This is used to determine batch operations required.
   *
   * @param int $data_retention
   *   The data retention period in seconds.
   *
   * @return array
   *   Array with counts of old records in each table.
   */
  public function countOldData($data_retention) {
    // Calculate the cutoff timestamp
    $cutoff = $this->time->getRequestTime() - $data_retention;
    
    try {
      // Count old statistics records
      $stats_count = $this->database->select('virtuous_personalization_stats', 's')
        ->condition('s.timestamp', $cutoff, '<')
        ->countQuery()
        ->execute()
        ->fetchField();
      
      // Count old visitor mappings
      $mappings_count = $this->database->select('virtuous_visitor_map', 'v')
        ->condition('v.created', $cutoff, '<')
        ->countQuery()
        ->execute()
        ->fetchField();
      
      return [
        'statistics' => (int) $stats_count,
        'mappings' => (int) $mappings_count,
        'total' => (int) $stats_count + (int) $mappings_count,
      ];
    }
    catch (\Exception $e) {
      $this->logger->error('Error counting old data: @error', [
        '@error' => $e->getMessage(),
      ]);
      return ['statistics' => 0, 'mappings' => 0, 'total' => 0];
    }
  }

  /**
   * Deletes old statistics records.
   *
   * @param int $limit
   *   Maximum number of records to delete in one batch.
   * @param int $data_retention
   *   The data retention period in seconds.
   *
   * @return int
   *   Number of records deleted.
   */
  public function deleteOldStatistics($limit, $data_retention) {
    // Calculate the cutoff timestamp
    $cutoff = $this->time->getRequestTime() - $data_retention;
    
    try {
      // Find record IDs to delete
      $query = $this->database->select('virtuous_personalization_stats', 's')
        ->fields('s', ['id'])
        ->condition('s.timestamp', $cutoff, '<')
        ->range(0, $limit);
      
      $ids = $query->execute()->fetchCol();
      
      if (empty($ids)) {
        return 0;
      }
      
      // Delete the records
      $deleted = $this->database->delete('virtuous_personalization_stats')
        ->condition('id', $ids, 'IN')
        ->execute();
      
      $this->logger->debug('Deleted @count old statistics records', [
        '@count' => $deleted,
      ]);
      
      return $deleted;
    }
    catch (\Exception $e) {
      $this->logger->error('Error deleting old statistics: @error', [
        '@error' => $e->getMessage(),
      ]);
      return 0;
    }
  }

  /**
   * Deletes old visitor mapping records.
   *
   * @param int $limit
   *   Maximum number of records to delete in one batch.
   * @param int $data_retention
   *   The data retention period in seconds.
   *
   * @return int
   *   Number of records deleted.
   */
  public function deleteOldMappings($limit, $data_retention) {
    // Calculate the cutoff timestamp
    $cutoff = $this->time->getRequestTime() - $data_retention;
    
    try {
      // Find visitor IDs to delete
      $query = $this->database->select('virtuous_visitor_map', 'v')
        ->fields('v', ['visitor_id'])
        ->condition('v.created', $cutoff, '<')
        ->range(0, $limit);
      
      $ids = $query->execute()->fetchCol();
      
      if (empty($ids)) {
        return 0;
      }
      
      // Delete the records
      $deleted = $this->database->delete('virtuous_visitor_map')
        ->condition('visitor_id', $ids, 'IN')
        ->execute();
      
      $this->logger->debug('Deleted @count old visitor mappings', [
        '@count' => $deleted,
      ]);
      
      return $deleted;
    }
    catch (\Exception $e) {
      $this->logger->error('Error deleting old visitor mappings: @error', [
        '@error' => $e->getMessage(),
      ]);
      return 0;
    }
  }
}
