<?php

namespace Drupal\virtuous_personalization;

use Drupal\Core\Database\Connection;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Handles statistics for Virtuous Personalization.
 */
class Statistics {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new Statistics object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    Connection $database,
    LoggerChannelFactoryInterface $logger_factory,
    RequestStack $request_stack,
    TimeInterface $time,
    ConfigFactoryInterface $config_factory
  ) {
    $this->database = $database;
    $this->logger = $logger_factory->get('virtuous_personalization');
    $this->requestStack = $request_stack;
    $this->time = $time;
    $this->configFactory = $config_factory;
  }

  /**
   * Records a statistics event.
   *
   * @param string $event_type
   *   The type of event (identification, mapping, etc.).
   * @param string $status
   *   The status of the event (success, failure, etc.).
   * @param array $data
   *   Additional data to store with the event.
   */
  public function recordEvent($event_type, $status, array $data = []) {
    try {
      // Get privacy settings
      $config = $this->configFactory->get('virtuous_personalization.settings');
      $record_ip = $config->get('record_ip') ?? TRUE;
      
      // Get client IP and anonymize it if IP recording is enabled
      $ip = NULL;
      if ($record_ip) {
        $ip = $this->requestStack->getCurrentRequest()->getClientIp();
        if ($ip) {
          // Anonymize the IP (remove last octet)
          $ip_parts = explode('.', $ip);
          if (count($ip_parts) === 4) {
            $ip_parts[3] = '0';
            $ip = implode('.', $ip_parts);
          }
        }
      }
      
      // Ensure there's always a meaningful individual_id
      if (empty($data['individual_id']) || !in_array($data['individual_id'], ['primary', 'secondary'])) {
        $data['individual_id'] = 'primary';
      }
      
      // Ensure there's always a message
      if (empty($data['message'])) {
        $individual = $data['individual_id'];
        
        // Set default messages based on event type and status
        if ($event_type === 'identification') {
          if ($status === 'success') {
            $data['message'] = 'Successfully identified visitor as ' . $individual . ' individual';
          }
          elseif ($status === 'attempt') {
            $data['message'] = 'Identification attempt started';
          }
          elseif ($status === 'failure') {
            $data['message'] = 'Failed to identify visitor';
          }
          else {
            $data['message'] = 'Identification ' . $status;
          }
        }
        elseif ($event_type === 'mapping') {
          if ($status === 'create') {
            $data['message'] = 'Created new mapping for ' . $individual . ' individual';
          }
          elseif ($status === 'update') {
            $data['message'] = 'Updated existing mapping for ' . $individual . ' individual';
          }
          else {
            $data['message'] = 'Mapping ' . $status;
          }
        }
        else {
          $data['message'] = $event_type . ' ' . $status;
        }
      }
      
      // Prepare sanitized data for insertion
      $fields = [
        'event_type' => $event_type,
        'status' => $status,
        'visitor_id' => $data['visitor_id'] ?? NULL,
        'uid' => $data['uid'] ?? NULL,
        'individual_id' => $data['individual_id'],
        'message' => $data['message'],
        'timestamp' => $this->time->getRequestTime(),
      ];
      
      // Only add IP if it should be recorded
      if ($record_ip && $ip) {
        $fields['ip'] = $ip;
      }
      
      $this->database->insert('virtuous_personalization_stats')
        ->fields($fields)
        ->execute();
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to record statistics event: @error', [
        '@error' => $e->getMessage(),
      ]);
    }
  }

  /**
   * Gets identification statistics.
   *
   * @param string $period
   *   The time period to get statistics for (day, week, month, all).
   *
   * @return array
   *   An array of statistics.
   */
  public function getIdentificationStats($period = 'all') {
    try {
      // Determine timestamp for period
      $timestamp = 0;
      switch ($period) {
        case 'day':
          $timestamp = $this->time->getRequestTime() - 86400;
          break;
        case 'week':
          $timestamp = $this->time->getRequestTime() - 604800;
          break;
        case 'month':
          $timestamp = $this->time->getRequestTime() - 2592000;
          break;
      }
      
      // Base query for all events
      $query = $this->database->select('virtuous_personalization_stats', 's')
        ->fields('s', ['event_type', 'status', 'individual_id'])
        ->condition('s.event_type', 'identification');
      
      if ($timestamp > 0) {
        $query->condition('s.timestamp', $timestamp, '>=');
      }
      
      // Add expression for counting events
      $query->addExpression('COUNT(s.id)', 'count');
      
      // Group by event type, status, and individual_id
      $query->groupBy('s.event_type')
        ->groupBy('s.status')
        ->groupBy('s.individual_id');
      
      $results = $query->execute()->fetchAll();
      
      // Format the results
      $stats = [
        'total_attempts' => 0,
        'successful' => 0,
        'successful_primary' => 0,
        'successful_secondary' => 0,
        'failed' => 0,
      ];
      
      foreach ($results as $result) {
        $stats['total_attempts'] += $result->count;
        
        if ($result->status === 'success') {
          $stats['successful'] += $result->count;
          
          if ($result->individual_id === 'primary') {
            $stats['successful_primary'] += $result->count;
          }
          elseif ($result->individual_id === 'secondary') {
            $stats['successful_secondary'] += $result->count;
          }
        }
        else {
          $stats['failed'] += $result->count;
        }
      }
      
      // Calculate success rate
      $stats['success_rate'] = ($stats['total_attempts'] > 0) 
        ? round(($stats['successful'] / $stats['total_attempts']) * 100, 2) 
        : 0;
      
      return $stats;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to get identification statistics: @error', [
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Gets detailed event log entries.
   *
   * @param string $period
   *   The time period to get statistics for (day, week, month, all).
   * @param int $limit
   *   Maximum number of records to return.
   *
   * @return array
   *   An array of log entries.
   */
  public function getEventLog($period = 'all', $limit = 50) {
    try {
      // Determine timestamp for period
      $timestamp = 0;
      switch ($period) {
        case 'day':
          $timestamp = $this->time->getRequestTime() - 86400;
          break;
        case 'week':
          $timestamp = $this->time->getRequestTime() - 604800;
          break;
        case 'month':
          $timestamp = $this->time->getRequestTime() - 2592000;
          break;
      }
      
      // Query for log entries
      $query = $this->database->select('virtuous_personalization_stats', 's')
        ->fields('s')
        ->orderBy('s.timestamp', 'DESC')
        ->range(0, $limit);
      
      if ($timestamp > 0) {
        $query->condition('s.timestamp', $timestamp, '>=');
      }
      
      return $query->execute()->fetchAll();
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to get event log: @error', [
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }
  
  /**
   * Gets filtered event log entries.
   *
   * @param string $period
   *   The time period to get statistics for (day, week, month, all).
   * @param int $limit
   *   Maximum number of records to return.
   * @param array $filters
   *   Filters to apply to the query.
   * @param string $sort_field
   *   Field to sort by.
   * @param string $sort_order
   *   Sort order (ASC or DESC).
   *
   * @return array
   *   An array of log entries.
   */
  public function getFilteredEventLog($period = 'all', $limit = 50, array $filters = [], $sort_field = 'id', $sort_order = 'DESC') {
    try {
      // Get the query builder
      $query = $this->getFilteredEventQueryBuilder($period, $filters, $sort_field, $sort_order);
      
      // Apply limit
      $query->range(0, $limit);
      
      // Execute query
      return $query->execute()->fetchAll();
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to get filtered event log: @error', [
        '@error' => $e->getMessage(),
      ]);
      return [];
    }
  }
  
  /**
   * Gets a query builder for filtered event log.
   *
   * This returns a query builder rather than results to allow for paging.
   *
   * @param string $period
   *   The time period to get statistics for (day, week, month, all).
   * @param array $filters
   *   Filters to apply to the query.
   * @param string $sort_field
   *   Field to sort by.
   * @param string $sort_order
   *   Sort order (ASC or DESC).
   *
   * @return \Drupal\Core\Database\Query\SelectInterface
   *   A database query object that can be extended.
   */
  public function getFilteredEventQueryBuilder($period = 'all', array $filters = [], $sort_field = 'id', $sort_order = 'DESC') {
    try {
      // Determine timestamp for period
      $timestamp = 0;
      switch ($period) {
        case 'day':
          $timestamp = $this->time->getRequestTime() - 86400;
          break;
        case 'week':
          $timestamp = $this->time->getRequestTime() - 604800;
          break;
        case 'month':
          $timestamp = $this->time->getRequestTime() - 2592000;
          break;
      }
      
      // Query for log entries
      $query = $this->database->select('virtuous_personalization_stats', 's')
        ->fields('s');
        
      // Join with user table for name filtering if needed
      if (!empty($filters['first_name']) || !empty($filters['last_name'])) {
        $query->leftJoin('users_field_data', 'u', 's.uid = u.uid');
        $query->leftJoin('user__primary_first_name', 'pf', 'u.uid = pf.entity_id');
        $query->leftJoin('user__primary_last_name', 'pl', 'u.uid = pl.entity_id');
        $query->leftJoin('user__secondary_first_name', 'sf', 'u.uid = sf.entity_id');
        $query->leftJoin('user__secondary_last_name', 'sl', 'u.uid = sl.entity_id');
      }
      
      if ($timestamp > 0) {
        $query->condition('s.timestamp', $timestamp, '>=');
      }
      
      // Apply filters
      if (!empty($filters['event_type'])) {
        $query->condition('s.event_type', $filters['event_type']);
      }
      
      if (!empty($filters['status'])) {
        $query->condition('s.status', $filters['status']);
      }
      
      if (!empty($filters['individual_type'])) {
        $query->condition('s.individual_id', $filters['individual_type']);
      }
      
      if (!empty($filters['message'])) {
        $query->condition('s.message', '%' . $this->database->escapeLike($filters['message']) . '%', 'LIKE');
      }
      
      // First name filter
      if (!empty($filters['first_name'])) {
        $or = $query->orConditionGroup()
          ->condition('pf.primary_first_name_value', '%' . $this->database->escapeLike($filters['first_name']) . '%', 'LIKE')
          ->condition('sf.secondary_first_name_value', '%' . $this->database->escapeLike($filters['first_name']) . '%', 'LIKE');
        $query->condition($or);
      }
      
      // Last name filter
      if (!empty($filters['last_name'])) {
        $or = $query->orConditionGroup()
          ->condition('pl.primary_last_name_value', '%' . $this->database->escapeLike($filters['last_name']) . '%', 'LIKE')
          ->condition('sl.secondary_last_name_value', '%' . $this->database->escapeLike($filters['last_name']) . '%', 'LIKE');
        $query->condition($or);
      }
      
      // Apply sorting
      $valid_sort_fields = ['id', 'event_type', 'status', 'uid', 'timestamp'];
      if (in_array($sort_field, $valid_sort_fields)) {
        $query->orderBy('s.' . $sort_field, $sort_order);
      }
      // Special case for fields that aren't directly in the stats table
      elseif ($sort_field == 'contact_id' || $sort_field == 'individual_id') {
        // Default sort by ID if we can't sort by these fields
        $query->orderBy('s.id', $sort_order);
      }
      else {
        // Default sort
        $query->orderBy('s.id', 'DESC');
      }
      
      return $query;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to create filtered event log query: @error', [
        '@error' => $e->getMessage(),
      ]);
      // Return an empty query in case of error
      return $this->database->select('virtuous_personalization_stats', 's')
        ->fields('s')
        ->condition('id', 0); // This will return no results
    }
  }
  
  /**
   * Sanitizes personal data according to privacy settings.
   *
   * This can be used for GDPR compliance to remove personal data
   * for a specific user or visitor ID.
   *
   * @param array $options
   *   Options for sanitization (uid or visitor_id).
   *
   * @return int
   *   The number of records affected.
   */
  public function sanitizePersonalData(array $options) {
    try {
      // Create update query
      $query = $this->database->update('virtuous_personalization_stats');
      
      // Fields to sanitize
      $fields = [
        'ip' => NULL,
        'message' => 'Data removed per privacy request',
      ];
      
      // Set fields
      $query->fields($fields);
      
      // Add conditions based on options
      if (!empty($options['uid'])) {
        $query->condition('uid', $options['uid']);
      }
      elseif (!empty($options['visitor_id'])) {
        $query->condition('visitor_id', $options['visitor_id']);
      }
      else {
        throw new \InvalidArgumentException('Either uid or visitor_id must be provided');
      }
      
      // Execute the query
      $affected = $query->execute();
      
      // Log the action
      $this->logger->notice('Sanitized personal data: @count records affected', [
        '@count' => $affected,
      ]);
      
      return $affected;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to sanitize personal data: @error', [
        '@error' => $e->getMessage(),
      ]);
      return 0;
    }
  }
}
