<?php

namespace Drupal\virtuous_personalization;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\virtuous_personalization\Statistics;

/**
 * Manages visitor ID mappings and personalization.
 */
class VisitorManager {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new VisitorManager object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\virtuous_personalization\Statistics $statistics
   *   The statistics service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    AccountProxyInterface $current_user,
    Statistics $statistics,
    CacheBackendInterface $cache,
    ConfigFactoryInterface $config_factory
  ) {
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('virtuous_personalization');
    $this->currentUser = $current_user;
    $this->statistics = $statistics;
    $this->cache = $cache;
    $this->configFactory = $config_factory;
  }

  /**
   * Gets a user by Virtuous visitor ID.
   *
   * Uses cache to improve performance for repeated lookups.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   *
   * @return array|null
   *   An array containing the user entity and individual type, or NULL if not found.
   */
  public function getUserByVisitorId($visitor_id) {
    try {
      // Generate a cache ID for this visitor
      $cid = 'virtuous_personalization:visitor:' . $visitor_id;
      
      // Check if we have a cached result
      if ($cache = $this->cache->get($cid)) {
        $this->logger->notice('Visitor data for ID @id retrieved from cache', [
          '@id' => $visitor_id,
        ]);
        return $cache->data;
      }
      
      // If not in cache, query the database
      $result = $this->lookupUserByVisitorId($visitor_id);
      
      // Cache the result if found
      if ($result) {
        // Get cache lifetime from settings
        $config = $this->configFactory->get('virtuous_personalization.settings');
        $cache_lifetime = $config->get('cache_lifetime') ?: 3600; // Default: 1 hour
        
        // Cache the user data
        $this->cache->set(
          $cid,
          $result,
          time() + $cache_lifetime,
          [
            'virtuous_personalization',
            'virtuous_visitor',
            'user:' . $result['user']->id(),
          ]
        );
        
        $this->logger->debug('Cached visitor data for ID @id', [
          '@id' => $visitor_id,
        ]);
      }
      
      return $result;
    }
    catch (\Exception $e) {
      $this->logger->error('Error finding user for visitor ID @id: @error', [
        '@id' => $visitor_id,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
  }

  /**
   * Looks up a user by Virtuous visitor ID without using cache.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   *
   * @return array|null
   *   An array containing the user entity and individual type, or NULL if not found.
   */
  protected function lookupUserByVisitorId($visitor_id) {
    // First check the mapping table.
    $query = $this->database->select('virtuous_visitor_map', 'vvm')
      ->fields('vvm', ['uid', 'individual_id'])
      ->condition('vvm.visitor_id', $visitor_id)
      ->execute();
    
    $result = $query->fetchAssoc();
    
    if ($result && !empty($result['uid'])) {
      $user = $this->entityTypeManager->getStorage('user')->load($result['uid']);
      return [
        'user' => $user,
        'individual_id' => $result['individual_id'] ?? 'primary',
      ];
    }
    
    // Try by primary_id
    $primary_users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['primary_id' => $visitor_id]);
    
    if ($primary_users) {
      return [
        'user' => reset($primary_users),
        'individual_id' => 'primary',
      ];
    }
    
    // Try by secondary_id
    $secondary_users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['secondary_id' => $visitor_id]);
    
    if ($secondary_users) {
      return [
        'user' => reset($secondary_users),
        'individual_id' => 'secondary',
      ];
    }
    
    // Try by contact_id directly.
    $users = $this->entityTypeManager->getStorage('user')
      ->loadByProperties(['contact_id' => $visitor_id]);
    
    if ($users) {
      return [
        'user' => reset($users),
        'individual_id' => 'primary',  // Default to primary for contact_id matches
      ];
    }
    
    return NULL;
  }

  /**
   * Creates or updates a visitor mapping.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   * @param int $uid
   *   The Drupal user ID.
   * @param string $individual_id
   *   Which individual this visitor ID represents ('primary' or 'secondary').
   *
   * @return bool
   *   TRUE on success, FALSE on failure.
   */
  public function saveVisitorMapping($visitor_id, $uid, $individual_id = NULL) {
    try {
      // Always ensure we have a valid individual_id
      if (empty($individual_id) || !in_array($individual_id, ['primary', 'secondary'])) {
        $individual_id = 'primary'; // Default to primary if not specified or invalid
      }
      
      // Check if mapping already exists.
      $existing = $this->database->select('virtuous_visitor_map', 'vvm')
        ->fields('vvm', ['visitor_id'])
        ->condition('vvm.visitor_id', $visitor_id)
        ->execute()
        ->fetchField();
      
      if ($existing) {
        // Update existing mapping.
        $fields = [
          'uid' => $uid,
          'individual_id' => $individual_id,
          'created' => \Drupal::time()->getRequestTime(),
        ];
        
        $this->database->update('virtuous_visitor_map')
          ->fields($fields)
          ->condition('visitor_id', $visitor_id)
          ->execute();
        
        $this->logger->notice('Updated existing visitor mapping: @id to @uid', [
          '@id' => $visitor_id, 
          '@uid' => $uid,
        ]);
        
        $this->statistics->recordEvent('mapping', 'update', [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'individual_id' => $individual_id,
        ]);
      }
      else {
        // Insert new mapping.
        $fields = [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'individual_id' => $individual_id,
          'created' => \Drupal::time()->getRequestTime(),
        ];
        
        $this->database->insert('virtuous_visitor_map')
          ->fields($fields)
          ->execute();
        
        $this->logger->notice('Created new visitor mapping: @id to @uid', [
          '@id' => $visitor_id, 
          '@uid' => $uid,
        ]);
        
        $this->statistics->recordEvent('mapping', 'create', [
          'visitor_id' => $visitor_id,
          'uid' => $uid,
          'individual_id' => $individual_id,
        ]);
      }
      
      // Invalidate any existing cache for this visitor
      $this->invalidateVisitorCache($visitor_id, $uid);
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to save visitor mapping: @error', [
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * Invalidates cache entries related to a visitor.
   *
   * @param string $visitor_id
   *   The Virtuous visitor ID.
   * @param int $uid
   *   The Drupal user ID.
   */
  protected function invalidateVisitorCache($visitor_id, $uid) {
    // Invalidate specific visitor cache
    $this->cache->invalidate('virtuous_personalization:visitor:' . $visitor_id);
    
    // Invalidate caches for this user
    if ($uid) {
      $this->cache->invalidateTags(['user:' . $uid]);
    }
    
    $this->logger->debug('Invalidated cache for visitor @visitor and user @uid', [
      '@visitor' => $visitor_id,
      '@uid' => $uid ?: 'none',
    ]);
  }
}
