<?php

namespace Drupal\Tests\virtuous_personalization\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\User;

/**
 * Tests the Virtuous Personalization integration.
 *
 * @group virtuous_personalization
 */
class PersonalizationTest extends BrowserTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'virtuous',
    'virtuous_personalization',
    'key',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user with permission to administer Virtuous configuration.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;
  
  /**
   * A test user with primary contact info.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $contactUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create and log in an admin user
    $this->adminUser = $this->drupalCreateUser([
      'administer virtuous configuration',
      'administer keys',
    ]);
    
    // Create a user with Virtuous contact IDs
    $this->contactUser = User::create([
      'name' => 'contact_user',
      'mail' => 'contact@example.com',
      'status' => 1,
      'contact_id' => 'test-contact-id',
      'primary_id' => 'test-primary-id',
      'primary_first_name' => 'Test',
      'primary_last_name' => 'User',
      'pass' => $this->randomString(),
    ]);
    $this->contactUser->save();
    
    // Create custom field storage and fields for contact information
    $this->createContactFields();
  }
  
  /**
   * Creates the necessary field storage and fields for contact information.
   */
  protected function createContactFields() {
    // This would be created by the virtuous module, but we mock it for testing
    if (!field_info_field_storage_exists('user', 'contact_id')) {
      $this->createStorage('contact_id', 'string');
    }
    if (!field_info_field_exists('user', 'contact_id')) {
      $this->createField('contact_id', 'Contact ID', 'string');
    }
    
    if (!field_info_field_storage_exists('user', 'primary_id')) {
      $this->createStorage('primary_id', 'string');
    }
    if (!field_info_field_exists('user', 'primary_id')) {
      $this->createField('primary_id', 'Primary Individual ID', 'string');
    }
    
    if (!field_info_field_storage_exists('user', 'primary_first_name')) {
      $this->createStorage('primary_first_name', 'string');
    }
    if (!field_info_field_exists('user', 'primary_first_name')) {
      $this->createField('primary_first_name', 'Primary First Name', 'string');
    }
    
    if (!field_info_field_storage_exists('user', 'primary_last_name')) {
      $this->createStorage('primary_last_name', 'string');
    }
    if (!field_info_field_exists('user', 'primary_last_name')) {
      $this->createField('primary_last_name', 'Primary Last Name', 'string');
    }
  }
  
  /**
   * Helper function to create field storage.
   */
  protected function createStorage($field_name, $type) {
    $storage = \Drupal::entityTypeManager()
      ->getStorage('field_storage_config')
      ->create([
        'field_name' => $field_name,
        'entity_type' => 'user',
        'type' => $type,
        'cardinality' => 1,
      ]);
    $storage->save();
  }
  
  /**
   * Helper function to create field instance.
   */
  protected function createField($field_name, $label, $type) {
    $field = \Drupal::entityTypeManager()
      ->getStorage('field_config')
      ->create([
        'field_name' => $field_name,
        'entity_type' => 'user',
        'bundle' => 'user',
        'label' => $label,
        'required' => FALSE,
      ]);
    $field->save();
    
    // Set the form display
    \Drupal::entityTypeManager()
      ->getStorage('entity_form_display')
      ->load('user.user.default')
      ->setComponent($field_name, [
        'type' => $type . '_textfield',
        'weight' => 10,
      ])
      ->save();
  }

  /**
   * Tests the personalization settings form.
   */
  public function testPersonalizationSettingsForm() {
    // Log in as admin
    $this->drupalLogin($this->adminUser);
    
    // Create a test key for Organization ID
    $edit = [
      'id' => 'virtuous_org_id',
      'label' => 'Virtuous Organization ID',
      'key_type' => 'authentication',
      'key_provider' => 'config',
      'key_input' => 'text_field',
      'key_value' => '6936b0fa-2a91-47b3-8c5b-19e2c7ab02c5',
    ];
    $this->drupalGet('admin/config/system/keys/add');
    $this->submitForm($edit, 'Save');
    $this->assertSession()->pageTextContains('The key has been added.');
    
    // Access the personalization settings form
    $this->drupalGet('admin/config/virtuous/personalization');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Virtuous Personalization Settings');
    
    // Submit the form with test values
    $edit = [
      'enabled' => TRUE,
      'organization_id_key' => 'virtuous_org_id',
      'visitor_recording' => TRUE,
      'name_display' => 'first',
      'greeting_text' => 'Hello, [NAME]! Welcome back.',
      'data_retention' => '7776000',
      'record_ip' => TRUE,
      'consent_required' => FALSE,
      'cache_lifetime' => '3600',
      'batch_size' => '100',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    
    // Verify the saved settings
    $this->drupalGet('admin/config/virtuous/personalization');
    $this->assertSession()->checkboxChecked('edit-enabled');
    $this->assertSession()->fieldValueEquals('organization_id_key', 'virtuous_org_id');
    $this->assertSession()->checkboxChecked('edit-visitor-recording');
    $this->assertSession()->fieldValueEquals('name_display', 'first');
    $this->assertSession()->fieldValueEquals('greeting_text', 'Hello, [NAME]! Welcome back.');
  }
  
  /**
   * Tests the statistics page.
   */
  public function testStatisticsPage() {
    // Log in as admin
    $this->drupalLogin($this->adminUser);
    
    // Access the statistics page
    $this->drupalGet('admin/config/virtuous/personalization/statistics');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Identification Statistics');
    $this->assertSession()->pageTextContains('Recent Events');
    
    // Verify the presence of filter form
    $this->assertSession()->fieldExists('event_type');
    $this->assertSession()->fieldExists('status');
    $this->assertSession()->fieldExists('individual_type');
    
    // Verify the cleanup button exists
    $this->assertSession()->linkExists('Clean up old data');
  }
  
  /**
   * Tests the visitor identification API.
   */
  public function testVisitorIdentificationApi() {
    // Create a test visitor ID
    $visitor_id = 'test-visitor-id-' . $this->randomMachineName();
    
    // Test visitor lookup with unknown ID
    $this->drupalGet('/virtuous/visitor/' . $visitor_id);
    $this->assertSession()->statusCodeEquals(200);
    
    // Parse the JSON response
    $response = json_decode($this->getSession()->getPage()->getContent(), TRUE);
    $this->assertFalse($response['isIdentified'], 'Unknown visitor should not be identified');
    
    // Test visitor lookup with known primary ID
    $this->drupalGet('/virtuous/visitor/' . $this->contactUser->get('primary_id')->value);
    $this->assertSession()->statusCodeEquals(200);
    
    // Parse the JSON response
    $response = json_decode($this->getSession()->getPage()->getContent(), TRUE);
    $this->assertTrue($response['isIdentified'], 'User should be identified by primary ID');
    $this->assertEquals('Test User', $response['contactName'], 'Contact name should match user data');
    
    // Test visitor mapping with CSRF token
    $csrf_token = $this->drupalGetToken();
    $this->drupalGet('/virtuous/record-visitor/' . $visitor_id . '?_format=json', [
      'headers' => [
        'X-CSRF-Token' => $csrf_token,
      ],
      'method' => 'POST',
    ]);
    $this->assertSession()->statusCodeEquals(200);
    
    // Parse the JSON response
    $response = json_decode($this->getSession()->getPage()->getContent(), TRUE);
    $this->assertTrue($response['success'], 'Visitor mapping should be successful');
  }
  
  /**
   * Tests the privacy settings functionality.
   */
  public function testPrivacySettings() {
    // Log in as admin
    $this->drupalLogin($this->adminUser);
    
    // Configure privacy settings
    $edit = [
      'enabled' => TRUE,
      'data_retention' => '2592000', // 30 days
      'record_ip' => FALSE,
      'consent_required' => TRUE,
      'consent_cookie' => 'test_consent',
    ];
    $this->drupalGet('admin/config/virtuous/personalization');
    $this->submitForm($edit, 'Save configuration');
    
    // Verify settings were saved
    $config = $this->config('virtuous_personalization.settings');
    $this->assertEquals('2592000', $config->get('data_retention'));
    $this->assertFalse($config->get('record_ip'));
    $this->assertTrue($config->get('consent_required'));
    $this->assertEquals('test_consent', $config->get('consent_cookie'));
    
    // Create a test statistic entry with anonymized user data
    $time = \Drupal::time()->getRequestTime();
    \Drupal::database()->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'visitor_id' => 'test-visitor-id',
        'uid' => $this->contactUser->id(),
        'individual_id' => 'primary',
        'message' => 'Test identification',
        'timestamp' => $time,
        'ip' => NULL, // Should be NULL due to the privacy setting
      ])
      ->execute();
    
    // Verify the entry was created without IP
    $result = \Drupal::database()->select('virtuous_personalization_stats', 's')
      ->fields('s')
      ->condition('s.visitor_id', 'test-visitor-id')
      ->execute()
      ->fetchAssoc();
    
    $this->assertNotEmpty($result, 'Statistics entry should exist');
    $this->assertNull($result['ip'], 'IP address should be NULL');
  }
  
  /**
   * Tests the batch processing for cleanup.
   */
  public function testBatchCleanup() {
    // Log in as admin
    $this->drupalLogin($this->adminUser);
    
    // Configure settings for batch size
    $edit = [
      'batch_size' => '50',
    ];
    $this->drupalGet('admin/config/virtuous/personalization');
    $this->submitForm($edit, 'Save configuration');
    
    // Create 100 test statistics entries with old timestamps
    $time = \Drupal::time()->getRequestTime() - 31536000; // 1 year ago
    $query = \Drupal::database()->insert('virtuous_personalization_stats')
      ->fields(['event_type', 'status', 'visitor_id', 'individual_id', 'message', 'timestamp']);
    
    for ($i = 0; $i < 100; $i++) {
      $query->values([
        'event_type' => 'test',
        'status' => 'test',
        'visitor_id' => 'batch-test-' . $i,
        'individual_id' => 'primary',
        'message' => 'Batch test entry ' . $i,
        'timestamp' => $time,
      ]);
    }
    $query->execute();
    
    // Create 50 test visitor mappings with old timestamps
    $query = \Drupal::database()->insert('virtuous_visitor_map')
      ->fields(['visitor_id', 'uid', 'individual_id', 'created']);
    
    for ($i = 0; $i < 50; $i++) {
      $query->values([
        'visitor_id' => 'batch-map-test-' . $i,
        'uid' => $this->contactUser->id(),
        'individual_id' => 'primary',
        'created' => $time,
      ]);
    }
    $query->execute();
    
    // Count records before cleanup
    $stats_count_before = \Drupal::database()
      ->select('virtuous_personalization_stats', 's')
      ->countQuery()
      ->execute()
      ->fetchField();
    
    $maps_count_before = \Drupal::database()
      ->select('virtuous_visitor_map', 'v')
      ->countQuery()
      ->execute()
      ->fetchField();
    
    // Run the cleanup process
    $this->drupalGet('admin/config/virtuous/personalization/cleanup');
    
    // The batch processing should redirect back to the statistics page
    $this->assertSession()->addressEquals('admin/config/virtuous/personalization/statistics');
    $this->assertSession()->pageTextContains('Successfully cleaned up');
    
    // Count records after cleanup
    $stats_count_after = \Drupal::database()
      ->select('virtuous_personalization_stats', 's')
      ->countQuery()
      ->execute()
      ->fetchField();
    
    $maps_count_after = \Drupal::database()
      ->select('virtuous_visitor_map', 'v')
      ->countQuery()
      ->execute()
      ->fetchField();
    
    // Verify cleanup was effective
    $this->assertLessThan($stats_count_before, $stats_count_after, 'Statistics records should be reduced after cleanup');
    $this->assertLessThan($maps_count_before, $maps_count_after, 'Visitor mapping records should be reduced after cleanup');
  }
}
