<?php

namespace Drupal\Tests\virtuous_personalization\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\key\Entity\Key;

/**
 * Tests the settings form for Virtuous Personalization.
 *
 * @group virtuous_personalization
 */
class VirtuousPersonalizationSettingsTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'virtuous',
    'virtuous_personalization',
    'key',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create and log in admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer virtuous configuration',
      'administer keys',
    ]);
    $this->drupalLogin($this->adminUser);

    // Create a test key for the Organization ID.
    $key = Key::create([
      'id' => 'test_organization_id',
      'label' => 'Test Organization ID',
      'key_type' => 'authentication',
      'key_provider' => 'config',
      'key_input' => 'text_field',
      'key_provider_settings' => [
        'key_value' => '6936b0fa-2a91-47b3-8c5b-19e2c7ab02c5',
      ],
    ]);
    $key->save();
  }

  /**
   * Tests the settings form.
   */
  public function testSettingsForm() {
    // Access the settings form.
    $this->drupalGet('admin/config/virtuous/personalization');
    $this->assertSession()->statusCodeEquals(200);

    // Check that form elements exist.
    $this->assertSession()->fieldExists('enabled');
    $this->assertSession()->fieldExists('organization_id_key');
    $this->assertSession()->fieldExists('visitor_recording');
    
    // Ensure the key we created appears in the dropdown.
    $this->assertSession()->optionExists('organization_id_key', 'test_organization_id');

    // Submit the form with settings enabled.
    $edit = [
      'enabled' => 1,
      'organization_id_key' => 'test_organization_id',
      'visitor_recording' => 1,
    ];
    $this->submitForm($edit, 'Save configuration');
    
    // Verify success message and configs saved.
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    
    // Verify config values were saved.
    $config = $this->config('virtuous_personalization.settings');
    $this->assertEquals(TRUE, $config->get('enabled'));
    $this->assertEquals('test_organization_id', $config->get('organization_id_key'));
    $this->assertEquals(TRUE, $config->get('visitor_recording'));
  }

  /**
   * Tests the script preview display.
   */
  public function testScriptPreview() {
    // Access the settings form.
    $this->drupalGet('admin/config/virtuous/personalization');
    
    // The script preview should not be visible initially.
    $this->assertSession()->elementNotExists('css', '#script-preview:visible');
    
    // Select the organization ID key.
    $this->getSession()->getPage()->selectFieldOption('organization_id_key', 'test_organization_id');
    
    // The script preview should now be visible.
    $this->assertSession()->elementExists('css', '#script-preview');
    
    // The script preview should contain the organization ID.
    $this->assertSession()->responseContains('6936b0fa-2a91-47b3-8c5b-19e2c7ab02c5');
  }
}
