<?php

namespace Drupal\Tests\virtuous_personalization\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\user\Entity\User;
use Drupal\virtuous_personalization\Statistics;

/**
 * Tests statistics functionality for Virtuous Personalization.
 *
 * @group virtuous_personalization
 */
class StatisticsTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'virtuous',
    'virtuous_personalization',
  ];

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installSchema('system', ['sequences']);
    $this->installSchema('user', ['users_data']);
    $this->installEntitySchema('user');
    $this->installSchema('virtuous_personalization', [
      'virtuous_visitor_map',
      'virtuous_personalization_stats',
    ]);

    // Create the statistics service.
    $this->statistics = $this->container->get('virtuous_personalization.statistics');
  }

  /**
   * Tests the recording and retrieval of statistics events.
   */
  public function testStatisticsRecording() {
    // Create a test user.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $user->save();

    // Generate a test visitor ID.
    $visitor_id = '12345-test-visitor-id';

    // Test recording an event.
    $this->statistics->recordEvent('identification', 'attempt', [
      'visitor_id' => $visitor_id,
    ]);

    // Test recording a success event.
    $this->statistics->recordEvent('identification', 'success', [
      'visitor_id' => $visitor_id,
      'uid' => $user->id(),
      'individual_id' => 'primary',
      'message' => 'Test identification',
    ]);

    // Test retrieving statistics.
    $stats = $this->statistics->getIdentificationStats('all');

    // Verify the statistics.
    $this->assertEquals(2, $stats['total_attempts'], 'Total attempts recorded correctly');
    $this->assertEquals(1, $stats['successful'], 'Successful attempts recorded correctly');
    $this->assertEquals(1, $stats['successful_primary'], 'Successful primary attempts recorded correctly');
    $this->assertEquals(0, $stats['successful_secondary'], 'Successful secondary attempts recorded correctly');
    $this->assertEquals(1, $stats['failed'], 'Failed attempts recorded correctly');
    $this->assertEquals(50, $stats['success_rate'], 'Success rate calculated correctly');

    // Test event log retrieval.
    $log_entries = $this->statistics->getEventLog('all', 10);
    $this->assertCount(2, $log_entries, 'Correct number of log entries retrieved');

    // Verify the log entries.
    $success_entry = NULL;
    foreach ($log_entries as $entry) {
      if ($entry->status === 'success') {
        $success_entry = $entry;
        break;
      }
    }

    $this->assertNotNull($success_entry, 'Success entry found in log');
    $this->assertEquals('identification', $success_entry->event_type, 'Event type recorded correctly');
    $this->assertEquals('success', $success_entry->status, 'Status recorded correctly');
    $this->assertEquals($visitor_id, $success_entry->visitor_id, 'Visitor ID recorded correctly');
    $this->assertEquals($user->id(), $success_entry->uid, 'User ID recorded correctly');
    $this->assertEquals('primary', $success_entry->individual_id, 'Individual ID recorded correctly');
    $this->assertEquals('Test identification', $success_entry->message, 'Message recorded correctly');
  }

  /**
   * Tests statistics filtering by time period.
   */
  public function testStatisticsPeriodFiltering() {
    // Create events with different timestamps.
    $now = \Drupal::time()->getRequestTime();
    $day_ago = $now - 86400;
    $week_ago = $now - 604800;
    $month_ago = $now - 2592000;
    $older = $now - 3000000;

    // Insert test records directly into the database with different timestamps.
    $this->container->get('database')->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'timestamp' => $now,
      ])
      ->execute();

    $this->container->get('database')->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'timestamp' => $day_ago + 3600, // Within day range
      ])
      ->execute();

    $this->container->get('database')->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'timestamp' => $week_ago + 3600, // Within week range
      ])
      ->execute();

    $this->container->get('database')->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'timestamp' => $month_ago + 3600, // Within month range
      ])
      ->execute();

    $this->container->get('database')->insert('virtuous_personalization_stats')
      ->fields([
        'event_type' => 'identification',
        'status' => 'success',
        'timestamp' => $older, // Older than month
      ])
      ->execute();

    // Test retrieving statistics for different time periods.
    $daily_stats = $this->statistics->getIdentificationStats('day');
    $weekly_stats = $this->statistics->getIdentificationStats('week');
    $monthly_stats = $this->statistics->getIdentificationStats('month');
    $all_time_stats = $this->statistics->getIdentificationStats('all');

    // Verify the statistics.
    $this->assertEquals(2, $daily_stats['successful'], 'Daily statistics filtered correctly');
    $this->assertEquals(3, $weekly_stats['successful'], 'Weekly statistics filtered correctly');
    $this->assertEquals(4, $monthly_stats['successful'], 'Monthly statistics filtered correctly');
    $this->assertEquals(5, $all_time_stats['successful'], 'All-time statistics include all records');
  }
}
