<?php

namespace Drupal\Tests\virtuous_personalization\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\user\Entity\User;

/**
 * Tests database functionality for Virtuous Personalization.
 *
 * @group virtuous_personalization
 */
class VirtuousPersonalizationDatabaseTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'virtuous',
    'virtuous_personalization',
  ];

  /**
   * The visitor manager service.
   *
   * @var \Drupal\virtuous_personalization\VisitorManager
   */
  protected $visitorManager;

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics
   */
  protected $statistics;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installSchema('system', ['sequences']);
    $this->installSchema('user', ['users_data']);
    $this->installEntitySchema('user');
    $this->installSchema('virtuous_personalization', [
      'virtuous_visitor_map',
      'virtuous_personalization_stats',
    ]);

    // Create the visitor manager service.
    $this->visitorManager = $this->container->get('virtuous_personalization.visitor_manager');
    
    // Create the statistics service.
    $this->statistics = $this->container->get('virtuous_personalization.statistics');
  }

  /**
   * Tests the database visitor mapping operations.
   */
  public function testVisitorMapping() {
    // Create a test user.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $user->save();

    // Generate a test visitor ID.
    $visitor_id = '12345-test-visitor-id';

    // Test saving a visitor mapping.
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $user->id());
    $this->assertTrue($result, 'Successfully saved visitor mapping');

    // Test retrieving a user by visitor ID.
    $retrieved_user = $this->visitorManager->getUserByVisitorId($visitor_id);
    $this->assertNotNull($retrieved_user, 'User was retrieved by visitor ID');
    $this->assertEquals($user->id(), $retrieved_user['user']->id(), 'Retrieved user matches expected user');

    // Test updating a visitor mapping.
    $user2 = User::create([
      'name' => 'testuser2',
      'mail' => 'test2@example.com',
      'status' => 1,
    ]);
    $user2->save();

    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $user2->id());
    $this->assertTrue($result, 'Successfully updated visitor mapping');

    // Verify the mapping was updated.
    $retrieved_user = $this->visitorManager->getUserByVisitorId($visitor_id);
    $this->assertEquals($user2->id(), $retrieved_user['user']->id(), 'Retrieved user matches updated user');
  }

  /**
   * Tests the visitor mapping with individual_id.
   */
  public function testVisitorMappingWithIndividualId() {
    // Create a test user.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $user->save();

    // Generate a test visitor ID.
    $visitor_id = '12345-test-visitor-id';

    // Test saving a visitor mapping with individual_id.
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $user->id(), 'primary');
    $this->assertTrue($result, 'Successfully saved visitor mapping with individual_id');

    // Test retrieving a user by visitor ID with individual_id.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId($visitor_id);
    $this->assertNotNull($retrieved_user_info, 'User was retrieved by visitor ID');
    $this->assertEquals($user->id(), $retrieved_user_info['user']->id(), 'Retrieved user matches expected user');
    $this->assertEquals('primary', $retrieved_user_info['individual_id'], 'Retrieved individual_id matches expected value');

    // Test updating a visitor mapping with individual_id.
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $user->id(), 'secondary');
    $this->assertTrue($result, 'Successfully updated visitor mapping with individual_id');

    // Verify the mapping was updated.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId($visitor_id);
    $this->assertEquals('secondary', $retrieved_user_info['individual_id'], 'Retrieved individual_id was updated');
  }

  /**
   * Tests the primary_id lookup functionality.
   */
  public function testPrimaryIdLookup() {
    // Create a test user with primary_id field.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    // Set a custom field value for primary_id.
    $user->set('primary_id', '67890-primary-id');
    $user->save();

    // Test retrieving a user by primary_id.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId('67890-primary-id');
    $this->assertNotNull($retrieved_user_info, 'User was retrieved by primary_id');
    $this->assertEquals($user->id(), $retrieved_user_info['user']->id(), 'Retrieved user matches expected user');
    $this->assertEquals('primary', $retrieved_user_info['individual_id'], 'Retrieved individual_id is primary');
  }

  /**
   * Tests the secondary_id lookup functionality.
   */
  public function testSecondaryIdLookup() {
    // Create a test user with secondary_id field.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    // Set a custom field value for secondary_id.
    $user->set('secondary_id', '54321-secondary-id');
    $user->save();

    // Test retrieving a user by secondary_id.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId('54321-secondary-id');
    $this->assertNotNull($retrieved_user_info, 'User was retrieved by secondary_id');
    $this->assertEquals($user->id(), $retrieved_user_info['user']->id(), 'Retrieved user matches expected user');
    $this->assertEquals('secondary', $retrieved_user_info['individual_id'], 'Retrieved individual_id is secondary');
  }

  /**
   * Tests the contact_id lookup functionality.
   */
  public function testContactIdLookup() {
    // Create a test user with contact_id field.
    $user = User::create([
      'name' => 'testuser',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    // Set a custom field value for contact_id.
    $user->set('contact_id', '11111-contact-id');
    $user->save();

    // Test retrieving a user by contact_id.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId('11111-contact-id');
    $this->assertNotNull($retrieved_user_info, 'User was retrieved by contact_id');
    $this->assertEquals($user->id(), $retrieved_user_info['user']->id(), 'Retrieved user matches expected user');
    $this->assertEquals('primary', $retrieved_user_info['individual_id'], 'Retrieved individual_id defaults to primary for contact_id lookups');
  }

  /**
   * Tests lookup priority (mapping table first, then ID fields).
   */
  public function testLookupPriority() {
    // Create two test users.
    $user1 = User::create([
      'name' => 'testuser1',
      'mail' => 'test1@example.com',
      'status' => 1,
    ]);
    $user1->set('primary_id', 'priority-test-id');
    $user1->save();

    $user2 = User::create([
      'name' => 'testuser2',
      'mail' => 'test2@example.com',
      'status' => 1,
    ]);
    $user2->save();

    // Create a mapping for user2 with the same ID that user1 has as primary_id.
    $result = $this->visitorManager->saveVisitorMapping('priority-test-id', $user2->id());
    $this->assertTrue($result, 'Successfully saved visitor mapping');

    // Verify that the mapping table has priority over the primary_id field.
    $retrieved_user_info = $this->visitorManager->getUserByVisitorId('priority-test-id');
    $this->assertEquals($user2->id(), $retrieved_user_info['user']->id(), 'Mapping table lookup has priority over primary_id field lookup');
  }
}
