<?php

namespace Drupal\Tests\virtuous_personalization\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\virtuous_personalization\VisitorManager;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\virtuous_personalization\Statistics;
use Drupal\user\UserInterface;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * @coversDefaultClass \Drupal\virtuous_personalization\VisitorManager
 * @group virtuous_personalization
 */
class VisitorManagerTest extends UnitTestCase {
  use ProphecyTrait;
  
  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $entityTypeManager;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $loggerFactory;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $logger;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $currentUser;

  /**
   * The statistics service.
   *
   * @var \Drupal\virtuous_personalization\Statistics|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $statistics;

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $cache;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $configFactory;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $config;

  /**
   * The user storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $userStorage;

  /**
   * The visitor manager service.
   *
   * @var \Drupal\virtuous_personalization\VisitorManager
   */
  protected $visitorManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create mock objects
    $this->database = $this->prophesize(Connection::class);
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);
    $this->loggerFactory = $this->prophesize(LoggerChannelFactoryInterface::class);
    $this->logger = $this->prophesize(LoggerChannelInterface::class);
    $this->currentUser = $this->prophesize(AccountProxyInterface::class);
    $this->statistics = $this->prophesize(Statistics::class);
    $this->cache = $this->prophesize(CacheBackendInterface::class);
    $this->configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $this->config = $this->prophesize(ImmutableConfig::class);
    $this->userStorage = $this->prophesize(EntityStorageInterface::class);

    // Set up mock behavior
    $this->loggerFactory->get('virtuous_personalization')->willReturn($this->logger->reveal());
    $this->entityTypeManager->getStorage('user')->willReturn($this->userStorage->reveal());
    $this->configFactory->get('virtuous_personalization.settings')->willReturn($this->config->reveal());
    $this->config->get('cache_lifetime')->willReturn(3600);

    // Create the service
    $this->visitorManager = new VisitorManager(
      $this->database->reveal(),
      $this->entityTypeManager->reveal(),
      $this->loggerFactory->reveal(),
      $this->currentUser->reveal(),
      $this->statistics->reveal(),
      $this->cache->reveal(),
      $this->configFactory->reveal()
    );
  }

  /**
   * Tests the getUserByVisitorId method with cache hit.
   *
   * @covers ::getUserByVisitorId
   */
  public function testGetUserByVisitorIdCacheHit() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $uid = 42;
    $user = $this->prophesize(UserInterface::class);
    $user->id()->willReturn($uid);
    
    $cached_data = (object) [
      'data' => [
        'user' => $user->reveal(),
        'individual_id' => 'primary',
      ],
    ];

    // Set up cache hit
    $this->cache->get('virtuous_personalization:visitor:' . $visitor_id)
      ->willReturn($cached_data);
    
    // Logger should record the cache hit
    $this->logger->notice('Visitor data for ID @id retrieved from cache', [
      '@id' => $visitor_id,
    ])->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->getUserByVisitorId($visitor_id);

    // Assertions
    $this->assertSame($user->reveal(), $result['user']);
    $this->assertEquals('primary', $result['individual_id']);
  }

  /**
   * Tests the getUserByVisitorId method with cache miss (lookup by mapping).
   *
   * @covers ::getUserByVisitorId
   * @covers ::lookupUserByVisitorId
   */
  public function testGetUserByVisitorIdCacheMissLookupByMapping() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $uid = 42;
    $user = $this->prophesize(UserInterface::class);
    $user->id()->willReturn($uid);
    
    // Set up cache miss and database query
    $this->cache->get('virtuous_personalization:visitor:' . $visitor_id)
      ->willReturn(FALSE);
    
    // Mock database statement for mapping lookup
    $statement = $this->prophesize(\Drupal\Core\Database\StatementInterface::class);
    $statement->fetchAssoc()->willReturn([
      'uid' => $uid,
      'individual_id' => 'primary',
    ]);
    
    // Mock database query for mapping lookup
    $query = $this->prophesize(\Drupal\Core\Database\Query\SelectInterface::class);
    $query->execute()->willReturn($statement->reveal());
    
    // Set up database select expectation
    $this->database->select('virtuous_visitor_map', 'vvm')
      ->willReturn($query->reveal());
    $query->fields('vvm', ['uid', 'individual_id'])
      ->willReturn($query->reveal());
    $query->condition('vvm.visitor_id', $visitor_id)
      ->willReturn($query->reveal());
    
    // Set up user load
    $this->userStorage->load($uid)
      ->willReturn($user->reveal());
    
    // Set cache expectations
    $this->cache->set(
      'virtuous_personalization:visitor:' . $visitor_id,
      [
        'user' => $user->reveal(),
        'individual_id' => 'primary',
      ],
      $this->isType('integer'),
      [
        'virtuous_personalization',
        'virtuous_visitor',
        'user:' . $uid,
      ]
    )->shouldBeCalled();
    
    // Logger should record the cache debug message
    $this->logger->debug('Cached visitor data for ID @id', [
      '@id' => $visitor_id,
    ])->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->getUserByVisitorId($visitor_id);

    // Assertions
    $this->assertSame($user->reveal(), $result['user']);
    $this->assertEquals('primary', $result['individual_id']);
  }

  /**
   * Tests the getUserByVisitorId method with error.
   *
   * @covers ::getUserByVisitorId
   */
  public function testGetUserByVisitorIdWithError() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $exception_message = 'Database error';
    
    // Set up cache miss
    $this->cache->get('virtuous_personalization:visitor:' . $visitor_id)
      ->willReturn(FALSE);
    
    // Simulate a database exception
    $this->database->select('virtuous_visitor_map', 'vvm')
      ->willThrow(new \Exception($exception_message));
    
    // Logger should record the error
    $this->logger->error('Error finding user for visitor ID @id: @error', [
      '@id' => $visitor_id,
      '@error' => $exception_message,
    ])->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->getUserByVisitorId($visitor_id);

    // Assertions
    $this->assertNull($result);
  }

  /**
   * Tests the saveVisitorMapping method (create new mapping).
   *
   * @covers ::saveVisitorMapping
   */
  public function testSaveVisitorMappingCreate() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $uid = 42;
    $individual_id = 'primary';
    
    // Mock database query for existing check
    $statement = $this->prophesize(\Drupal\Core\Database\StatementInterface::class);
    $statement->fetchField()->willReturn(FALSE); // No existing mapping
    
    $query = $this->prophesize(\Drupal\Core\Database\Query\SelectInterface::class);
    $query->execute()->willReturn($statement->reveal());
    
    // Set up database select expectation for existing check
    $this->database->select('virtuous_visitor_map', 'vvm')
      ->willReturn($query->reveal());
    $query->fields('vvm', ['visitor_id'])
      ->willReturn($query->reveal());
    $query->condition('vvm.visitor_id', $visitor_id)
      ->willReturn($query->reveal());
    
    // Mock insert query
    $insert = $this->prophesize(\Drupal\Core\Database\Query\Insert::class);
    $this->database->insert('virtuous_visitor_map')
      ->willReturn($insert->reveal());
    
    // Set up insert expectations
    $insert->fields($this->anything())->willReturn($insert->reveal());
    $insert->execute()->willReturn(42); // Return insert ID
    
    // Statistics service should record the event
    $this->statistics->recordEvent('mapping', 'create', [
      'visitor_id' => $visitor_id,
      'uid' => $uid,
      'individual_id' => $individual_id,
    ])->shouldBeCalled();
    
    // Logger should record the creation
    $this->logger->notice('Created new visitor mapping: @id to @uid', [
      '@id' => $visitor_id, 
      '@uid' => $uid,
    ])->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $uid, $individual_id);

    // Assertions
    $this->assertTrue($result);
  }

  /**
   * Tests the saveVisitorMapping method (update existing mapping).
   *
   * @covers ::saveVisitorMapping
   */
  public function testSaveVisitorMappingUpdate() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $uid = 42;
    $individual_id = 'primary';
    
    // Mock database query for existing check
    $statement = $this->prophesize(\Drupal\Core\Database\StatementInterface::class);
    $statement->fetchField()->willReturn($visitor_id); // Existing mapping found
    
    $query = $this->prophesize(\Drupal\Core\Database\Query\SelectInterface::class);
    $query->execute()->willReturn($statement->reveal());
    
    // Set up database select expectation for existing check
    $this->database->select('virtuous_visitor_map', 'vvm')
      ->willReturn($query->reveal());
    $query->fields('vvm', ['visitor_id'])
      ->willReturn($query->reveal());
    $query->condition('vvm.visitor_id', $visitor_id)
      ->willReturn($query->reveal());
    
    // Mock update query
    $update = $this->prophesize(\Drupal\Core\Database\Query\Update::class);
    $this->database->update('virtuous_visitor_map')
      ->willReturn($update->reveal());
    
    // Set up update expectations
    $update->fields($this->anything())->willReturn($update->reveal());
    $update->condition('visitor_id', $visitor_id)->willReturn($update->reveal());
    $update->execute()->willReturn(1); // Return number of affected rows
    
    // Statistics service should record the event
    $this->statistics->recordEvent('mapping', 'update', [
      'visitor_id' => $visitor_id,
      'uid' => $uid,
      'individual_id' => $individual_id,
    ])->shouldBeCalled();
    
    // Logger should record the update
    $this->logger->notice('Updated existing visitor mapping: @id to @uid', [
      '@id' => $visitor_id, 
      '@uid' => $uid,
    ])->shouldBeCalled();
    
    // Cache should be invalidated
    $this->cache->invalidate('virtuous_personalization:visitor:' . $visitor_id)
      ->shouldBeCalled();
    $this->cache->invalidateTags(['user:' . $uid])
      ->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $uid, $individual_id);

    // Assertions
    $this->assertTrue($result);
  }

  /**
   * Tests the saveVisitorMapping method with error.
   *
   * @covers ::saveVisitorMapping
   */
  public function testSaveVisitorMappingWithError() {
    // Set up test data
    $visitor_id = 'test-visitor-id';
    $uid = 42;
    $individual_id = 'primary';
    $exception_message = 'Database error';
    
    // Simulate a database exception
    $this->database->select('virtuous_visitor_map', 'vvm')
      ->willThrow(new \Exception($exception_message));
    
    // Logger should record the error
    $this->logger->error('Failed to save visitor mapping: @error', [
      '@error' => $exception_message,
    ])->shouldBeCalled();

    // Execute the method
    $result = $this->visitorManager->saveVisitorMapping($visitor_id, $uid, $individual_id);

    // Assertions
    $this->assertFalse($result);
  }
}
