<?php

namespace Drupal\virtuous\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\virtuous\Service\VirtuousSyncService;
use Drupal\virtuous\Service\VirtuousApiService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Controller for Virtuous sync operations.
 */
class VirtuousSyncController extends ControllerBase {

  /**
   * The Virtuous sync service.
   *
   * @var \Drupal\virtuous\Service\VirtuousSyncService
   */
  protected $virtuousSyncService;

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeService;

  /**
   * Constructs a VirtuousSyncController object.
   *
   * @param \Drupal\virtuous\Service\VirtuousSyncService $virtuous_sync_service
   *   The Virtuous sync service.
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Component\Datetime\TimeInterface $time_service
   *   The time service.
   */
  public function __construct(
    VirtuousSyncService $virtuous_sync_service,
    VirtuousApiService $virtuous_api_service,
    MessengerInterface $messenger,
    TimeInterface $time_service
  ) {
    $this->virtuousSyncService = $virtuous_sync_service;
    $this->virtuousApiService = $virtuous_api_service;
    $this->messenger = $messenger;
    $this->timeService = $time_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('virtuous.sync_service'),
      $container->get('virtuous.api_service'),
      $container->get('messenger'),
      $container->get('datetime.time')
    );
  }

  /**
   * Syncs a user from Virtuous.
   *
   * @param int $user
   *   The user ID.
   * @param string $contact_id
   *   The Virtuous contact ID.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   A redirect response.
   */
  public function syncUser($user, $contact_id) {
    // Get full contact data from API
    $contact = $this->virtuousApiService->getContact($contact_id);
    
    if (!$contact) {
      $this->messenger()->addError($this->t('Failed to fetch contact @id from Virtuous.', [
        '@id' => $contact_id,
      ]));
      return $this->redirect('entity.user.edit_form', ['user' => $user]);
    }
    
    // Sync the contact
    $result = $this->virtuousSyncService->syncContact(['ContactId' => $contact_id]);
    
    // Update last sync time
    $this->state()->set('virtuous.last_sync_time', $this->timeService->getRequestTime());
    
    if ($result['success']) {
      $this->messenger()->addStatus($this->t('Contact @id has been synchronized.', [
        '@id' => $contact_id,
      ]));
    }
    else {
      $this->messenger()->addError($this->t('Failed to sync contact @id: @message', [
        '@id' => $contact_id,
        '@message' => $result['message'],
      ]));
    }
    
    // Redirect back to the user edit page
    return $this->redirect('entity.user.edit_form', ['user' => $user]);
  }

}
