<?php

namespace Drupal\virtuous\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\virtuous\Service\VirtuousSyncService;
use Drupal\virtuous\Service\VirtuousApiService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Controller for handling Virtuous webhook endpoints.
 */
class VirtuousWebhookController extends ControllerBase {

  /**
   * The Virtuous sync service.
   *
   * @var \Drupal\virtuous\Service\VirtuousSyncService
   */
  protected $virtuousSyncService;

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * Constructs a VirtuousWebhookController object.
   *
   * @param \Drupal\virtuous\Service\VirtuousSyncService $virtuous_sync_service
   *   The Virtuous sync service.
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   */
  public function __construct(
    VirtuousSyncService $virtuous_sync_service,
    VirtuousApiService $virtuous_api_service
  ) {
    $this->virtuousSyncService = $virtuous_sync_service;
    $this->virtuousApiService = $virtuous_api_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('virtuous.sync_service'),
      $container->get('virtuous.api_service')
    );
  }

  /**
   * Simple test endpoint.
   *
   * @return array
   *   A simple render array.
   */
  public function test() {
    return [
      '#markup' => 'VirtuousWebhookController test method works!',
    ];
  }

  /**
   * Validates webhook request authorization.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return bool
   *   TRUE if the webhook is authorized, FALSE otherwise.
   */
  protected function validateWebhookAuthorization(Request $request) {
    // Extract the security token from the request
    $content = json_decode($request->getContent(), TRUE);
    $token = $content['securityToken'] ?? '';
    
    // Validate the token
    if (empty($token) || !$this->virtuousApiService->validateWebhookToken($token)) {
      $this->getLogger('virtuous')->warning('Unauthorized webhook attempt detected.');
      return FALSE;
    }
    
    return TRUE;
  }

  /**
   * Webhook endpoint for contact created events.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response object.
   */
  public function contactCreated(Request $request) {
    // Authorization validation disabled
    
    $content = json_decode($request->getContent(), TRUE);
    
    // Extract contact ID from the correct location in the webhook payload
    $contact_id = $content['contact']['id'] ?? NULL;
    
    if (!$contact_id) {
      $this->getLogger('virtuous')->error('Missing contact ID in webhook payload: @payload', [
        '@payload' => print_r($content, TRUE),
      ]);
      return new JsonResponse(['error' => 'Missing contact ID'], 400);
    }

    // Log the webhook receipt
    $this->getLogger('virtuous')->info('Received contact created webhook for contact @id', [
      '@id' => $contact_id,
    ]);

    // Sync the contact
    $result = $this->virtuousSyncService->syncContact(['ContactId' => $contact_id]);
    
    return new JsonResponse($result);
  }

  /**
   * Webhook endpoint for contact updated events.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response object.
   */
  public function contactUpdated(Request $request) {
    // Authorization validation disabled
    
    $content = json_decode($request->getContent(), TRUE);
    
    // Extract contact ID from the correct location in the webhook payload
    $contact_id = $content['contact']['id'] ?? NULL;
    
    if (!$contact_id) {
      $this->getLogger('virtuous')->error('Missing contact ID in webhook payload: @payload', [
        '@payload' => print_r($content, TRUE),
      ]);
      return new JsonResponse(['error' => 'Missing contact ID'], 400);
    }

    // Log the webhook receipt with the full payload for debugging
    $this->getLogger('virtuous')->info('Received contact updated webhook for contact @id: @payload', [
      '@id' => $contact_id,
      '@payload' => json_encode($content),
    ]);

    // Sync the contact
    $result = $this->virtuousSyncService->syncContact(['ContactId' => $contact_id]);
    
    return new JsonResponse($result);
  }
}
