<?php

namespace Drupal\virtuous\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\key\KeyRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\virtuous\Service\VirtuousApiService;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;

/**
 * Configure Virtuous settings for this site.
 */
class VirtuousConfigForm extends ConfigFormBase {

  /**
   * The key repository.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructor for VirtuousConfigForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config_manager
   *   The typed config manager.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository service.
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config_manager,
    KeyRepositoryInterface $key_repository,
    VirtuousApiService $virtuous_api_service,
    StateInterface $state,
    DateFormatterInterface $date_formatter,
  ) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->keyRepository = $key_repository;
    $this->virtuousApiService = $virtuous_api_service;
    $this->state = $state;
    $this->dateFormatter = $date_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('key.repository'),
      $container->get('virtuous.api_service'),
      $container->get('state'),
      $container->get('date.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['virtuous.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'virtuous_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Attach the admin library with custom CSS.
    $form['#attached']['library'][] = 'virtuous/admin';
    
    $config = $this->config('virtuous.settings');

    $form['api'] = [
      '#type' => 'details',
      '#title' => $this->t('API Configuration'),
      '#open' => TRUE,
    ];

    $form['api']['api_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('API Mode'),
      '#description' => $this->t('Select "Test" mode while setting up the integration. Once you\'ve confirmed the connection is working, switch to "Live" mode.'),
      '#options' => [
        'test' => $this->t('Test'),
        'live' => $this->t('Live'),
      ],
      '#default_value' => $config->get('api_mode') ?: 'test',
      '#required' => TRUE,
    ];

    // Get all available keys.
    $key_options = [];
    $keys = $this->keyRepository->getKeys();
    foreach ($keys as $key) {
      $key_options[$key->id()] = $key->label();
    }

    $form['api']['api_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Virtuous API Key'),
      '#description' => $this->t('Select the key that contains your Virtuous API token. If you need to create a new key, <a href=":url">click here</a>.', [
        ':url' => Url::fromRoute('entity.key.add_form')->toString(),
      ]),
      '#options' => $key_options,
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
      '#empty_option' => $this->t('- Select a key -'),
    ];

    $form['api']['test_contact_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact ID for Testing'),
      '#description' => $this->t('Enter a valid contact ID from your Virtuous account to test the API connection.'),
      '#size' => 10,
      '#maxlength' => 10,
      '#states' => [
        'visible' => [
          ':input[name="api_key"]' => ['!value' => ''],
        ],
        'required' => [
          ':input[name="test_connection"]' => ['clicked' => TRUE],
        ],
      ],
    ];

    $form['api']['test_connection'] = [
      '#type' => 'submit',
      '#value' => $this->t('Test API Connection'),
      '#submit' => ['::testApiConnection'],
      '#limit_validation_errors' => [['api_key'], ['test_contact_id']],
      '#states' => [
        'disabled' => [
          ':input[name="api_key"]' => ['value' => ''],
        ],
      ],
    ];

    // Add API response display container
    $api_test_response = $form_state->get('api_test_response');
    if ($api_test_response) {
      $form['api']['response_container'] = [
        '#type' => 'details',
        '#title' => $this->t('API Response'),
        '#open' => TRUE,
        '#attributes' => [
          'class' => ['virtuous-api-response', 
                     $api_test_response['success'] ? 'virtuous-api-response--success' : 'virtuous-api-response--error'],
        ],
      ];
      
      $status_class = $api_test_response['success'] ? 'virtuous-api-response__status--success' : 'virtuous-api-response__status--error';
      
      $form['api']['response_container']['status'] = [
        '#type' => 'item',
        '#title' => $this->t('Status'),
        '#markup' => '<div class="virtuous-api-response__status ' . $status_class . '">' . 
                     ($api_test_response['success'] ? $this->t('Success') : $this->t('Error')) . 
                     '</div>',
      ];
      
      $form['api']['response_container']['message'] = [
        '#type' => 'item',
        '#title' => $this->t('Message'),
        '#markup' => '<div class="virtuous-api-response__message">' . $api_test_response['message'] . '</div>',
      ];
      
      if (!empty($api_test_response['status_code'])) {
        $form['api']['response_container']['status_code'] = [
          '#type' => 'item',
          '#title' => $this->t('Status Code'),
          '#markup' => '<div class="virtuous-api-response__code">' . (string) $api_test_response['status_code'] . '</div>',
        ];
      }
      
      // If there's a raw response, display it
      if (!empty($api_test_response['raw_response'])) {
        $form['api']['response_container']['raw'] = [
          '#type' => 'details',
          '#title' => $this->t('Raw Response'),
          '#open' => FALSE,
          '#attributes' => [
            'class' => ['virtuous-api-response__raw-container'],
          ],
        ];
        
        $form['api']['response_container']['raw']['content'] = [
          '#type' => 'markup',
          '#markup' => '<div class="virtuous-api-response__raw"><pre>' . 
                       htmlspecialchars(json_encode($api_test_response['raw_response'], JSON_PRETTY_PRINT)) . 
                       '</pre></div>',
        ];
      }
    }

    $form['sync'] = [
      '#type' => 'details',
      '#title' => $this->t('Synchronization Settings'),
      '#open' => TRUE,
    ];

    $form['sync']['sync_frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Automatic Sync Frequency'),
      '#description' => $this->t('How often should contacts be automatically synchronized from Virtuous?'),
      '#options' => [
        'never' => $this->t('Never (manual only)'),
        'daily' => $this->t('Daily'),
        'weekly' => $this->t('Weekly'),
        'monthly' => $this->t('Monthly'),
      ],
      '#default_value' => $config->get('sync_frequency') ?: 'never',
    ];

    // Display last sync time and stats.
    $last_sync_time = $this->state->get('virtuous.last_sync_time', 0);
    $sync_stats = $this->state->get('virtuous.sync_stats', []);

    if ($last_sync_time) {
      $form['sync']['last_sync'] = [
        '#type' => 'item',
        '#title' => $this->t('Last Synchronization'),
        '#markup' => $this->t('Last sync completed: @date', [
          '@date' => $this->dateFormatter->format($last_sync_time, 'medium'),
        ]),
      ];

      if (!empty($sync_stats)) {
        $form['sync']['sync_stats'] = [
          '#type' => 'item',
          '#title' => $this->t('Synchronization Statistics'),
          '#markup' => $this->t('Created: @created, Updated: @updated, Errors: @errors', [
            '@created' => $sync_stats['created'] ?? 0,
            '@updated' => $sync_stats['updated'] ?? 0,
            '@errors' => $sync_stats['errors'] ?? 0,
          ]),
        ];
      }
    }

    $form['sync']['sync_now'] = [
      '#type' => 'link',
      '#title' => $this->t('Sync Contacts Now'),
      '#url' => Url::fromRoute('virtuous.sync_now'),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];

    $form['webhook'] = [
      '#type' => 'details',
      '#title' => $this->t('Webhook Settings'),
      '#open' => TRUE,
    ];

    $form['webhook']['webhook_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Webhooks'),
      '#description' => $this->t('Allow Virtuous to send real-time updates to your Drupal site when contacts are created or updated.'),
      '#default_value' => $config->get('webhook_enabled') ?: FALSE,
    ];

    $form['webhook']['webhook_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Webhook Security Token'),
      '#description' => $this->t('This token will be used to validate webhook requests from Virtuous. Enter the same token in your Virtuous webhook configuration.'),
      '#default_value' => $config->get('webhook_token'),
      '#states' => [
        'visible' => [
          ':input[name="webhook_enabled"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name="webhook_enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Webhook URLs.
    $request = $this->getRequest();
    $base_url = $request->getSchemeAndHttpHost();

    $webhook_urls = [
      'created' => $base_url . Url::fromRoute('virtuous.webhook.contact_created')->toString(),
      'updated' => $base_url . Url::fromRoute('virtuous.webhook.contact_updated')->toString(),
    ];

    $form['webhook']['webhook_urls'] = [
      '#type' => 'details',
      '#title' => $this->t('Webhook Payload URLs'),
      '#description' => $this->t('Configure these URLs in your Virtuous account to receive real-time updates.'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="webhook_enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['webhook']['webhook_urls']['create_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Create'),
      '#value' => $webhook_urls['created'],
      '#attributes' => ['readonly' => 'readonly'],
    ];

    $form['webhook']['webhook_urls']['update_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contact Update'),
      '#value' => $webhook_urls['updated'],
      '#attributes' => ['readonly' => 'readonly'],
    ];

    $form['two_way_sync'] = [
      '#type' => 'details',
      '#title' => $this->t('Two-way Synchronization'),
      '#open' => TRUE,
    ];

    $form['two_way_sync']['two_way_sync_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable two-way synchronization'),
      '#description' => $this->t('When enabled, changes made to users in Drupal will be sent back to Virtuous.'),
      '#default_value' => $config->get('two_way_sync_enabled') ?: FALSE,
    ];

    $form['two_way_sync']['warning'] = [
      '#type' => 'markup',
      '#markup' => '<div class="messages messages--warning">' .
      $this->t('Two-way synchronization will send updates to Virtuous when Drupal user fields are modified. This could potentially overwrite data in Virtuous if both systems are updated simultaneously. Use with caution.') .
      '</div>',
      '#states' => [
        'visible' => [
          ':input[name="two_way_sync_enabled"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Submit handler for testing the API connection.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function testApiConnection(array &$form, FormStateInterface $form_state): void {
    $api_key_id = $form_state->getValue('api_key');
    $contact_id = $form_state->getValue('test_contact_id');

    if (empty($api_key_id)) {
      $this->messenger()->addError($this->t('Please select an API key before testing the connection.'));
      return;
    }

    if (empty($contact_id)) {
      $this->messenger()->addError($this->t('Please enter a contact ID to test the connection.'));
      return;
    }

    // Test the connection.
    $result = $this->virtuousApiService->testConnection($api_key_id, $contact_id);
    
    // Store the full response in temporary storage
    $form_state->set('api_test_response', $result);
    
    // Set a flag to rebuild the form to show the response
    $form_state->setRebuild(TRUE);

    if ($result['success']) {
      $this->messenger()->addStatus($this->t('Successfully connected to Virtuous API.'));
    }
    else {
      $this->messenger()->addError($this->t('Failed to connect to Virtuous API: @error', [
        '@error' => $result['message'],
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate webhook token if webhooks are enabled.
    if ($form_state->getValue('webhook_enabled') && empty($form_state->getValue('webhook_token'))) {
      $form_state->setErrorByName('webhook_token', $this->t('A security token is required when webhooks are enabled.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('virtuous.settings')
      ->set('api_mode', $form_state->getValue('api_mode'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('sync_frequency', $form_state->getValue('sync_frequency'))
      ->set('webhook_enabled', $form_state->getValue('webhook_enabled'))
      ->set('webhook_token', $form_state->getValue('webhook_token'))
      ->set('two_way_sync_enabled', $form_state->getValue('two_way_sync_enabled'))
      ->save();

    parent::submitForm($form, $form_state);
  }
}
