<?php

namespace Drupal\virtuous\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\virtuous\Service\VirtuousSyncService;
use Drupal\virtuous\Service\VirtuousApiService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;

/**
 * Form for manual synchronization of Virtuous contacts.
 */
class VirtuousSyncForm extends FormBase {

  /**
   * The Virtuous sync service.
   *
   * @var \Drupal\virtuous\Service\VirtuousSyncService
   */
  protected $virtuousSyncService;

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a VirtuousSyncForm object.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\virtuous\Service\VirtuousSyncService $virtuous_sync_service
   *   The Virtuous sync service.
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    TranslationInterface $string_translation,
    VirtuousSyncService $virtuous_sync_service,
    VirtuousApiService $virtuous_api_service,
    StateInterface $state,
    DateFormatterInterface $date_formatter,
    EntityTypeManagerInterface $entity_type_manager,
    MessengerInterface $messenger,
  ) {
    $this->setStringTranslation($string_translation);
    $this->virtuousSyncService = $virtuous_sync_service;
    $this->virtuousApiService = $virtuous_api_service;
    $this->state = $state;
    $this->dateFormatter = $date_formatter;
    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('string_translation'),
          $container->get('virtuous.sync_service'),
          $container->get('virtuous.api_service'),
          $container->get('state'),
          $container->get('date.formatter'),
          $container->get('entity_type.manager'),
          $container->get('messenger')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'virtuous_sync_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $user = NULL) {
    // Display last sync information.
    $last_sync_time = $this->state->get('virtuous.last_sync_time', 0);
    $sync_stats = $this->state->get('virtuous.sync_stats', []);

    $form['sync_status'] = [
      '#theme' => 'virtuous_sync_status',
      '#sync_status' => !empty($last_sync_time) ? 'complete' : 'never',
      '#last_sync' => !empty($last_sync_time) ? $this->dateFormatter->format($last_sync_time, 'medium') : NULL,
      '#sync_count' => $sync_stats,
    ];

    $form['sync_warning'] = [
      '#type' => 'markup',
      '#markup' => '<div class="messages messages--warning">' . $this->t('This process may take some time depending on the number of contacts in your Virtuous account. Please do not close this page once synchronization has started.') . '</div>',
    ];

    // If a specific user is provided, show user-specific sync option.
    if ($user) {
      $user_entity = $this->entityTypeManager->getStorage('user')->load($user);

      if ($user_entity && !$user_entity->get('contact_id')->isEmpty()) {
        $contact_id = $user_entity->get('contact_id')->value;
        $contact_name = $user_entity->get('contact_name')->value ?: $user_entity->getAccountName();

        $form['user_sync'] = [
          '#type' => 'details',
          '#title' => $this->t('Sync Individual Contact'),
          '#open' => TRUE,
        ];

        $form['user_sync']['info'] = [
          '#markup' => $this->t(
              'You are about to synchronize the contact information for user @name (Contact ID: @id).', [
                '@name' => $contact_name,
                '@id' => $contact_id,
              ]
          ),
        ];

        $form['user_sync']['contact_id'] = [
          '#type' => 'hidden',
          '#value' => $contact_id,
        ];

        $form['user_sync']['sync_user'] = [
          '#type' => 'submit',
          '#value' => $this->t('Sync This Contact Only'),
          '#submit' => ['::submitSyncUser'],
          '#attributes' => [
            'class' => ['button', 'button--primary'],
          ],
        ];
      }
    }

    // Add section for syncing a single contact by ID
    $form['single_contact'] = [
      '#type' => 'details',
      '#title' => $this->t('Sync Individual Contact'),
      '#open' => TRUE,
    ];

    $form['single_contact']['contact_id_input'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Virtuous Contact ID'),
      '#description' => $this->t('Enter the Virtuous Contact ID to synchronize a single contact.'),
      '#size' => 20,
    ];

    $form['single_contact']['sync_single'] = [
      '#type' => 'submit',
      '#value' => $this->t('Sync This Contact Only'),
      '#submit' => ['::submitSyncSingleContact'],
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Sync All Contacts'),
      '#button_type' => 'primary',
    ];

    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => \Drupal::destination()->get(),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Sync all contacts with batch processing.
    $this->virtuousSyncService->syncContacts(TRUE);
  }

  /**
   * Submit handler for syncing a single contact by ID.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitSyncSingleContact(array &$form, FormStateInterface $form_state) {
    $contact_id = $form_state->getValue('contact_id_input');
    
    if (empty($contact_id)) {
      $this->messenger->addError($this->t('Please enter a valid Contact ID.'));
      return;
    }
    
    // Get full contact data from API
    $contact = $this->virtuousApiService->getContact($contact_id);
    
    if (!$contact) {
      $this->messenger->addError($this->t('Failed to fetch contact @id from Virtuous.', ['@id' => $contact_id]));
      return;
    }
    
    // Initialize sync stats
    $this->state->set(
      'virtuous.sync_stats', [
        'created' => 0,
        'updated' => 0,
        'errors' => 0,
      ]
    );
    
    // Sync the contact
    $result = $this->virtuousSyncService->syncContact(['ContactId' => $contact_id]);
    
    // Update last sync time
    $this->state->set('virtuous.last_sync_time', \Drupal::time()->getRequestTime());
    
    if ($result['success']) {
      $this->messenger->addStatus($this->t('Contact @id has been synchronized.', ['@id' => $contact_id]));
    }
    else {
      $this->messenger->addError($this->t('Failed to sync contact @id: @message', [
        '@id' => $contact_id,
        '@message' => $result['message'],
      ]));
    }
  }

  /**
   * Submit handler for syncing a single user.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitSyncUser(array &$form, FormStateInterface $form_state): void {
    $contact_id = $form_state->getValue('contact_id');

    if ($contact_id) {
      // Get full contact data from API.
      $contact = $this->virtuousApiService->getContact($contact_id);

      if ($contact) {
        // Initialize sync stats.
        $this->state->set(
              'virtuous.sync_stats', [
                'created' => 0,
                'updated' => 0,
                'errors' => 0,
              ]
          );

        // Sync single contact.
        $result = $this->virtuousSyncService->syncContact(['ContactId' => $contact_id]);

        // Update last sync time.
        $this->state->set('virtuous.last_sync_time', \Drupal::time()->getRequestTime());

        if ($result['success']) {
          $this->messenger->addStatus(
            $this->t('Contact @id has been synchronized.', ['@id' => $contact_id])
          );
        }
        else {
          $this->messenger->addError(
            $this->t('Failed to sync contact @id: @message', [
              '@id' => $contact_id,
              '@message' => $result['message'],
            ])
          );
        }
      }
      else {
        $this->messenger->addError(
              $this->t(
                  'Failed to fetch contact @id from Virtuous.', [
                    '@id' => $contact_id,
                  ]
              )
          );
      }
    }
  }

}
