<?php

namespace Drupal\virtuous\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\key\KeyRepositoryInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Service for interacting with the Virtuous API.
 */
class VirtuousApiService {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The key repository.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Base URL for the Virtuous API.
   *
   * @var string
   */
  protected $apiBaseUrl = 'https://api.virtuoussoftware.com/';

  /**
   * Constructs a VirtuousApiService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    KeyRepositoryInterface $key_repository,
    LoggerChannelFactoryInterface $logger_factory,
    ClientInterface $http_client,
  ) {
    $this->configFactory = $config_factory;
    $this->keyRepository = $key_repository;
    $this->logger = $logger_factory->get('virtuous');
    $this->httpClient = $http_client;
  }

  /**
   * Get the API key.
   *
   * @return string|null
   *   The API key or NULL if it cannot be retrieved.
   */
  protected function getApiKey(): ?string {
    $config = $this->configFactory->get('virtuous.settings');
    $key_id = $config->get('api_key');

    if (empty($key_id)) {
      $this->logger->error('No Virtuous API key configured.');
      return NULL;
    }

    $key = $this->keyRepository->getKey($key_id);
    if (!$key) {
      $this->logger->error('Virtuous API key not found: @key_id', ['@key_id' => $key_id]);
      return NULL;
    }

    return $key->getKeyValue();
  }

  /**
   * Test the connection to the Virtuous API.
   *
   * @param string|null $api_key_id
   *   (optional) The key ID to use for testing. If not provided, the configured
   *   key will be used.
   * @param string|null $contact_id
   *   (optional) The contact ID to use for testing.
   *
   * @return array
   *   An associative array with keys:
   *   - success: (bool) TRUE if the connection was successful, FALSE otherwise.
   *   - message: (string) A message describing the result.
   *   - raw_response: (array|null) The raw response data if available.
   */
  public function testConnection(?string $api_key_id = NULL, ?string $contact_id = NULL): array {
    if ($api_key_id) {
      $key = $this->keyRepository->getKey($api_key_id);
      $api_key = $key ? $key->getKeyValue() : NULL;
    }
    else {
      $api_key = $this->getApiKey();
    }

    if (empty($api_key)) {
      return [
        'success' => FALSE,
        'message' => 'No API key available.',
        'raw_response' => NULL,
      ];
    }

    if (empty($contact_id)) {
      return [
        'success' => FALSE,
        'message' => 'No contact ID provided for testing.',
        'raw_response' => NULL,
      ];
    }

    try {
      $response = $this->httpClient->request('GET', $this->apiBaseUrl . 'api/Contact/' . $contact_id, [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $api_key,
        ],
      ]);

      $body = (string) $response->getBody();
      $raw_response = json_decode($body, TRUE);

      if ($response->getStatusCode() == 200) {
        return [
          'success' => TRUE,
          'message' => 'Connection successful.',
          'raw_response' => $raw_response,
          'status_code' => $response->getStatusCode(),
        ];
      }
      else {
        return [
          'success' => FALSE,
          'message' => 'Unexpected status code: ' . $response->getStatusCode(),
          'raw_response' => $raw_response,
          'status_code' => $response->getStatusCode(),
        ];
      }
    }
    catch (RequestException $e) {
      $this->logger->error('Error testing Virtuous API connection: @error', [
        '@error' => $e->getMessage(),
      ]);

      return [
        'success' => FALSE,
        'message' => $e->getMessage(),
        'raw_response' => NULL,
        'status_code' => $e->hasResponse() ? $e->getResponse()->getStatusCode() : NULL,
      ];
    }
  }

  /**
   * Fetches contacts from the Virtuous API.
   *
   * @param array $params
   *   Query parameters to filter the results.
   * @param int $page
   *   The page number to fetch.
   * @param int $pageSize
   *   The number of contacts per page.
   *
   * @return array|null
   *   An array of contacts or NULL if the request failed.
   */
  public function getContacts(array $params = [], int $page = 1, int $pageSize = 100): ?array {
    $api_key = $this->getApiKey();
    if (empty($api_key)) {
      return NULL;
    }

    $query = [
      'page' => $page,
      'pageSize' => $pageSize,
    ] + $params;

    try {
      $response = $this->httpClient->request('GET', $this->apiBaseUrl . 'api/Contact/Query', [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $api_key,
        ],
        'query' => $query,
      ]);

      if ($response->getStatusCode() == 200) {
        $data = json_decode((string) $response->getBody(), TRUE);
        return $data;
      }
      else {
        $this->logger->error('Virtuous API returned status code @code', [
          '@code' => $response->getStatusCode(),
        ]);
        return NULL;
      }
    }
    catch (RequestException $e) {
      $this->logger->error('Error fetching contacts from Virtuous API: @error', [
        '@error' => $e->getMessage(),
      ]);

      return NULL;
    }
  }

  /**
   * Fetches a single contact from the Virtuous API by ID.
   *
   * @param int $contact_id
   *   The Virtuous contact ID.
   *
   * @return array|null
   *   The contact data or NULL if the request failed.
   */
  public function getContact(int $contact_id): ?array {
    $api_key = $this->getApiKey();
    if (empty($api_key)) {
      return NULL;
    }

    try {
      $response = $this->httpClient->request('GET', $this->apiBaseUrl . 'api/Contact/' . $contact_id, [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $api_key,
        ],
      ]);

      if ($response->getStatusCode() == 200) {
        $data = json_decode((string) $response->getBody(), TRUE);
        return $data;
      }
      else {
        $this->logger->error('Virtuous API returned status code @code when fetching contact @id', [
          '@code' => $response->getStatusCode(),
          '@id' => $contact_id,
        ]);
        return NULL;
      }
    }
    catch (RequestException $e) {
      $this->logger->error('Error fetching contact @id from Virtuous API: @error', [
        '@id' => $contact_id,
        '@error' => $e->getMessage(),
      ]);

      return NULL;
    }
  }

  /**
   * Fetches total contact count from the Virtuous API.
   *
   * @return int|null
   *   The total number of contacts or NULL if the request failed.
   */
  public function getContactCount(): ?int {
    $api_key = $this->getApiKey();
    if (empty($api_key)) {
      return NULL;
    }

    try {
      $response = $this->httpClient->request('GET', $this->apiBaseUrl . 'api/Contact/Count', [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $api_key,
        ],
      ]);

      if ($response->getStatusCode() == 200) {
        $data = json_decode((string) $response->getBody(), TRUE);
        return $data['count'] ?? 0;
      }
      else {
        $this->logger->error('Virtuous API returned status code @code when fetching contact count', [
          '@code' => $response->getStatusCode(),
        ]);
        return NULL;
      }
    }
    catch (RequestException $e) {
      $this->logger->error('Error fetching contact count from Virtuous API: @error', [
        '@error' => $e->getMessage(),
      ]);

      return NULL;
    }
  }

  /**
   * Validates a webhook request.
   *
   * @param string $token
   *   The token from the webhook request.
   *
   * @return bool
   *   TRUE if the token is valid, FALSE otherwise.
   */
  public function validateWebhookToken(string $token): bool {
    $config = $this->configFactory->get('virtuous.settings');
    $stored_token = $config->get('webhook_token');

    if (empty($stored_token) || empty($token)) {
      return FALSE;
    }

    // Use hash_equals for timing-safe comparison to prevent timing attacks.
    return hash_equals($stored_token, $token);
  }

  /**
   * Updates a contact in Virtuous using the Contact Transaction API.
   *
   * @param array $contact_data
   *   The contact data to send to Virtuous.
   *
   * @return array
   *   The response from the Virtuous API.
   */
  public function updateContactInVirtuous(array $contact_data): array {
    $api_key = $this->getApiKey();
    if (empty($api_key)) {
      return [
        'success' => FALSE,
        'message' => 'No API key available.',
      ];
    }

    try {
      $response = $this->httpClient->request('POST', $this->apiBaseUrl . 'api/Contact/Transaction', [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $api_key,
        ],
        'json' => $contact_data,
      ]);

      if ($response->getStatusCode() == 200) {
        $data = json_decode((string) $response->getBody(), TRUE);
        return [
          'success' => TRUE,
          'data' => $data,
        ];
      }
      else {
        $this->logger->error('Virtuous API returned status code @code when updating contact', [
          '@code' => $response->getStatusCode(),
        ]);
        return [
          'success' => FALSE,
          'message' => 'Unexpected status code: ' . $response->getStatusCode(),
        ];
      }
    }
    catch (RequestException $e) {
      $this->logger->error('Error updating contact in Virtuous: @error', [
        '@error' => $e->getMessage(),
      ]);

      return [
        'success' => FALSE,
        'message' => $e->getMessage(),
      ];
    }
  }

}
