<?php

namespace Drupal\virtuous\Service;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Queue\QueueInterface;
use Drupal\virtuous\VirtuousConstants;

/**
 * Service for syncing data with Virtuous.
 */
class VirtuousSyncService {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The queue service.
   *
   * @var object
   */
  protected $queueService;

  /**
   * Constructs a new VirtuousSyncService.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger channel factory.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   * @param object $queue_service
   *   The queue service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    Connection $connection,
    MessengerInterface $messenger,
    StateInterface $state,
    VirtuousApiService $virtuous_api_service,
    $queue_service
  ) {
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->state = $state;
    $this->virtuousApiService = $virtuous_api_service;
    $this->queueService = $queue_service;
  }

  /**
   * Simple test method to verify service works.
   */
  public function test() {
    return 'VirtuousSyncService is working';
  }

  /**
   * Syncs all contacts from Virtuous using batch processing.
   *
   * @param bool $is_manual
   *   TRUE if this is a manual sync, FALSE if automated.
   */
  public function syncContacts($is_manual = FALSE) {
    // Initialize sync stats.
    $this->state->set('virtuous.sync_stats', [
      'created' => 0,
      'updated' => 0,
      'errors' => 0,
    ]);

    // Get the total number of contacts from Virtuous.
    $total_contacts = $this->virtuousApiService->getContactCount();
    
    if ($total_contacts === NULL) {
      $this->messenger->addError($this->t('Could not retrieve contact count from Virtuous API.'));
      $this->loggerFactory->get('virtuous')->error('Failed to retrieve contact count during sync.');
      return;
    }
    
    // If no contacts are available, end early.
    if ($total_contacts === 0) {
      $this->messenger->addStatus($this->t('No contacts found in Virtuous.'));
      $this->state->set('virtuous.last_sync_time', \Drupal::time()->getRequestTime());
      return;
    }

    // Determine batch size.
    $batch_size = VirtuousConstants::SYNC_BATCH_SIZE;
    $num_batches = ceil($total_contacts / $batch_size);
    
    // Create a new batch.
    $batch_builder = new BatchBuilder();
    $batch_builder
      ->setTitle($this->t('Synchronizing @total contacts from Virtuous', ['@total' => $total_contacts]))
      ->setInitMessage($this->t('Starting contact synchronization...'))
      ->setProgressMessage($this->t('Processed @current out of @total.'))
      ->setErrorMessage($this->t('An error occurred during synchronization.'));
    
    // Add operations to the batch.
    for ($i = 0; $i < $num_batches; $i++) {
      $page = $i + 1; // API pages are 1-indexed
      $batch_builder->addOperation(
        [$this, 'processBatch'],
        [$page, $batch_size, $is_manual]
      );
    }
    
    // Add a finished callback.
    $batch_builder->setFinishCallback([$this, 'finishBatch']);
    
    // Set the batch.
    batch_set($batch_builder->toArray());
    
    // Process the batch immediately if this is called programmatically.
    if (!$is_manual) {
      drush_backend_batch_process();
    }
  }

  /**
   * Process a batch of contacts.
   *
   * @param int $page
   *   The page number to fetch.
   * @param int $batch_size
   *   The batch size.
   * @param bool $is_manual
   *   Whether this is a manual sync.
   * @param array $context
   *   The batch context.
   */
  public function processBatch($page, $batch_size, $is_manual, &$context) {
    // Initialize the results array if it doesn't exist.
    if (!isset($context['results']['stats'])) {
      $context['results']['stats'] = [
        'created' => 0,
        'updated' => 0,
        'errors' => 0,
      ];
    }
    
    // Get a batch of contacts from the API.
    $contacts = $this->virtuousApiService->getContacts([], $page, $batch_size);
    
    if (!$contacts) {
      $context['results']['stats']['errors']++;
      $context['message'] = $this->t('Failed to retrieve contacts from page @page', ['@page' => $page]);
      $this->loggerFactory->get('virtuous')->error('Failed to retrieve contacts from page @page', ['@page' => $page]);
      return;
    }
    
    // Process each contact.
    foreach ($contacts as $contact) {
      $result = $this->processContact($contact);
      
      // Update stats based on result.
      if ($result['success']) {
        if ($result['action'] === 'created') {
          $context['results']['stats']['created']++;
        }
        elseif ($result['action'] === 'updated') {
          $context['results']['stats']['updated']++;
        }
      }
      else {
        $context['results']['stats']['errors']++;
      }
    }
    
    // Update the progress information.
    $context['message'] = $this->t('Processed @count contacts from page @page', [
      '@count' => count($contacts),
      '@page' => $page,
    ]);
  }

  /**
   * Finish callback for the batch operation.
   *
   * @param bool $success
   *   Indicates whether the batch process was successful.
   * @param array $results
   *   The results from the batch process.
   * @param array $operations
   *   The operations that were processed.
   */
  public function finishBatch($success, $results, $operations) {
    if ($success) {
      // Update the sync statistics.
      $this->state->set('virtuous.sync_stats', $results['stats']);
      
      // Update the last sync time.
      $this->state->set('virtuous.last_sync_time', \Drupal::time()->getRequestTime());
      
      // Display a message to the user.
      $this->messenger->addStatus($this->t('Synchronization completed. Created: @created, Updated: @updated, Errors: @errors', [
        '@created' => $results['stats']['created'],
        '@updated' => $results['stats']['updated'],
        '@errors' => $results['stats']['errors'],
      ]));
      
      // Log the completion.
      $this->loggerFactory->get('virtuous')->info('Virtuous synchronization completed. Created: @created, Updated: @updated, Errors: @errors', [
        '@created' => $results['stats']['created'],
        '@updated' => $results['stats']['updated'],
        '@errors' => $results['stats']['errors'],
      ]);
    }
    else {
      // Display an error message.
      $this->messenger->addError($this->t('An error occurred during synchronization.'));
      
      // Log the error.
      $this->loggerFactory->get('virtuous')->error('Virtuous synchronization failed.');
    }
  }

  /**
   * Process a single contact.
   *
   * @param array $contact
   *   The contact data from Virtuous.
   *
   * @return array
   *   An array with the processing result.
   */
  protected function processContact(array $contact) {
    try {
      // Process the contact data for field mapping.
      $processed_data = virtuous_process_contact_data($contact);
      
      $user = NULL;
      $existing_users_found = FALSE;
      
      // First check for existing user with the contact_id.
      $users = $this->entityTypeManager->getStorage('user')
        ->loadByProperties(['contact_id' => $processed_data['contact_id']]);
      
      $user = reset($users);
      
      if ($user) {
        $this->loggerFactory->get('virtuous')->info('Found user by contact_id: @contact_id (User ID: @uid)', [
          '@contact_id' => $processed_data['contact_id'],
          '@uid' => $user->id(),
        ]);
        $existing_users_found = TRUE;
      }
      
      // If no user found by contact_id, try finding by primary_id.
      if (!$user && !empty($processed_data['primary_id'])) {
        $primary_users = $this->entityTypeManager->getStorage('user')
          ->loadByProperties(['primary_id' => $processed_data['primary_id']]);
        
        $user = reset($primary_users);
        
        if ($user) {
          $this->loggerFactory->get('virtuous')->info('Found user by primary_id: @primary_id (User ID: @uid)', [
            '@primary_id' => $processed_data['primary_id'],
            '@uid' => $user->id(),
          ]);
          $existing_users_found = TRUE;
        }
      }
      
      // If still no user found, try finding by secondary_id.
      if (!$user && !empty($processed_data['secondary_id'])) {
        $secondary_users = $this->entityTypeManager->getStorage('user')
          ->loadByProperties(['secondary_id' => $processed_data['secondary_id']]);
        
        $user = reset($secondary_users);
        
        if ($user) {
          $this->loggerFactory->get('virtuous')->info('Found user by secondary_id: @secondary_id (User ID: @uid)', [
            '@secondary_id' => $processed_data['secondary_id'],
            '@uid' => $user->id(),
          ]);
          $existing_users_found = TRUE;
        }
      }
      
      // If still no user found, try finding by email.
      if (!$user && !empty($processed_data['mail'])) {
        $email_users = $this->entityTypeManager->getStorage('user')
          ->loadByProperties(['mail' => $processed_data['mail']]);
        
        $user = reset($email_users);
        
        if ($user) {
          $this->loggerFactory->get('virtuous')->info('Found user by email: @email (User ID: @uid)', [
            '@email' => $processed_data['mail'],
            '@uid' => $user->id(),
          ]);
          $existing_users_found = TRUE;
        }
      }
      
      if ($user) {
        // Check if any relevant data has actually changed
        $has_changes = FALSE;
        $changed_fields = [];
        
        // Primary email change requires special handling
        $primary_email_updated = FALSE;
        if (!empty($processed_data['primary_email_value']) && 
            $user->getEmail() !== $processed_data['primary_email_value'] &&
            !empty($processed_data['primary_email_is_primary'])) {
          
          $has_changes = TRUE;
          $changed_fields[] = 'mail';
          $primary_email_updated = TRUE;
        }
        
        // Check all other fields for changes
        foreach ($processed_data as $field_name => $new_value) {
          // Skip fields that don't exist on the user entity
          if (!$user->hasField($field_name)) {
            continue;
          }
          
          // Skip empty values
          if ($new_value === NULL || $new_value === '') {
            continue;
          }
          
          // Handle different field types appropriately
          if (in_array($field_name, ['contact_type', 'contact_marital_status', 'primary_gender', 'secondary_gender'])) {
            // For taxonomy term reference fields, compare target_id
            $current_value = $user->get($field_name)->target_id;
            if ($current_value != $new_value) {
              $has_changes = TRUE;
              $changed_fields[] = $field_name;
            }
          }
          else if ($user->get($field_name)->isEmpty()) {
            // If the field is currently empty but we have a new value
            $has_changes = TRUE;
            $changed_fields[] = $field_name;
          }
          else {
            // Regular field comparison
            $current_value = $user->get($field_name)->value;
            if ((string)$current_value !== (string)$new_value) {
              $has_changes = TRUE;
              $changed_fields[] = $field_name;
            }
          }
        }
        
        // If no relevant changes detected, log and skip update
        if (!$has_changes) {
          $this->loggerFactory->get('virtuous')->info('No data changes detected for contact ID: @id, user ID: @uid - skipping update', [
            '@id' => $processed_data['contact_id'],
            '@uid' => $user->id(),
          ]);
          
          return [
            'success' => TRUE,
            'action' => 'skipped',
            'message' => 'No data changes detected - update skipped',
          ];
        }
        
        // Log the fields that changed
        $this->loggerFactory->get('virtuous')->notice('Data changes detected for user @uid (contact @id). Changed fields: @fields', [
          '@uid' => $user->id(),
          '@id' => $processed_data['contact_id'],
          '@fields' => implode(', ', $changed_fields),
        ]);
        
        // Update the user since we detected changes
        foreach ($processed_data as $field_name => $value) {
          if ($user->hasField($field_name)) {
            // For taxonomy term reference fields, ensure they're set correctly
            if (in_array($field_name, ['contact_type', 'contact_marital_status', 'primary_gender', 'secondary_gender']) && !empty($value)) {
              $user->set($field_name, ['target_id' => $value]);
            }
            else {
              $user->set($field_name, $value);
            }
          }
        }
        
        // If primary email changed, update user's mail field
        if ($primary_email_updated) {
          $user->setEmail($processed_data['primary_email_value']);
        }
        
        $user->save();
        
        return [
          'success' => TRUE,
          'action' => 'updated',
          'message' => 'User updated: ' . $user->id() . '. Changed fields: ' . implode(', ', $changed_fields),
        ];
      }
      else if ($existing_users_found) {
        // If we found existing users but couldn't load one properly, log it
        $this->loggerFactory->get('virtuous')->warning('Inconsistent data: Found related user records but could not load a valid user entity for contact @id', [
          '@id' => $processed_data['contact_id'],
        ]);
        
        return [
          'success' => FALSE,
          'message' => 'Inconsistent data: Found user records but could not load a valid user entity',
        ];
      }
      else {
        // Only create new user if we have an email AND couldn't find any existing user
        if (!empty($processed_data['mail'])) {
          // Double check no user exists with this email to prevent duplicates
          $existing_email_check = $this->entityTypeManager->getStorage('user')
            ->loadByProperties(['mail' => $processed_data['mail']]);
          
          if (!empty($existing_email_check)) {
            $this->loggerFactory->get('virtuous')->warning('Attempted to create duplicate user with email @email for contact @id. Existing user found.', [
              '@email' => $processed_data['mail'],
              '@id' => $processed_data['contact_id'],
            ]);
            
            return [
              'success' => FALSE,
              'message' => 'Cannot create user: User with this email already exists',
            ];
          }
          
          $user = $this->entityTypeManager->getStorage('user')->create([
            'name' => $processed_data['mail'],
            'mail' => $processed_data['mail'],
            'status' => 1,
          ]);
          
          // Set all the fields from processed data.
          foreach ($processed_data as $field_name => $value) {
            if ($user->hasField($field_name)) {
              // For taxonomy term reference fields, ensure they're set correctly
              if (in_array($field_name, ['contact_type', 'contact_marital_status', 'primary_gender', 'secondary_gender']) && !empty($value)) {
                $user->set($field_name, ['target_id' => $value]);
              }
              else {
                $user->set($field_name, $value);
              }
            }
          }
          
          // Assign the Virtuous Contact role
          $user->addRole('virtuous_contact');
          
          $user->save();
          
          $this->loggerFactory->get('virtuous')->notice('Created new user @uid with "Virtuous Contact" role for contact @id', [
            '@uid' => $user->id(),
            '@id' => $processed_data['contact_id'],
          ]);
          
          return [
            'success' => TRUE,
            'action' => 'created',
            'message' => 'User created: ' . $user->id() . ' with "Virtuous Contact" role',
          ];
        }
        else {
          // No primary email available for primary individual, log this as an error.
          $this->loggerFactory->get('virtuous')->warning('Cannot create user for contact @id: Primary individual has no primary email address.', [
            '@id' => $processed_data['contact_id'],
          ]);
          
          return [
            'success' => FALSE,
            'message' => 'Cannot create user: Primary individual has no primary email address.',
          ];
        }
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('virtuous')->error('Error processing contact @id: @message', [
        '@id' => $contact['id'] ?? 'unknown',
        '@message' => $e->getMessage(),
      ]);
      
      return [
        'success' => FALSE,
        'message' => 'Error: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Syncs a contact from Virtuous.
   *
   * @param array $webhook_data
   *   The webhook data from Virtuous.
   *
   * @return array
   *   Result information.
   */
  public function syncContact(array $webhook_data) {
    try {
      // Extract Contact ID from webhook data.
      $contact_id = $webhook_data['ContactId'] ?? NULL;
      if (!$contact_id) {
        $this->loggerFactory->get('virtuous')->error('No Contact ID found in webhook data.');
        return [
          'success' => FALSE,
          'message' => 'No Contact ID found in webhook data.',
        ];
      }

      // Log the contact ID
      $this->loggerFactory->get('virtuous')->info('Processing contact with ID: @id', ['@id' => $contact_id]);
      
      // Get the full contact data from the API
      $contact = $this->virtuousApiService->getContact($contact_id);
      
      if (!$contact) {
        $this->loggerFactory->get('virtuous')->error('Could not retrieve contact @id from API.', ['@id' => $contact_id]);
        return [
          'success' => FALSE,
          'message' => "Could not retrieve contact {$contact_id} from API.",
        ];
      }
      
      // Process the contact
      $result = $this->processContact($contact);
      
      // Update sync stats
      $stats = $this->state->get('virtuous.sync_stats', [
        'created' => 0,
        'updated' => 0,
        'errors' => 0,
      ]);
      
      if ($result['success']) {
        if ($result['action'] === 'created') {
          $stats['created']++;
        }
        elseif ($result['action'] === 'updated') {
          $stats['updated']++;
        }
      }
      else {
        $stats['errors']++;
      }
      
      $this->state->set('virtuous.sync_stats', $stats);
      
      return $result;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('virtuous')->error('Error syncing contact: @message', ['@message' => $e->getMessage()]);
      return [
        'success' => FALSE,
        'message' => 'Error syncing contact: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Handles a webhook request from Virtuous.
   *
   * @param array $data
   *   The webhook data.
   *
   * @return array
   *   An array containing the result of the webhook processing.
   */
  public function handleWebhook(array $data) {
    $result = [
      'success' => FALSE,
      'message' => '',
    ];
    
    try {
      // Log the received data for debugging
      $this->loggerFactory->get('virtuous')->info('Received webhook data: @data', [
        '@data' => json_encode($data),
      ]);
      
      // Extract event type from the payload
      $event_type = $data['event'] ?? NULL;
      
      // Only process ContactCreate and ContactUpdate events
      if ($event_type === 'ContactCreate' || $event_type === 'ContactUpdate') {
        // Extract contact ID from the payload
        $contact_id = $data['contact']['id'] ?? NULL;
        
        if (!$contact_id) {
          $result['message'] = 'No Contact ID found in webhook data';
          $this->loggerFactory->get('virtuous')->error('Missing contact ID in webhook payload: @payload', [
            '@payload' => json_encode($data),
          ]);
          return $result;
        }
        
        // Process the contact
        $this->loggerFactory->get('virtuous')->info('Processing @event event for contact ID: @id', [
          '@event' => $event_type,
          '@id' => $contact_id,
        ]);
        
        $result = $this->syncContact(['ContactId' => $contact_id]);
      } 
      else {
        // For all other event types, return success but don't process
        $this->loggerFactory->get('virtuous')->info('Received non-contact event type: @type - not processing', [
          '@type' => $event_type,
        ]);
        
        $result['success'] = TRUE;
        $result['message'] = 'Non-contact event ignored: ' . $event_type;
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('virtuous')
        ->error('Error handling webhook: @message', ['@message' => $e->getMessage()]);
      $result['message'] = 'Error processing webhook: ' . $e->getMessage();
    }
    
    return $result;
  }
}
