<?php

namespace Drupal\virtuous\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service for two-way synchronization between Drupal and Virtuous.
 */
class VirtuousTwoWaySyncService {

  /**
   * The Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a VirtuousTwoWaySyncService object.
   *
   * @param \Drupal\virtuous\Service\VirtuousApiService $virtuous_api_service
   *   The Virtuous API service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    VirtuousApiService $virtuous_api_service,
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    $this->virtuousApiService = $virtuous_api_service;
    $this->configFactory = $config_factory;
    $this->logger = $logger_factory->get('virtuous');
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Handles user update to sync data back to Virtuous.
   *
   * @param \Drupal\user\UserInterface $user
   *   The updated user entity.
   *
   * @return bool
   *   TRUE if the update was successful, FALSE otherwise.
   */
  public function sendUserUpdatesToVirtuous(UserInterface $user): bool {
    // Only sync if two-way sync is enabled and the user has a contact_id.
    $config = $this->configFactory->get('virtuous.settings');
    if (!$config->get('two_way_sync_enabled') || $user->get('contact_id')->isEmpty()) {
      return FALSE;
    }

    // Map Drupal user fields to Virtuous Transaction API format.
    $contact_data = $this->mapUserToVirtuousTransaction($user);

    // Send the update to Virtuous.
    $result = $this->virtuousApiService->updateContactInVirtuous($contact_data);

    if ($result['success']) {
      $this->logger->notice(
            'Successfully updated contact @id in Virtuous for user @name.', [
              '@id' => $user->get('contact_id')->value,
              '@name' => $user->getAccountName(),
            ]
        );
      return TRUE;
    }
    else {
      $this->logger->error(
            'Failed to update contact @id in Virtuous: @error', [
              '@id' => $user->get('contact_id')->value,
              '@error' => $result['message'],
            ]
        );
      return FALSE;
    }
  }

  /**
   * Maps a Drupal user to Virtuous Transaction API format.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   *
   * @return array
   *   The mapped data for Virtuous Transaction API.
   */
  protected function mapUserToVirtuousTransaction(UserInterface $user): array {
    $data = [
      'referenceSource' => $user->get('contact_reference_source')->value,
      'referenceId' => $user->get('contact_reference_id')->value,
      'contactType' => $this->getTermName($user->get('contact_type')->target_id),
    ];
    
    // Check if name fields should be locked (not updated in Virtuous)
    $lock_name_settings = !$user->get('contact_lock_name_settings')->isEmpty() && 
                          $user->get('contact_lock_name_settings')->value;
    
    // Only add name fields if they're not locked
    if (!$lock_name_settings) {
      // Add name fields
      if (!$user->get('contact_default_name')->isEmpty()) {
        $data['name'] = $user->get('contact_default_name')->value;
      }
      if (!$user->get('contact_informal_name')->isEmpty()) {
        $data['informalName'] = $user->get('contact_informal_name')->value;
      }
      if (!$user->get('contact_formal_name')->isEmpty()) {
        $data['formalContactName'] = $user->get('contact_formal_name')->value;
      }
      if (!$user->get('contact_alternate_name')->isEmpty()) {
        $data['alternateContactName'] = $user->get('contact_alternate_name')->value;
      }
      if (!$user->get('contact_preferred_salutation')->isEmpty()) {
        $data['preferredSalutationName'] = $user->get('contact_preferred_salutation')->value;
      }
      if (!$user->get('contact_preferred_addressee')->isEmpty()) {
        $data['preferredAddresseeName'] = $user->get('contact_preferred_addressee')->value;
      }
    }

    // Add primary individual fields if present.
    if (!$user->get('primary_first_name')->isEmpty()) {
      $data['firstName'] = $user->get('primary_first_name')->value;
      $data['middleName'] = $user->get('primary_middle_name')->value;
      $data['lastName'] = $user->get('primary_last_name')->value;
      $data['prefix'] = $user->get('primary_prefix')->value;
      $data['suffix'] = $user->get('primary_suffix')->value;
    }

    // Add email if present.
    if (!$user->get('mail')->isEmpty()) {
      $data['email'] = $user->getEmail();
      $data['emailType'] = 'Primary';
    }

    // Add address if present.
    if (!$user->get('contact_address')->isEmpty()) {
      $address = $user->get('contact_address')->first();
      $data['address1'] = $address->address_line1;
      $data['address2'] = $address->address_line2;
      $data['city'] = $address->locality;
      $data['state'] = $address->administrative_area;
      $data['postal'] = $address->postal_code;
      $data['country'] = $address->country_code;
    }

    return $data;
  }

  /**
   * Gets the term name from a term ID.
   *
   * @param int|null $tid
   *   The taxonomy term ID.
   *
   * @return string|null
   *   The term name or NULL if not found.
   */
  protected function getTermName(?int $tid): ?string {
    if (!$tid) {
      return NULL;
    }

    $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($tid);
    return $term ? $term->getName() : NULL;
  }

}
