<?php

namespace Drupal\Tests\virtuous\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\key\Entity\Key;
use Drupal\Core\Url;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Middleware;

/**
 * Tests the Virtuous configuration form.
 *
 * @group virtuous
 */
class VirtuousConfigFormTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'taxonomy',
    'key',
    'virtuous',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The user to test with.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * The history container for the mocked client.
   *
   * @var array
   */
  protected $container = [];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create and log in an admin user.
    $this->adminUser = $this->drupalCreateUser(
          [
            'administer virtuous configuration',
            'administer keys',
          ]
      );
    $this->drupalLogin($this->adminUser);

    // Create a test key for the API.
    $key = Key::create(
          [
            'id' => 'virtuous_test_key',
            'label' => 'Virtuous Test Key',
            'key_type' => 'authentication',
            'key_provider' => 'config',
            'key_input' => 'text_field',
            'key_provider_settings' => [
              'key_value' => 'test-api-key',
            ],
          ]
      );
    $key->save();

    // Set up mock for the HTTP client.
    $mock = new MockHandler(
          [
            new Response(200, [], '{"success":true}'),
          ]
      );
    $handlerStack = HandlerStack::create($mock);

    // Add history middleware to the handler stack.
    $this->container = [];
    $history = Middleware::history($this->container);
    $handlerStack->push($history);

    // Create a mocked HTTP client.
    $client = new Client(['handler' => $handlerStack]);

    // Set the client in the container.
    $this->container->set('http_client', $client);
  }

  /**
   * Tests the configuration form display.
   */
  public function testConfigFormDisplay() {
    // Access the configuration form.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));

    // Assert the form fields are displayed.
    $this->assertSession()->fieldExists('api_mode');
    $this->assertSession()->fieldExists('api_key');
    $this->assertSession()->fieldExists('test_contact_id');
    $this->assertSession()->buttonExists('Test API Connection');
    $this->assertSession()->fieldExists('sync_frequency');
    $this->assertSession()->fieldExists('webhook_enabled');
    $this->assertSession()->fieldExists('two_way_sync_enabled');
    $this->assertSession()->buttonExists('Save configuration');
  }

  /**
   * Tests saving the configuration form.
   */
  public function testConfigFormSubmission() {
    // Set the form values.
    $edit = [
      'api_mode' => 'live',
      'api_key' => 'virtuous_test_key',
      'sync_frequency' => 'daily',
      'webhook_enabled' => 1,
      'webhook_token' => 'secret-webhook-token',
      'two_way_sync_enabled' => 1,
    ];

    // Submit the form.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Save configuration');

    // Assert the form was saved successfully.
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Load the configuration and verify it was saved correctly.
    $config = $this->config('virtuous.settings');
    $this->assertEquals('live', $config->get('api_mode'));
    $this->assertEquals('virtuous_test_key', $config->get('api_key'));
    $this->assertEquals('daily', $config->get('sync_frequency'));
    $this->assertEquals(TRUE, $config->get('webhook_enabled'));
    $this->assertEquals('secret-webhook-token', $config->get('webhook_token'));
    $this->assertEquals(TRUE, $config->get('two_way_sync_enabled'));
  }

  /**
   * Tests the webhook token validation.
   */
  public function testWebhookTokenValidation() {
    // Try to submit the form with webhook enabled but no token.
    $edit = [
      'api_mode' => 'test',
      'api_key' => 'virtuous_test_key',
      'webhook_enabled' => 1,
      'webhook_token' => '',
      'two_way_sync_enabled' => 0,
    ];

    // Submit the form.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Save configuration');

    // Assert the form validation error message.
    $this->assertSession()->pageTextContains('A security token is required when webhooks are enabled.');
  }

  /**
   * Tests the API connection test functionality.
   */
  public function testApiConnectionTest() {
    // Set the API key and contact ID.
    $edit = [
      'api_key' => 'virtuous_test_key',
      'test_contact_id' => '12345',
    ];

    // Submit the form using the test connection button.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Test API Connection');

    // Assert the success message.
    $this->assertSession()->pageTextContains('Successfully connected to Virtuous API.');
  }

  /**
   * Tests the API connection test with empty key.
   */
  public function testApiConnectionTestWithEmptyKey() {
    // Submit the form with no API key.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm([], 'Test API Connection');

    // Assert the error message.
    $this->assertSession()->pageTextContains('Please select an API key before testing the connection.');
  }

  /**
   * Tests the API connection test with empty contact ID.
   */
  public function testApiConnectionTestWithEmptyContactId() {
    // Submit the form with API key but no contact ID.
    $edit = [
      'api_key' => 'virtuous_test_key',
      'test_contact_id' => '',
    ];
    
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Test API Connection');

    // Assert the error message.
    $this->assertSession()->pageTextContains('Please enter a contact ID to test the connection.');
  }

  /**
   * Tests the webhook URLs display.
   */
  public function testWebhookUrlsDisplay() {
    // Enable webhooks to show the URLs.
    $edit = [
      'webhook_enabled' => 1,
    ];

    // Submit the form.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Save configuration');

    // Assert the webhook URLs are displayed.
    $this->assertSession()->fieldExists('create_url');
    $this->assertSession()->fieldExists('update_url');
    $this->assertSession()->fieldExists('delete_url');

    // Check the URL values.
    $baseUrl = $this->getAbsoluteUrl('');
    $this->assertSession()->fieldValueEquals('create_url', $baseUrl . '/virtuous/webhook/contact-created');
    $this->assertSession()->fieldValueEquals('update_url', $baseUrl . '/virtuous/webhook/contact-updated');
    $this->assertSession()->fieldValueEquals('delete_url', $baseUrl . '/virtuous/webhook/contact-deleted');
  }

  /**
   * Tests the API response display area.
   */
  public function testApiResponseDisplay() {
    // Set up a test with successful API response.
    $edit = [
      'api_key' => 'virtuous_test_key',
      'test_contact_id' => '12345',
    ];

    // Submit the form using the test connection button.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    $this->submitForm($edit, 'Test API Connection');

    // Assert the API response container is displayed.
    $this->assertSession()->elementExists('css', '.virtuous-api-response');
    $this->assertSession()->elementExists('css', '.virtuous-api-response__status');
    $this->assertSession()->elementExists('css', '.virtuous-api-response__message');
  }

  /**
   * Tests the two-way sync warning display.
   */
  public function testTwoWaySyncWarningDisplay() {
    // Visit the config form.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    
    // Initially the warning shouldn't be visible.
    $warning = $this->xpath('//div[contains(@class, "messages--warning") and contains(text(), "Two-way synchronization will send updates to Virtuous")]');
    $this->assertEmpty($warning);
    
    // Enable two-way sync.
    $edit = [
      'two_way_sync_enabled' => 1,
    ];
    
    // Submit the form.
    $this->submitForm($edit, 'Save configuration');
    
    // Visit the config form again.
    $this->drupalGet(Url::fromRoute('virtuous.config_form'));
    
    // The warning should now be visible.
    $warning = $this->xpath('//div[contains(@class, "messages--warning") and contains(text(), "Two-way synchronization will send updates to Virtuous")]');
    $this->assertNotEmpty($warning);
  }

}
