<?php

namespace Drupal\Tests\virtuous\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\virtuous\Service\VirtuousApiService;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Psr7\Request;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Tests the VirtuousApiService class.
 *
 * @group virtuous
 */
class VirtuousApiServiceTest extends KernelTestBase {

  use ProphecyTrait;

  /**
   * The modules to enable.
   *
   * @var array
   */
  protected static $modules = ['system', 'user', 'field', 'key', 'virtuous'];

  /**
   * The VirtuousApiService to test.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $apiService;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The key repository service.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The logger factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The mocked HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The mock handler.
   *
   * @var \GuzzleHttp\Handler\MockHandler
   */
  protected $mockHandler;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installConfig(['system', 'field', 'virtuous']);

    // Set up mock key repository.
    $key = $this->prophesize('Drupal\key\Entity\Key');
    $key->getKeyValue()->willReturn('test-api-key');
    $key->id()->willReturn('virtuous_api_key');

    $this->keyRepository = $this->prophesize('Drupal\key\KeyRepositoryInterface');
    $this->keyRepository->getKey('virtuous_api_key')->willReturn($key->reveal());

    // Set up mock config.
    $this->configFactory = $this->container->get('config.factory');
    $this->configFactory->getEditable('virtuous.settings')
      ->set('api_key', 'virtuous_api_key')
      ->set('api_mode', 'test')
      ->save();

    // Set up mock logger factory.
    $this->loggerFactory = $this->container->get('logger.factory');

    // Set up mock HTTP client with mock handler.
    $this->mockHandler = new MockHandler();
    $handlerStack = HandlerStack::create($this->mockHandler);
    $this->httpClient = new Client(['handler' => $handlerStack]);

    // Create the API service with mock dependencies.
    $this->apiService = new VirtuousApiService(
          $this->configFactory,
          $this->keyRepository->reveal(),
          $this->loggerFactory,
          $this->httpClient
      );
  }

  /**
   * Tests the testConnection method with successful response.
   */
  public function testTestConnectionSuccess() {
    // Mock a successful response.
    $this->mockHandler->append(new Response(200, [], '{"success":true}'));

    // Call the method.
    $result = $this->apiService->testConnection();

    // Assert the result.
    $this->assertTrue($result['success']);
    $this->assertEquals('Connection successful.', $result['message']);
  }

  /**
   * Tests the testConnection method with error response.
   */
  public function testTestConnectionError() {
    // Mock an error response.
    $this->mockHandler->append(new Response(403, [], '{"error":"Unauthorized"}'));

    // Call the method.
    $result = $this->apiService->testConnection();

    // Assert the result.
    $this->assertFalse($result['success']);
    $this->assertEquals('Unexpected status code: 403', $result['message']);
  }

  /**
   * Tests the testConnection method with exception.
   */
  public function testTestConnectionException() {
    // Mock a request exception.
    $request = new Request('GET', 'https://api.virtuoussoftware.com/api/v1/System/Ping');
    $this->mockHandler->append(new RequestException('Connection error', $request));

    // Call the method.
    $result = $this->apiService->testConnection();

    // Assert the result.
    $this->assertFalse($result['success']);
    $this->assertEquals('Connection error', $result['message']);
  }

  /**
   * Tests the getContactCount method with successful response.
   */
  public function testGetContactCountSuccess() {
    // Mock a successful response.
    $this->mockHandler->append(new Response(200, [], '{"count":42}'));

    // Call the method.
    $result = $this->apiService->getContactCount();

    // Assert the result.
    $this->assertEquals(42, $result);
  }

  /**
   * Tests the getContactCount method with error response.
   */
  public function testGetContactCountError() {
    // Mock an error response.
    $this->mockHandler->append(new Response(500, [], '{"error":"Server error"}'));

    // Call the method.
    $result = $this->apiService->getContactCount();

    // Assert the result.
    $this->assertNull($result);
  }

  /**
   * Tests the getContacts method with successful response.
   */
  public function testGetContactsSuccess() {
    // Mock a successful response with sample contacts including new fields.
    $contactsData = [
      [
        'id' => 1, 
        'name' => 'Contact 1',
        'formalContactName' => 'Formal Contact 1',
        'alternateContactName' => 'Alt Contact 1',
        'preferredSalutationName' => 'Dear Contact 1',
        'preferredAddresseeName' => 'Contact 1 Address',
        'nameFieldsLocked' => TRUE,
        'contactIndividuals' => [
          [
            'id' => 101,
            'firstName' => 'John',
            'nickname' => 'Johnny',
            'preMarriageName' => 'Smith',
            'prefix' => 'Mr.',
            'prefix2' => 'Dr.',
          ],
        ],
      ],
      [
        'id' => 2, 
        'name' => 'Contact 2',
        'formalContactName' => 'Formal Contact 2',
        'alternateContactName' => 'Alt Contact 2',
        'preferredSalutationName' => 'Dear Contact 2',
        'preferredAddresseeName' => 'Contact 2 Address',
        'nameFieldsLocked' => FALSE,
        'contactIndividuals' => [
          [
            'id' => 201,
            'firstName' => 'Jane',
            'nickname' => 'Janie',
            'preMarriageName' => 'Johnson',
            'prefix' => 'Mrs.',
            'prefix2' => 'Prof.',
          ],
        ],
      ],
    ];
    $this->mockHandler->append(new Response(200, [], json_encode($contactsData)));

    // Call the method.
    $result = $this->apiService->getContacts([], 1, 10);

    // Assert the result.
    $this->assertIsArray($result);
    $this->assertCount(2, $result);
    
    // Check first contact
    $this->assertEquals(1, $result[0]['id']);
    $this->assertEquals('Contact 1', $result[0]['name']);
    $this->assertEquals('Formal Contact 1', $result[0]['formalContactName']);
    $this->assertEquals('Alt Contact 1', $result[0]['alternateContactName']);
    $this->assertEquals('Dear Contact 1', $result[0]['preferredSalutationName']);
    $this->assertEquals('Contact 1 Address', $result[0]['preferredAddresseeName']);
    $this->assertEquals(TRUE, $result[0]['nameFieldsLocked']);
    
    // Check first contact's primary individual
    $this->assertEquals(101, $result[0]['contactIndividuals'][0]['id']);
    $this->assertEquals('John', $result[0]['contactIndividuals'][0]['firstName']);
    $this->assertEquals('Johnny', $result[0]['contactIndividuals'][0]['nickname']);
    $this->assertEquals('Smith', $result[0]['contactIndividuals'][0]['preMarriageName']);
    $this->assertEquals('Mr.', $result[0]['contactIndividuals'][0]['prefix']);
    $this->assertEquals('Dr.', $result[0]['contactIndividuals'][0]['prefix2']);
    
    // Check second contact
    $this->assertEquals(2, $result[1]['id']);
    $this->assertEquals('Contact 2', $result[1]['name']);
    $this->assertEquals('Formal Contact 2', $result[1]['formalContactName']);
    $this->assertEquals('Alt Contact 2', $result[1]['alternateContactName']);
    $this->assertEquals('Dear Contact 2', $result[1]['preferredSalutationName']);
    $this->assertEquals('Contact 2 Address', $result[1]['preferredAddresseeName']);
    $this->assertEquals(FALSE, $result[1]['nameFieldsLocked']);
    
    // Check second contact's primary individual
    $this->assertEquals(201, $result[1]['contactIndividuals'][0]['id']);
    $this->assertEquals('Jane', $result[1]['contactIndividuals'][0]['firstName']);
    $this->assertEquals('Janie', $result[1]['contactIndividuals'][0]['nickname']);
    $this->assertEquals('Johnson', $result[1]['contactIndividuals'][0]['preMarriageName']);
    $this->assertEquals('Mrs.', $result[1]['contactIndividuals'][0]['prefix']);
    $this->assertEquals('Prof.', $result[1]['contactIndividuals'][0]['prefix2']);
  }

  /**
   * Tests the getContact method with successful response.
   */
  public function testGetContactSuccess() {
    // Mock a successful response with a sample contact including new fields.
    $contactData = [
      'id' => 1, 
      'name' => 'Contact 1',
      'formalContactName' => 'Formal Contact 1',
      'alternateContactName' => 'Alt Contact 1',
      'preferredSalutationName' => 'Dear Contact 1',
      'preferredAddresseeName' => 'Contact 1 Address',
      'nameFieldsLocked' => TRUE,
      'contactIndividuals' => [
        [
          'id' => 101,
          'firstName' => 'John',
          'nickname' => 'Johnny',
          'preMarriageName' => 'Smith',
          'prefix' => 'Mr.',
          'prefix2' => 'Dr.',
          'isDeceased' => FALSE,
          'deceasedDate' => NULL,
        ],
      ],
    ];
    $this->mockHandler->append(new Response(200, [], json_encode($contactData)));

    // Call the method.
    $result = $this->apiService->getContact(1);

    // Assert the result.
    $this->assertIsArray($result);
    $this->assertEquals(1, $result['id']);
    $this->assertEquals('Contact 1', $result['name']);
    $this->assertEquals('Formal Contact 1', $result['formalContactName']);
    $this->assertEquals('Alt Contact 1', $result['alternateContactName']);
    $this->assertEquals('Dear Contact 1', $result['preferredSalutationName']);
    $this->assertEquals('Contact 1 Address', $result['preferredAddresseeName']);
    $this->assertEquals(TRUE, $result['nameFieldsLocked']);
    
    // Check primary individual
    $this->assertEquals(101, $result['contactIndividuals'][0]['id']);
    $this->assertEquals('John', $result['contactIndividuals'][0]['firstName']);
    $this->assertEquals('Johnny', $result['contactIndividuals'][0]['nickname']);
    $this->assertEquals('Smith', $result['contactIndividuals'][0]['preMarriageName']);
    $this->assertEquals('Mr.', $result['contactIndividuals'][0]['prefix']);
    $this->assertEquals('Dr.', $result['contactIndividuals'][0]['prefix2']);
    $this->assertEquals(FALSE, $result['contactIndividuals'][0]['isDeceased']);
    $this->assertNull($result['contactIndividuals'][0]['deceasedDate']);
  }

  /**
   * Tests the validateWebhookToken method.
   */
  public function testValidateWebhookToken() {
    // Set up configuration.
    $this->configFactory->getEditable('virtuous.settings')
      ->set('webhook_token', 'valid-token')
      ->save();

    // Test with valid token.
    $result = $this->apiService->validateWebhookToken('valid-token');
    $this->assertTrue($result);

    // Test with invalid token.
    $result = $this->apiService->validateWebhookToken('invalid-token');
    $this->assertFalse($result);

    // Test with empty token.
    $result = $this->apiService->validateWebhookToken('');
    $this->assertFalse($result);
  }

  /**
   * Tests the updateContactInVirtuous method with successful response.
   */
  public function testUpdateContactInVirtuousSuccess() {
    // Mock a successful response.
    $this->mockHandler->append(new Response(200, [], '{"id":1,"status":"updated"}'));

    // Call the method with expanded contact data including new fields.
    $contactData = [
      'referenceId' => 'USR-001', 
      'name' => 'Updated Name',
      'formalContactName' => 'Updated Formal Name',
      'alternateContactName' => 'Updated Alt Name',
      'preferredSalutationName' => 'Updated Salutation',
      'preferredAddresseeName' => 'Updated Addressee',
      'nameFieldsLocked' => TRUE,
      'firstName' => 'John',
      'middleName' => 'Robert',
      'lastName' => 'Doe',
      'nickname' => 'Johnny',
      'preMarriageName' => 'Smith',
      'prefix' => 'Mr.',
      'prefix2' => 'Dr.',
      'suffix' => 'Jr.',
    ];
    $result = $this->apiService->updateContactInVirtuous($contactData);

    // Assert the result.
    $this->assertTrue($result['success']);
    $this->assertEquals(['id' => 1, 'status' => 'updated'], $result['data']);
  }

  /**
   * Tests the updateContactInVirtuous method with error response.
   */
  public function testUpdateContactInVirtuousError() {
    // Mock an error response.
    $this->mockHandler->append(new Response(400, [], '{"error":"Invalid data"}'));

    // Call the method with expanded contact data including new fields.
    $contactData = [
      'referenceId' => 'USR-001', 
      'name' => 'Updated Name',
      'formalContactName' => 'Updated Formal Name',
      'alternateContactName' => 'Updated Alt Name',
      'preferredSalutationName' => 'Updated Salutation',
      'preferredAddresseeName' => 'Updated Addressee',
      'nameFieldsLocked' => TRUE,
      'firstName' => 'John',
      'middleName' => 'Robert',
      'lastName' => 'Doe',
      'nickname' => 'Johnny',
      'preMarriageName' => 'Smith',
      'prefix' => 'Mr.',
      'prefix2' => 'Dr.',
      'suffix' => 'Jr.',
    ];
    $result = $this->apiService->updateContactInVirtuous($contactData);

    // Assert the result.
    $this->assertFalse($result['success']);
    $this->assertEquals('Unexpected status code: 400', $result['message']);
  }

}
