<?php

namespace Drupal\Tests\virtuous\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\virtuous\Service\VirtuousTwoWaySyncService;
use Drupal\virtuous\Service\VirtuousApiService;
use Drupal\user\Entity\User;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Tests the VirtuousTwoWaySyncService class.
 *
 * @group virtuous
 */
class VirtuousTwoWaySyncServiceTest extends KernelTestBase {

  use ProphecyTrait;

  /**
   * The modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'taxonomy',
    'key',
    'virtuous',
  ];

  /**
   * The VirtuousTwoWaySyncService to test.
   *
   * @var \Drupal\virtuous\Service\VirtuousTwoWaySyncService
   */
  protected $twoWaySyncService;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The mocked Virtuous API service.
   *
   * @var \Drupal\virtuous\Service\VirtuousApiService
   */
  protected $virtuousApiService;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger factory service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('taxonomy_term');
    $this->installSchema('system', ['sequences']);
    $this->installSchema('user', ['users_data']);
    $this->installConfig(['system', 'field', 'virtuous']);

    // Mock the Virtuous API service.
    $this->virtuousApiService = $this->prophesize(VirtuousApiService::class);

    // Get the config factory.
    $this->configFactory = $this->container->get('config.factory');
    
    // Set up the two-way sync config.
    $this->configFactory->getEditable('virtuous.settings')
      ->set('two_way_sync_enabled', TRUE)
      ->save();

    // Get the entity type manager.
    $this->entityTypeManager = $this->container->get('entity_type.manager');

    // Get the logger factory.
    $this->loggerFactory = $this->container->get('logger.factory');

    // Create the two-way sync service with mocked API service.
    $this->twoWaySyncService = new VirtuousTwoWaySyncService(
      $this->virtuousApiService->reveal(),
      $this->configFactory,
      $this->loggerFactory,
      $this->entityTypeManager
    );

    // Create user fields for testing.
    $this->createUserFields();
  }

  /**
   * Creates the necessary fields on the user entity for testing.
   */
  protected function createUserFields() {
    // Define the fields we need to test.
    $fields = [
      'contact_id' => 'string',
      'contact_reference_source' => 'string',
      'contact_reference_id' => 'string',
      'contact_name' => 'string',
      'contact_formal_name' => 'string',
      'contact_alternate_name' => 'string',
      'contact_preferred_salutation' => 'string',
      'contact_preferred_addressee' => 'string',
      'contact_lock_name_settings' => 'boolean',
      'primary_first_name' => 'string',
      'primary_middle_name' => 'string',
      'primary_last_name' => 'string',
      'primary_prefix' => 'string',
      'primary_prefix_2' => 'string',
      'primary_suffix' => 'string',
      'primary_nickname' => 'string',
      'primary_premarriage_name' => 'string',
    ];

    foreach ($fields as $field_name => $field_type) {
      // Skip if the field already exists.
      if (field_storage_config_load('user.' . $field_name)) {
        continue;
      }

      // Create the field storage.
      $field_storage = \Drupal::entityTypeManager()->getStorage('field_storage_config')->create([
        'field_name' => $field_name,
        'entity_type' => 'user',
        'type' => $field_type,
        'cardinality' => 1,
      ]);
      $field_storage->save();

      // Create the field instance.
      $field = \Drupal::entityTypeManager()->getStorage('field_config')->create([
        'field_storage' => $field_storage,
        'bundle' => 'user',
        'label' => ucfirst(str_replace('_', ' ', $field_name)),
      ]);
      $field->save();
    }
  }

  /**
   * Tests the sendUserUpdatesToVirtuous method with a successful API response.
   */
  public function testSendUserUpdatesToVirtuousSuccess() {
    // Create a test user with Virtuous fields.
    $user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'contact_id' => '12345',
      'contact_reference_source' => 'Drupal',
      'contact_reference_id' => 'USR-001',
      'contact_name' => 'Test User',
      'contact_formal_name' => 'Mr. Test User',
      'contact_alternate_name' => 'Tester',
      'contact_preferred_salutation' => 'Dear Test',
      'contact_preferred_addressee' => 'Test User Address',
      'contact_lock_name_settings' => TRUE,
      'primary_first_name' => 'Test',
      'primary_middle_name' => 'Middle',
      'primary_last_name' => 'User',
      'primary_prefix' => 'Mr.',
      'primary_prefix_2' => 'Dr.',
      'primary_suffix' => 'Jr.',
      'primary_nickname' => 'Testy',
      'primary_premarriage_name' => 'Original',
    ]);
    $user->save();

    // Set up the API service mock to return success.
    $this->virtuousApiService->updateContactInVirtuous(\Prophecy\Argument::any())->willReturn([
      'success' => TRUE,
      'data' => ['id' => 12345, 'status' => 'updated'],
    ]);

    // Call the method being tested.
    $result = $this->twoWaySyncService->sendUserUpdatesToVirtuous($user);

    // Assert the result.
    $this->assertTrue($result);

    // Verify the API service was called with the correct data.
    $this->virtuousApiService->updateContactInVirtuous(\Prophecy\Argument::that(function ($data) {
      return
        $data['referenceSource'] === 'Drupal' &&
        $data['referenceId'] === 'USR-001' &&
        $data['name'] === 'Test User' &&
        $data['formalContactName'] === 'Mr. Test User' &&
        $data['alternateContactName'] === 'Tester' &&
        $data['preferredSalutationName'] === 'Dear Test' &&
        $data['preferredAddresseeName'] === 'Test User Address' &&
        $data['nameFieldsLocked'] === TRUE &&
        $data['firstName'] === 'Test' &&
        $data['middleName'] === 'Middle' &&
        $data['lastName'] === 'User' &&
        $data['prefix'] === 'Mr.' &&
        $data['prefix2'] === 'Dr.' &&
        $data['suffix'] === 'Jr.' &&
        $data['nickname'] === 'Testy' &&
        $data['preMarriageName'] === 'Original' &&
        $data['email'] === 'test@example.com';
    }))->shouldHaveBeenCalled();
  }

  /**
   * Tests the sendUserUpdatesToVirtuous method with a failed API response.
   */
  public function testSendUserUpdatesToVirtuousFailure() {
    // Create a test user with Virtuous fields.
    $user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'contact_id' => '12345',
      'contact_reference_source' => 'Drupal',
      'contact_reference_id' => 'USR-001',
      'contact_name' => 'Test User',
      'contact_formal_name' => 'Mr. Test User',
      'contact_alternate_name' => 'Tester',
      'contact_preferred_salutation' => 'Dear Test',
      'contact_preferred_addressee' => 'Test User Address',
      'contact_lock_name_settings' => TRUE,
      'primary_first_name' => 'Test',
      'primary_middle_name' => 'Middle',
      'primary_last_name' => 'User',
      'primary_prefix' => 'Mr.',
      'primary_prefix_2' => 'Dr.',
      'primary_suffix' => 'Jr.',
      'primary_nickname' => 'Testy',
      'primary_premarriage_name' => 'Original',
    ]);
    $user->save();

    // Set up the API service mock to return failure.
    $this->virtuousApiService->updateContactInVirtuous(\Prophecy\Argument::any())->willReturn([
      'success' => FALSE,
      'message' => 'API Error',
    ]);

    // Call the method being tested.
    $result = $this->twoWaySyncService->sendUserUpdatesToVirtuous($user);

    // Assert the result.
    $this->assertFalse($result);
  }

  /**
   * Tests the sendUserUpdatesToVirtuous method when two-way sync is disabled.
   */
  public function testSendUserUpdatesToVirtuousSyncDisabled() {
    // Disable two-way sync.
    $this->configFactory->getEditable('virtuous.settings')
      ->set('two_way_sync_enabled', FALSE)
      ->save();

    // Create a test user with Virtuous fields.
    $user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'contact_id' => '12345',
    ]);
    $user->save();

    // Call the method being tested.
    $result = $this->twoWaySyncService->sendUserUpdatesToVirtuous($user);

    // Assert the result.
    $this->assertFalse($result);

    // Verify the API service was not called.
    $this->virtuousApiService->updateContactInVirtuous(\Prophecy\Argument::any())->shouldNotHaveBeenCalled();
  }

  /**
   * Tests the sendUserUpdatesToVirtuous method with no contact ID.
   */
  public function testSendUserUpdatesToVirtuousNoContactId() {
    // Create a test user without a contact ID.
    $user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
    ]);
    $user->save();

    // Call the method being tested.
    $result = $this->twoWaySyncService->sendUserUpdatesToVirtuous($user);

    // Assert the result.
    $this->assertFalse($result);

    // Verify the API service was not called.
    $this->virtuousApiService->updateContactInVirtuous(\Prophecy\Argument::any())->shouldNotHaveBeenCalled();
  }

}
