<?php

namespace Drupal\Tests\virtuous\Unit;

use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal
 * @group              virtuous
 */
class VirtuousProcessContactDataTest extends UnitTestCase {

  /**
   * Tests the virtuous_process_contact_data() function.
   *
   * @covers ::virtuous_process_contact_data
   */
  public function testProcessContactData() {
    // Include the module file with the function we're testing.
    include_once __DIR__ . '/../../../virtuous.module';

    // Create sample contact data that mimics Virtuous API response.
    $contact = [
      'id' => 12345,
      'contactType' => 'Household',
      'referenceSource' => 'Drupal',
      'referenceId' => 'USR-001',
      'name' => 'Doe Family',
      'informalName' => 'Doe Family',
      'description' => 'Regular donors',
      'website' => 'https://example.com',
      'maritalStatus' => 'Married',
      'anniversaryMonth' => 6,
      'anniversaryDay' => 15,
      'anniversaryYear' => 2000,
      'originSegmentId' => 'SEG-001',
      'isPrivate' => FALSE,
      'isArchived' => FALSE,
      // New contact fields
      'formalContactName' => 'Mr. & Mrs. John Doe',
      'alternateContactName' => 'The Doe Family',
      'preferredSalutationName' => 'John & Jane',
      'preferredAddresseeName' => 'John and Jane Doe',
      'nameFieldsLocked' => TRUE,
      'mergedIntoContactId' => NULL,
      'contactAddresses' => [
        [
          'id' => 9876,
          'label' => 'Home',
          'address1' => '123 Main St',
          'address2' => 'Apt 4B',
          'city' => 'Springfield',
          'state' => 'IL',
          'postal' => '62701',
          'country' => 'US',
          'isPrimary' => TRUE,
          'isArchived' => FALSE
        ],
      ],
      'contactIndividuals' => [
        [
          'id' => 101,
          'isPrimary' => TRUE,
          'isSecondary' => FALSE,
          'firstName' => 'John',
          'middleName' => 'Robert',
          'lastName' => 'Doe',
          'prefix' => 'Mr.',
          'prefix2' => 'Dr.',
          'suffix' => 'Jr.',
          'birthMonth' => 3,
          'birthDay' => 21,
          'birthYear' => 1975,
          'approximateAge' => 48,
          'gender' => 'Male',
          'passion' => 'Education',
          'canBePrimary' => TRUE,
          'canBeSecondary' => TRUE,
          'isDeceased' => FALSE,
          'deceasedDate' => NULL,
          'nickname' => 'Johnny',
          'preMarriageName' => '',
          'contactMethods' => [
            [
              'id' => 501,
              'type' => 'Home Email',
              'value' => 'john.doe@example.com',
              'isPrimary' => TRUE,
            ],
            [
              'id' => 502,
              'type' => 'Mobile Phone',
              'value' => '555-123-4567',
              'isPrimary' => TRUE,
            ],
          ],
        ],
        [
          'id' => 102,
          'isPrimary' => FALSE,
          'isSecondary' => TRUE,
          'firstName' => 'Jane',
          'middleName' => 'Ellen',
          'lastName' => 'Doe',
          'prefix' => 'Mrs.',
          'prefix2' => 'Prof.',
          'suffix' => '',
          'birthMonth' => 5,
          'birthDay' => 15,
          'birthYear' => 1978,
          'approximateAge' => 45,
          'gender' => 'Female',
          'passion' => 'Healthcare',
          'canBePrimary' => TRUE,
          'canBeSecondary' => TRUE,
          'isDeceased' => FALSE,
          'deceasedDate' => NULL,
          'nickname' => 'Janie',
          'preMarriageName' => 'Smith',
          'contactMethods' => [
            [
              'id' => 503,
              'type' => 'Work Email',
              'value' => 'jane.doe@example.com',
              'isPrimary' => TRUE,
            ],
            [
              'id' => 504,
              'type' => 'Home Phone',
              'value' => '555-987-6543',
              'isPrimary' => TRUE,
            ],
          ],
        ],
      ],
    ];

    // Process the contact data.
    $processed_data = virtuous_process_contact_data($contact);

    // Assert basic contact fields.
    $this->assertEquals(12345, $processed_data['contact_id']);
    $this->assertEquals('Household', $processed_data['contact_type']);
    $this->assertEquals('Drupal', $processed_data['contact_reference_source']);
    $this->assertEquals('USR-001', $processed_data['contact_reference_id']);
    $this->assertEquals('Doe Family', $processed_data['contact_default_name']);
    $this->assertEquals('Doe Family', $processed_data['contact_informal_name']);
    $this->assertEquals('Regular donors', $processed_data['contact_description']);
    $this->assertEquals('https://example.com', $processed_data['contact_website']);
    $this->assertEquals('Married', $processed_data['contact_marital_status']);
    $this->assertEquals(6, $processed_data['contact_anniversary_month']);
    $this->assertEquals(15, $processed_data['contact_anniversary_day']);
    $this->assertEquals(2000, $processed_data['contact_anniversary_year']);
    $this->assertEquals('SEG-001', $processed_data['contact_origin_segment_id']);
    $this->assertEquals(FALSE, $processed_data['contact_is_private']);
    $this->assertEquals(FALSE, $processed_data['contact_is_archived']);
    
    // Assert new contact fields
    $this->assertEquals('Mr. & Mrs. John Doe', $processed_data['contact_formal_name']);
    $this->assertEquals('The Doe Family', $processed_data['contact_alternate_name']);
    $this->assertEquals('John & Jane', $processed_data['contact_preferred_salutation']);
    $this->assertEquals('John and Jane Doe', $processed_data['contact_preferred_addressee']);
    $this->assertEquals(TRUE, $processed_data['contact_lock_name_settings']);
    $this->assertEquals(NULL, $processed_data['contact_merged_into']);

    // Assert address data - updated for individual address fields.
    $this->assertEquals(9876, $processed_data['contact_address_id']);
    $this->assertEquals('Home', $processed_data['contact_address_label']);
    $this->assertEquals('123 Main St', $processed_data['contact_address_line_1']);
    $this->assertEquals('Apt 4B', $processed_data['contact_address_line_2']);
    $this->assertEquals('Springfield', $processed_data['contact_address_city']);
    $this->assertEquals('IL', $processed_data['contact_address_state']);
    $this->assertEquals('62701', $processed_data['contact_address_postal']);
    $this->assertEquals('US', $processed_data['contact_address_country']);
    $this->assertEquals(TRUE, $processed_data['contact_address_is_primary']);

    // Assert primary individual fields.
    $this->assertEquals(101, $processed_data['primary_id']);
    $this->assertEquals(12345, $processed_data['primary_contact_id']);
    $this->assertEquals('Mr.', $processed_data['primary_prefix_1']);
    $this->assertEquals('Dr.', $processed_data['primary_prefix_2']);
    $this->assertEquals('John', $processed_data['primary_first_name']);
    $this->assertEquals('Robert', $processed_data['primary_middle_name']);
    $this->assertEquals('Doe', $processed_data['primary_last_name']);
    $this->assertEquals('Jr.', $processed_data['primary_suffix']);
    $this->assertEquals('Johnny', $processed_data['primary_nickname']);
    $this->assertEquals('', $processed_data['primary_premarriage_name']);
    $this->assertEquals(3, $processed_data['primary_birth_month']);
    $this->assertEquals(21, $processed_data['primary_birth_day']);
    $this->assertEquals(1975, $processed_data['primary_birth_year']);
    $this->assertEquals(48, $processed_data['primary_age']);
    $this->assertEquals('Male', $processed_data['primary_gender']);
    $this->assertEquals('Education', $processed_data['primary_passion']);
    $this->assertEquals(TRUE, $processed_data['primary_is_primary']);
    $this->assertEquals(TRUE, $processed_data['primary_can_be_primary']);
    $this->assertEquals(FALSE, $processed_data['primary_is_secondary']);
    $this->assertEquals(TRUE, $processed_data['primary_can_be_secondary']);
    $this->assertEquals(FALSE, $processed_data['primary_is_deceased']);
    $this->assertEquals(NULL, $processed_data['primary_deceased_date']);
    $this->assertEquals('john.doe@example.com', $processed_data['primary_email_value']);
    $this->assertEquals('Home Email', $processed_data['primary_email_type']);
    $this->assertEquals(TRUE, $processed_data['primary_email_is_primary']);
    $this->assertEquals(501, $processed_data['primary_email_id']);
    $this->assertEquals('john.doe@example.com', $processed_data['mail']);
    $this->assertEquals('555-123-4567', $processed_data['primary_phone_value']);
    $this->assertEquals('Mobile Phone', $processed_data['primary_phone_type']);
    $this->assertEquals(TRUE, $processed_data['primary_phone_is_primary']);
    $this->assertEquals(502, $processed_data['primary_phone_id']);

    // Assert secondary individual fields.
    $this->assertEquals(102, $processed_data['secondary_id']);
    $this->assertEquals(12345, $processed_data['secondary_contact_id']);
    $this->assertEquals('Mrs.', $processed_data['secondary_prefix_1']);
    $this->assertEquals('Prof.', $processed_data['secondary_prefix_2']);
    $this->assertEquals('Jane', $processed_data['secondary_first_name']);
    $this->assertEquals('Ellen', $processed_data['secondary_middle_name']);
    $this->assertEquals('Doe', $processed_data['secondary_last_name']);
    $this->assertEquals('', $processed_data['secondary_suffix']);
    $this->assertEquals('Janie', $processed_data['secondary_nickname']);
    $this->assertEquals('Smith', $processed_data['secondary_premarriage_name']);
    $this->assertEquals(5, $processed_data['secondary_birth_month']);
    $this->assertEquals(15, $processed_data['secondary_birth_day']);
    $this->assertEquals(1978, $processed_data['secondary_birth_year']);
    $this->assertEquals(45, $processed_data['secondary_age']);
    $this->assertEquals('Female', $processed_data['secondary_gender']);
    $this->assertEquals('Healthcare', $processed_data['secondary_passion']);
    $this->assertEquals(FALSE, $processed_data['secondary_is_primary']);
    $this->assertEquals(TRUE, $processed_data['secondary_can_be_primary']);
    $this->assertEquals(TRUE, $processed_data['secondary_is_secondary']);
    $this->assertEquals(TRUE, $processed_data['secondary_can_be_secondary']);
    $this->assertEquals(FALSE, $processed_data['secondary_is_deceased']);
    $this->assertEquals(NULL, $processed_data['secondary_deceased_date']);
    $this->assertEquals('jane.doe@example.com', $processed_data['secondary_email_value']);
    $this->assertEquals('Work Email', $processed_data['secondary_email_type']);
    $this->assertEquals(TRUE, $processed_data['secondary_email_is_primary']);
    $this->assertEquals(503, $processed_data['secondary_email_id']);
    $this->assertEquals('555-987-6543', $processed_data['secondary_phone_value']);
    $this->assertEquals('Home Phone', $processed_data['secondary_phone_type']);
    $this->assertEquals(TRUE, $processed_data['secondary_phone_is_primary']);
    $this->assertEquals(504, $processed_data['secondary_phone_id']);
  }

  /**
   * Tests the virtuous_process_contact_data() function with minimal data.
   *
   * @covers ::virtuous_process_contact_data
   */
  public function testProcessMinimalContactData() {
    // Include the module file with the function we're testing.
    include_once __DIR__ . '/../../../virtuous.module';

    // Create minimal sample contact data.
    $contact = [
      'id' => 12345,
      'name' => 'Test Contact',
    ];

    // Process the contact data.
    $processed_data = virtuous_process_contact_data($contact);

    // Assert only required fields are set, others are NULL.
    $this->assertEquals(12345, $processed_data['contact_id']);
    $this->assertEquals('Test Contact', $processed_data['contact_default_name']);
    $this->assertNull($processed_data['contact_type']);
    $this->assertNull($processed_data['contact_reference_source']);
    $this->assertFalse(isset($processed_data['contact_address_id']));
    $this->assertFalse(isset($processed_data['contact_address_line_1']));
    $this->assertFalse(isset($processed_data['primary_first_name']));
    $this->assertFalse(isset($processed_data['secondary_first_name']));
    
    // Assert new fields are not set
    $this->assertFalse(isset($processed_data['contact_formal_name']));
    $this->assertFalse(isset($processed_data['contact_alternate_name']));
    $this->assertFalse(isset($processed_data['contact_preferred_salutation']));
    $this->assertFalse(isset($processed_data['contact_preferred_addressee']));
    $this->assertFalse(isset($processed_data['contact_lock_name_settings']));
    $this->assertFalse(isset($processed_data['contact_merged_into']));
    $this->assertFalse(isset($processed_data['primary_nickname']));
    $this->assertFalse(isset($processed_data['primary_premarriage_name']));
    $this->assertFalse(isset($processed_data['primary_deceased_date']));
    $this->assertFalse(isset($processed_data['secondary_nickname']));
    $this->assertFalse(isset($processed_data['secondary_premarriage_name']));
    $this->assertFalse(isset($processed_data['secondary_deceased_date']));
  }

}
